@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -12,6 +12,8 @@
 #ifndef COMPILER_H_INCLUDED
 #define COMPILER_H_INCLUDED
 
+#include "mdq.h"
+
 /* Values for oprclass - Changes made here need to be reflected in cdbg_dump opr_type_names table */
 typedef enum
 {
@@ -88,7 +90,7 @@ typedef struct	mliteralstruct
 		struct	mliteralstruct	*fl,
 					*bl;
 	}			que;
-	INTPTR_T       		rt_addr;
+	INTPTR_T		rt_addr;
 	int			reference_count;	/* Used in the hash table to track references */
 	mval			v;
 } mliteral;
@@ -157,6 +159,17 @@ typedef struct
 	uint4		octype;
 } octabstruct;
 
+typedef union mtreenode_union
+{
+	struct mtreelinks_struct
+	{
+		union mtreenode_union	*lson,
+					*rson;
+	}		links;
+	mvar		var;
+	mlabel		lab;
+} mtreenode;
+
 /* Values for octype */
 #define OCT_NULL	0
 #define OCT_MVAL	1
@@ -172,7 +185,9 @@ typedef struct
 #define OCT_ARITH	512
 #define OCT_UNARY	1024
 #define OCT_NEGATED	2048
-#define OCT_REL	4096
+#define OCT_REL 	4096
+/* For booleans which have a side-effect somewhere in their (or a direct ancestor's) second operand's triple chain */
+#define OCT_SE		8192
 
 typedef struct
 {
@@ -348,7 +363,7 @@ typedef struct
 	TREF(expr_start_orig) = NULL;						\
 	TREF(shift_side_effects) = FALSE;					\
 	TREF(saw_side_effect) = FALSE;						\
-	dqinit(&(SS)->tmpchain, exorder);					\
+	exorder_init(&(SS)->tmpchain);						\
 	OLDCHAIN = setcurtchain(&(SS)->tmpchain);				\
 }
 
@@ -447,8 +462,10 @@ typedef struct
 	boolean_t	PROTECT_LVN, SE_NOTIFY;											\
 	triple 		*REF2;													\
 																\
-	if (PROTECT_LVN = (TREF(side_effect_base))[TREF(expr_depth)])	/* NOTE assignment */					\
+	if ((PROTECT_LVN = (TREF(side_effect_base))[TREF(expr_depth)]))	/* NOTE assignment */					\
 		SE_NOTIFY = SE_WARN_ON;												\
+	else															\
+		SE_NOTIFY = FALSE;												\
 	while (SB2 < SB1)													\
 	{															\
 		if (PROTECT_LVN && (SB2 > (SUBSCRIPTS + XTRA)) && ((SB1 - SB2) > 1)						\
@@ -527,7 +544,7 @@ typedef struct
  */
 #define CLEAR_MVAL_BITS(mvalptr) 			\
 {							\
-	((mval_b *)(mvalptr))->sgne = 0;		\
+	((mval_gen *)(mvalptr))->byte.sgne = 0;		\
 	(mvalptr)->fnpc_indx = 0xff;			\
 	UTF8_ONLY((mvalptr)->utfcgr_indx = 0xff);	\
 }
@@ -571,6 +588,7 @@ typedef struct
 	char		*lexical_ptr;
 	char		window_token;
 	triple		pos_in_chain;
+	boolean_t	rts_error_in_parse;
 } parse_save_block;
 
 #define SAVE_PARSE_STATE(SAVE_PARSE_PTR)									\
@@ -586,6 +604,7 @@ MBSTART {													\
 	SAVE_PARSE_PTR->source_len = (TREF(source_buffer)).len;							\
 	SAVE_PARSE_PTR->window_token = TREF(window_token);							\
 	SAVE_PARSE_PTR->pos_in_chain = TREF(pos_in_chain);							\
+	SAVE_PARSE_PTR->rts_error_in_parse = TREF(rts_error_in_parse);						\
 } MBEND
 
 #define RESTORE_PARSE_STATE(SAVE_PARSE_PTR)								\
@@ -602,6 +621,7 @@ MBSTART {													\
 	TREF(source_error_found) = SAVE_PARSE_PTR->source_error_found;						\
 	TREF(window_token) = SAVE_PARSE_PTR->window_token;							\
 	TREF(pos_in_chain) = SAVE_PARSE_PTR->pos_in_chain;							\
+	TREF(rts_error_in_parse) = SAVE_PARSE_PTR->rts_error_in_parse;						\
 } MBEND
 
 #define RETURN_IF_RTS_ERROR							\
@@ -636,11 +656,27 @@ MBSTART {									\
 # define CDREF_REF CDLT_REF
 #endif
 
+typedef int (*indir_fptr_void_t)(void);
+typedef int (*indir_fptr_opr_t)(oprtype *, opctype);
+
+#if defined(DEBUG) && (defined(__clang__) || defined(__GNUC__))
+#define DO_INDIR_FUNCTION_CHECK
+GBLREF unsigned int indir_fntype[];
+
+#define ASSERT_INDIR_FUNCTION_VOID(ARGCODE) assert(1 == indir_fntype[ARGCODE])
+#define ASSERT_INDIR_FUNCTION_OPR(ARGCODE) assert(2 == indir_fntype[ARGCODE])
+#else
+#define ASSERT_INDIR_FUNCTION_VOID(ARGCODE) (1)
+#define ASSERT_INDIR_FUNCTION_OPR(ARGCODE) (1)
+#endif
+
 int		actuallist(oprtype *opr);
 int		bool_expr(boolean_t sense, oprtype *addr);
 void		bx_boollit(triple *t);
 void		bx_boolop(triple *t, boolean_t jmp_type_one, boolean_t jmp_to_next, boolean_t sense, oprtype *addr);
 void		bx_relop(triple *t, opctype cmp, opctype tst, oprtype *addr);
+void		bx_sboolop(triple *t, boolean_t jmp_type_one, boolean_t jmp_to_next, boolean_t sense, oprtype *addr);
+triple		*bx_startbool(triple *t);
 void		bx_tail(triple *t, boolean_t sense, oprtype *addr);
 void		chktchain(triple *head);
 void		code_gen(void);
@@ -656,7 +692,8 @@ int		expratom(oprtype *a);
 int		exfunc(oprtype *a, boolean_t alias_target);
 int		expritem(oprtype *a);
 int		expr(oprtype *a, int m_type);
-void		ex_tail(oprtype *opr);
+void		ex_arithlit(triple *t);
+void		ex_tail(oprtype *opr, boolean_t is_boolchild, boolean_t parent_comval);
 int		extern_func(oprtype *a);
 int		f_ascii(oprtype *a, opctype op);
 int		f_char(oprtype *a, opctype op);
@@ -774,8 +811,14 @@ void		start_for_fetches(void);
 void		tnxtarg(oprtype *a);
 void		tripinit(void);
 boolean_t	unuse_literal(mval *x);
-void		walktree(mvar *n,void (*f)(),char *arg);
+void		walktree(mtreenode *node, void (*f)(mtreenode *, void *), void *arg);
 void		wrtcatopt(triple *r, triple ***lpx, triple **lptop);
 int		zlcompile(unsigned char len, unsigned char *addr);		/***type int added***/
 
+static inline void exorder_init(triple *chain)
+{
+	chain->opcode = OCQ_INVALID;
+	dqinit(chain, exorder);
+}
+
 #endif /* COMPILER_H_INCLUDED */
