@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -129,8 +129,6 @@ void db_auto_upgrade(gd_region *reg)
 				csd->statsdb_allocation = STATSDB_ALLOCATION;
 				csd->offset = 0;
 				csd->max_rec = 0;
-				csd->i_reserved_bytes = 0;
-				csd->db_got_to_V7_once = TRUE;
 				csd->last_start_backup = (gtm_timet) 0; /* GTM-8681, but this is default value anyway */
 				/*the next four lines are because gvcst_init_sysops might have used an outdated assumption */
 				csd->max_update_array_size = csd->max_non_bm_update_array_size
@@ -138,11 +136,20 @@ void db_auto_upgrade(gd_region *reg)
 				csd->max_update_array_size
 					+= (int4)(ROUND_UP2(MAX_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE));
 			case GDSMV70001:
-				/* GT.M V70002 added proactive block split option */
-				csd->problksplit = DEFAULT_PROBLKSPLIT;
-				break;		/* so a new "case" needs to be added BEFORE the assert. */
 			case GDSMV70002:
-				/* Nothing to do for this version since it is GDSMVCURR for now. */
+				/* GT.M V70002 added proactive block split option */
+				/* GT.M V71001 changed proactive block split default */
+				csd->problksplit = DEFAULT_PROBLKSPLIT;
+			case GDSMV71001:
+				/* GT.M V7.1-002 implemented index reserved bytes and changed default to zero */
+				csd->i_reserved_bytes = 0;
+				break; /* so a new "case" needs to be added BEFORE the assert. */
+			case GDSMV71002:
+				/* GT.M V7.1-003 changed freeze on fail default for statsDBs to FALSE */
+				if (IS_STATSDB_REGNAME(reg))
+					csd->freeze_on_fail = FALSE;
+				break;
+			case GDSMV71003:
 				assert(FALSE);
 				break;
 			default:
@@ -322,7 +329,12 @@ void v6_db_auto_upgrade(gd_region *reg)
 				udi = FILE_INFO(reg);
 				new_eof = (off_t)BLK_ZERO_OFF(csd->start_vbn) + (off_t)csd->trans_hist.total_blks * csd->blk_size;
 				DEBUG_ONLY(file_size = gds_file_size(reg->dyn.addr->file_cntl);)
-				assert((file_size * DISK_BLOCK_SIZE) == (new_eof + DISK_BLOCK_SIZE));
+				 /* About the assert below for the new_eof. It does not work with block sizes under 4096 which
+				  * became the default in V63001. Tests that randomize the block size below 4096 will fail the
+				  * assert.
+				  */
+				assert(((file_size * DISK_BLOCK_SIZE) == (new_eof + DISK_BLOCK_SIZE))
+						|| ((DISK_BLOCK_SIZE << 3) > csd->blk_size));
 				db_write_eof_block(udi, udi->fd, csd->blk_size, new_eof, &TREF(dio_buff));
 				/* GT.M V63001 introduced reservedDBFlags */
 				csd->reservedDBFlags = 0; /* RDBF_AUTODB = FALSE, RDBF_NOSTATS = FALSE, RDBF_STATSDB = FALSE */
@@ -349,17 +361,20 @@ void v6_db_auto_upgrade(gd_region *reg)
 				/* we do not initialize new post GDSMV63014 field csd->last_start_backup here because if
 				   it has never been set the default (all zeroes) is OK, and if it has, we do not want to
 				   overwrite it on our way up */
-				csd->db_got_to_V7_once = FALSE;
 				csd->offset = 0;
 				csd->max_rec = 0;
-				csd->i_reserved_bytes = 0;
 				/*the next four lines are because gvcst_init_sysops might have used an outdated assumption */
 				csd->max_update_array_size = csd->max_non_bm_update_array_size
 				= (int4)(ROUND_UP2(MAX_NON_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE));
 				csd->max_update_array_size
 				+= (int4)(ROUND_UP2(MAX_BITMAP_UPDATE_ARRAY_SIZE(csd), UPDATE_ARRAY_ALIGN_SIZE));
 				/* GT.M V70002 added proactive block split option */
-				csd->problksplit = DEFAULT_PROBLKSPLIT;
+				if (csd->last_mdb_ver < GDSMV71001)
+					csd->problksplit = DEFAULT_PROBLKSPLIT;
+				if (csd->last_mdb_ver < GDSMV71002)
+					csd->i_reserved_bytes = 0;
+				if ((csd->last_mdb_ver < GDSMV71003) && IS_STATSDB_REGNAME(reg))
+					csd->freeze_on_fail = FALSE;
 				break;
 			case GDSMV63015:
 				assert(FALSE);	/* if this should come to pass, add appropriate code above the assert */
