@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -48,6 +48,7 @@
 #include "db_snapshot.h"
 
 #define NEXT_EPOCH_TIME_SPACES	"                   " /* 19 spaces, we have 19 character field width to output Next Epoch Time */
+#define NEVER_STR "                   Never"
 
 GBLREF	sgmnt_addrs	*cs_addrs;
 GBLREF	gd_region	*gv_cur_region;
@@ -81,7 +82,8 @@ LITREF	char		*gtm_dbversion_table[];
 
 #define	NEED_TO_DUMP(string)													\
 	(is_dse_all ? (CLI_PRESENT == cli_present("ALL"))									\
-		: (CLI_PRESENT == cli_present(string) || CLI_PRESENT == cli_present("ALL") && CLI_NEGATED != cli_present(string)))
+		: ((CLI_PRESENT == cli_present(string))										\
+			|| ((CLI_PRESENT == cli_present("ALL")) && (CLI_NEGATED != cli_present(string)))))
 
 void dse_dmp_fhead (void)
 {
@@ -129,7 +131,7 @@ void dse_dmp_fhead (void)
 		is_mm = (csd->acc_meth == dba_mm);
 		util_out_print("  Access method                                  !AD", FALSE, 2, is_mm ? "MM" : "BG");
 		util_out_print("  Global Buffers                !12UL", TRUE, csd->n_bts);
-		util_out_print("  Reserved Bytes                !19UL", FALSE, csd->reserved_bytes);
+		util_out_print("                                                   ", FALSE);
 		util_out_print("  Block size (in bytes)         !12UL", TRUE, csd->blk_size);
 		util_out_print("  Maximum record size           !19UL", FALSE, csd->max_rec_size);
 		util_out_print("  Starting VBN                  !12UL", TRUE, csd->start_vbn);
@@ -173,13 +175,13 @@ void dse_dmp_fhead (void)
 		util_out_print("  No. of writes/flush           !12UL", TRUE, csd->n_wrt_per_flu);
 		util_out_print("  Maximum TN                     0x!16@XQ", FALSE, &csd->max_tn);
 		if (GDSVLAST > csd->certified_for_upgrade_to)
-			util_out_print("  Certified for Upgrade to                !AD", TRUE,
+			util_out_print("  Certified for Upgrade to               !R3AD", TRUE,
 				LEN_AND_STR(gtm_dbversion_table[csd->certified_for_upgrade_to]));
 		else	/* out of range so print hex */
 			util_out_print("  Certified for Upgrade to         0x!XL", TRUE, csd->certified_for_upgrade_to);
 		util_out_print("  Maximum TN Warn                0x!16@XQ", FALSE, &csd->max_tn_warn);
 		if (GDSVLAST > csd->desired_db_format)
-			util_out_print("  Desired DB Format                       !AD", TRUE,
+			util_out_print("  Desired DB Format                      !R3AD", TRUE,
 				       LEN_AND_STR(gtm_dbversion_table[csd->desired_db_format]));
 		else	/* out of range so print hex */
 			util_out_print("  Desired DB Format               0x!XL", TRUE, csd->desired_db_format);
@@ -254,13 +256,15 @@ void dse_dmp_fhead (void)
 		util_out_print("  Full Block Write                         !UL", TRUE, csd->write_fullblk);
 		if(db_shares_gvstats)
 			util_out_print("  StatsDB Allocation            !19UL", TRUE, csd->statsdb_allocation);
+		util_out_print("  Data Reserved Bytes           !19UL", FALSE, csd->reserved_bytes);
+		util_out_print("  Index Reserved Bytes          !12UL", TRUE, csd->i_reserved_bytes);
 	}
 	if (CLI_PRESENT == cli_present("ALL"))
 	{	/* Only dump these if -/ALL as if part of above display */
 
 		if (!csd->last_start_backup)		/* Handle backup timestamp: 0 == Never backed up */
 		{
-			strncpy(backup_start_buf, "                   Never", sizeof(backup_start_buf));
+			memcpy(backup_start_buf, NEVER_STR, sizeof(NEVER_STR));
 		} else
 		{
 			backup_start_tmp = (time_t) (csd->last_start_backup);	/* this will 32 squash on AIX */
@@ -275,7 +279,7 @@ void dse_dmp_fhead (void)
 		util_out_print("  Blks Last Record Backup        0x!16@XQ", FALSE, &(csd->last_rec_bkup_last_blk));
 		util_out_print("  Last GT.M Minor Version               !4UL", TRUE, csd->last_mdb_ver);
 		util_out_print("  Blks Last Stream Backup        0x!16@XQ", FALSE, &(csd->last_inc_bkup_last_blk));
-		util_out_print("  DB Creation Version                     !AD", TRUE,
+		util_out_print("  DB Creation Version                    !R3AD", TRUE,
 				LEN_AND_STR(gtm_dbversion_table[csd->creation_db_ver]));
 		util_out_print("  Blks Last Comprehensive Backup 0x!16@XQ", FALSE, &(csd->last_com_bkup_last_blk));
 		util_out_print("  DB Creation Minor Version             !4UL", TRUE, csd->creation_mdb_ver);
@@ -528,7 +532,7 @@ void dse_dmp_fhead (void)
 		util_out_print(0, TRUE);
 		util_out_print("  Database is Fully Upgraded                : !AD",
 			TRUE, 5, (csd->fully_upgraded ? " TRUE" : "FALSE"));
-		util_out_print("  Database WAS ONCE Fully Upgraded from V4  : !AD",
+		util_out_print("  Database WAS ONCE Fully Upgraded          : !AD",
 			TRUE, 5, (csd->db_got_to_v5_once ? " TRUE" : "FALSE"));
 		util_out_print("  Blocks to Upgrade subzero(negative) error : 0x!16@XQ", TRUE, &(csd->blks_to_upgrd_subzero_error));
 		util_out_print("  TN when Blocks to Upgrade last became 0   : 0x!16@XQ", TRUE, &csd->tn_upgrd_blks_0);
