@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -27,6 +27,7 @@
 #include "min_max.h"
 #include "util.h"
 #include "dse.h"
+#include "filestruct.h"
 
 /* Include prototypes */
 #include "t_qread.h"
@@ -36,6 +37,7 @@ GBLREF boolean_t	patch_find_root_search;
 GBLREF gd_region	*gv_cur_region;
 GBLREF short int	patch_path_count;
 GBLREF sgmnt_addrs	*cs_addrs;
+GBLREF sgmnt_data_ptr_t	cs_data;
 GBLREF VSIG_ATOMIC_T	util_interrupt;
 
 error_def(ERR_CTRLC);
@@ -45,7 +47,8 @@ error_def(ERR_DSEINVALBLKID);
 void dse_range(void)
 {
 	block_id	from, to, blk, blk_child;
-	boolean_t	busy_matters, free, got_lonely_star, index, lost, low, star, up, was_crit, was_hold_onto_crit, long_blk_id;
+	boolean_t	busy_matters, free, got_lonely_star, index, lost, low, star, up, was_crit, was_hold_onto_crit = FALSE,
+			long_blk_id;
 	cache_rec_ptr_t	dummy_cr;
 	char		level, lower[MAX_KEY_SZ + 1], targ_key[MAX_KEY_SZ + 1], upper[MAX_KEY_SZ + 1];
 	int		cnt, dummy, lower_len, upper_len;
@@ -54,6 +57,7 @@ void dse_range(void)
 	short int	rsize, size, size1;
 	sm_uc_ptr_t	bp, b_top, key_bot, key_top, key_top1, rp, r_top;
 
+	DSE_DB_IS_TOO_OLD(cs_addrs, cs_data, gv_cur_region);
 	if (CLI_PRESENT == cli_present("FROM"))
 	{
 		if (BADDSEBLK == (from = dse_getblk("FROM", DSEBMLOK, DSEBLKNOCUR)))	/* WARNING: assignment */
@@ -66,12 +70,12 @@ void dse_range(void)
 			return;
 	} else
 		to = cs_addrs->ti->total_blks - 1;
-	if (low = (CLI_PRESENT == cli_present("LOWER")))				/* WARNING: assignment */
+	if ((low = (CLI_PRESENT == cli_present("LOWER"))))				/* WARNING: assignment */
 	{
 		if (!dse_getki(&lower[0], &lower_len, LIT_AND_LEN("LOWER")))
 			return;
 	}
-	if (up = (CLI_PRESENT == cli_present("UPPER")))					/* WARNING: assignment */
+	if ((up = (CLI_PRESENT == cli_present("UPPER"))))					/* WARNING: assignment */
 	{
 		if (!dse_getki(&upper[0], &upper_len, LIT_AND_LEN("UPPER")))
 			return;
