@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -86,6 +86,8 @@ LITDEF nametabent svn_names[] =
 	,{ 6, "ZERROR" }
 	,{ 7, "ZGBLDIR" }
 	,{ 8, "ZHOROLOG" }
+	,{ 2, "ZI" }	/* legacy abbreviation for $ZININTERRUPT */
+	,{ 7, "ZICUVER" }
 	,{12, "ZININTERRUPT" }
 	,{10, "ZINTERRUPT"}
 	,{ 3, "ZIO" }
@@ -138,7 +140,7 @@ LITDEF nametabent svn_names[] =
 LITDEF unsigned char svn_index[27] = {
 	 0,  0,  0,  0,  2,  8,  8,  8, 10,	/* a b c d e f g h i */
 	12, 14 ,16, 16, 16, 16, 16, 18, 20,	/* j k l m n o p q r */
-	22, 28, 34 ,34, 34, 34, 35, 36, 101	/* s t u v w x y z ~ */
+	22, 28, 34 ,34, 34, 34, 35, 36, 103	/* s t u v w x y z ~ */
 };
 
 /* These entries correspond to the entries in the svn_names array */
@@ -183,6 +185,8 @@ LITDEF svn_data_type svn_data[] =
 	,{ SV_ZGBLDIR, TRUE, ALL_SYS }
 	,{ SV_ZHOROLOG, FALSE, ALL_SYS }
 	,{ SV_ZININTERRUPT, FALSE, ALL_SYS}
+	,{ SV_ZICUVER, FALSE, ALL_SYS}
+	,{ SV_ZININTERRUPT, FALSE, ALL_SYS}
 	,{ SV_ZINTERRUPT, TRUE, ALL_SYS}
 	,{ SV_ZIO, FALSE, ALL_SYS }
 	,{ SV_ZJOB, FALSE, ALL_SYS }
@@ -278,7 +282,7 @@ LITDEF nametabent fun_names[] =
 	,{7, "ZBITSTR"}
 	,{7, "ZBITXOR"}
 	,{5, "ZCHAR"}
-	,{3, "ZCO"}
+	,{3, "ZCO"}	 /* legacy abbreviation for $ZCONVERT() */
 	,{8, "ZCOLLATE"}
 	,{8, "ZCONVERT"}
 	,{2, "ZD"}	/* legacy abbreviation for $ZDATE() */
@@ -494,7 +498,7 @@ int expritem(oprtype *a)
 	assert(SIZEOF(svn_names)/SIZEOF(nametabent) == SIZEOF(svn_data)/SIZEOF(svn_data_type)); /* are all SVNs covered? */
 	assert(fun_index[26] == (SIZEOF(fun_names)/SIZEOF(nametabent)));
 	assert(SIZEOF(fun_names)/SIZEOF(nametabent) == SIZEOF(fun_data)/SIZEOF(fun_data_type)); /* are all functions covered? */
-	if (i = tokentable[TREF(window_token)].uo_type)		/* NOTE assignment */
+	if ((i = tokentable[TREF(window_token)].uo_type))		/* NOTE assignment */
 	{
 		type = tokentable[TREF(window_token)].opr_type;
 		advancewindow();
