@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -150,6 +150,7 @@ GBLDEF	bool		out_of_time;
 GBLDEF	io_pair		io_curr_device;		/* current device	*/
 GBLDEF	io_pair		io_std_device;		/* standard device	*/
 GBLDEF	io_log_name	*dollar_principal;	/* pointer to log name GTM$PRINCIPAL if defined */
+GBLDEF	boolean_t	async_dm_read;		/* used by dm_read to tell async_action whether to pop m-stack */
 GBLDEF	boolean_t	prin_dm_io;		/* used by op_dmode so mdb_condition_handler and iorm_readfl know it's active */
 GBLDEF	boolean_t	prin_in_dev_failure;	/* used in I/O to perform NOPRINCIO detection on input */
 GBLDEF	boolean_t	prin_out_dev_failure;	/* used in I/O to perform NOPRINCIO detection on output */
@@ -204,6 +205,7 @@ GBLDEF	int4		backup_close_errno,
 			forced_exit_err,
 			exit_state,
 			restore_read_errno;
+GBLDEF	ABS_TIME	mu_stop_tm_array[EXIT_IMMED - 1] = {{0, 0}, {0, 0}};
 GBLDEF	volatile int4	outofband;
 GBLDEF	int		mumps_status = SS_NORMAL;
 GBLDEF	uint4		stp_array_size;
@@ -227,6 +229,7 @@ GBLDEF	mval		dollar_zgbldir,
 			dollar_zyerror,
 			dollar_ztexit = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0);
 GBLDEF  uint4		dollar_zjob;
+GBLDEF	mstr		dollar_zicuver;
 GBLDEF	mval		dollar_zinterrupt;
 GBLDEF	volatile boolean_t	dollar_zininterrupt;
 GBLDEF	boolean_t	dollar_ztexit_bool; /* Truth value of dollar_ztexit when coerced to boolean */
@@ -327,10 +330,12 @@ GBLDEF	void			(*simple_timeout_timer_ptr)(TID tid, int4 hd_len, boolean_t **time
 GBLDEF	u_casemap_t 		gtm_strToTitle_ptr;		/* Function pointer for gtm_strToTitle */
 #endif
 GBLDEF	boolean_t		mu_reorg_process;		/* set to TRUE by MUPIP REORG */
+GBLDEF	boolean_t		mu_reorg_more_tries;		/* set to TRUE by MUPIP REORG / REORG -UPGRADE */
 GBLDEF	boolean_t		mu_reorg_in_swap_blk;		/* set to TRUE for the duration of the call to "mu_swap_blk" */
 GBLDEF	boolean_t		mu_rndwn_process;
 GBLDEF	gv_key			*gv_currkey_next_reorg;
 GBLDEF	gv_namehead		*reorg_gv_target;
+GBLDEF	gv_namehead		*upgrade_gv_target;		/* DT target's hist from gen_hist_for_blk */
 GBLDEF	struct sockaddr_un	gtmsecshr_sock_name;
 GBLDEF	struct sockaddr_un	gtmsecshr_cli_sock_name;
 GBLDEF	key_t			gtmsecshr_key;
@@ -755,13 +760,14 @@ GBLDEF	mident_fixed	zlink_mname;
 GBLDEF	sm_uc_ptr_t	reformat_buffer;
 GBLDEF	int		reformat_buffer_len;
 GBLDEF	volatile int	reformat_buffer_in_use;	/* used only in DEBUG mode */
-GBLDEF	boolean_t	mu_reorg_upgrd_dwngrd_in_prog;	/* TRUE if MUPIP REORG UPGRADE/DOWNGRADE is in progress */
 GBLDEF	boolean_t	mu_reorg_nosafejnl;		/* TRUE if NOSAFEJNL explicitly specified */
 GBLDEF	trans_num	mu_reorg_upgrd_dwngrd_blktn;	/* tn in blkhdr of current block processed by MUPIP REORG {UP,DOWN}GRADE */
+GBLDEF	uint4		mu_upgrade_in_prog;		/* non-zero when DB upgrade is in progress: 1=UPGRADE, 2=REORG -UPGRADE */
+GBLDEF	enum db_ver	upgrade_block_split_format;	/* MUPIP REORG -UPGRADE started;retain block format in mu_split/gvcst_put */
 GBLDEF	inctn_opcode_t	inctn_opcode = inctn_invalid_op;
 GBLDEF	inctn_detail_t	inctn_detail;			/* holds detail to fill in to inctn jnl record */
 GBLDEF	uint4		region_open_count;		/* Number of region "opens" we have executed */
-GBLDEF	uint4		gtm_blkupgrade_flag = UPGRADE_IF_NEEDED;	/* by default upgrade only if necessary */
+GBLDEF	uint4		gtm_blkupgrade_flag = UPGRADE_IF_NEEDED;	/* US1479867: DSE can manually upgrade a block */
 GBLDEF	boolean_t	disk_blk_read;
 GBLDEF	boolean_t	gtm_dbfilext_syslog_disable;	/* by default, log every file extension message */
 GBLDEF	int4		cws_reorg_remove_index;			/* see mu_swap_blk.c for comments on the need for these two */
@@ -771,6 +777,8 @@ GBLDEF	uint4		gtm_principal_editing_defaults;	/* ext_cap flags if tt */
 GBLDEF	enum db_ver	gtm_db_create_ver;		/* database creation version */
 GBLDEF	boolean_t	in_repl_inst_edit;		/* used by an assert in repl_inst_read/repl_inst_write */
 GBLDEF	boolean_t	in_repl_inst_create;		/* used by repl_inst_read/repl_inst_write */
+GBLDEF	boolean_t	in_rlbk;			/* used by repl_inst_read/repl_inst_write */
+GBLDEF	int		repl_inst_rlbk_fd;		/* used by repl_inst_read/repl_inst_write */
 GBLDEF	boolean_t	holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];	/* whether a particular replication semaphore is being held
 								 * by the current process or not. */
 GBLDEF	boolean_t	detail_specified;	/* Set to TRUE if -DETAIL is specified in MUPIP REPLIC -JNLPOOL or -EDITINST */
@@ -892,7 +900,6 @@ GBLDEF	mval		*alias_retarg;			/* Points to an alias return arg created by a "QUI
 #ifdef DEBUG_ALIAS
 GBLDEF	boolean_t	lvmon_enabled;			/* Enable lv_val monitoring */
 #endif
-GBLDEF	block_id	gtm_tp_allocation_clue;		/* block# hint to start allocation for created blocks in TP */
 GBLDEF	int4		gtm_zlib_cmp_level;		/* zlib compression level specified at process startup */
 GBLDEF	int4		repl_zlib_cmp_level;		/* zlib compression level currently in use in replication pipe.
 							 * This is a source-server specific variable and is non-zero only
@@ -1069,8 +1076,8 @@ GBLDEF	boolean_t	jnlpool_init_needed;		/* TRUE if jnlpool_init should be done at
 							 * anticipatory freeze supported configurations). The variable is set
 							 * explicitly by interested commands (eg., MUPIP REORG).
 							 */
-GBLDEF	char		repl_instfilename[MAX_FN_LEN + 1];	/* save first instance */
-GBLDEF	char		repl_inst_name[MAX_INSTNAME_LEN];	/* for syslog */
+GBLDEF	char		repl_instfilename[MAX_FN_LEN + 1];	/* save $gtm_repl_instance full path for gtmpcat */
+GBLDEF	char		repl_inst_name[MAX_INSTNAME_LEN];	/* used for syslog if first message for gtmpcat */
 GBLDEF	gd_addr		*repl_inst_from_gld;		/* if above obtained from directory */
 GBLDEF	boolean_t	span_nodes_disallowed;		/* Indicates whether spanning nodes are not allowed. For example,
 							 * they are not allowed for GT.CM OMI and GNP.
@@ -1183,6 +1190,11 @@ GBLDEF	boolean_t	in_mu_cre_file;			/* TRUE only if inside "mu_cre_file" function
 GBLDEF	enum dbg_wtfini_lcnt_t	dbg_wtfini_lcnt;	/* "lcnt" value for WCS_OPS_TRACE tracking purposes. This is dbg-only
 							 * and hence it is okay to be a gbldef instead of a threadgbldef.
 							 */
+GBLDEF	bool		only_usr_jnlpool_flush;		/* Debug-only marker used by instance-oriented processes like rollback
+							 * and the source server to indicate to a repl_instance_flush that the
+							 * process has obtained exclusive access to the instance and therefore
+							 * does not need the ordinarily-required locks.
+							 */
 #endif
 GBLDEF	sgm_info	*sgm_info_ptr;
 GBLDEF	tp_region	*tp_reg_free_list;	/* Ptr to list of tp_regions that are unused */
