@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2022 Fidelity National Information	*
+ * Copyright (c) 2010-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -87,13 +87,13 @@ THREADGBLDEF(temp_subs,				boolean_t)			/* flag temp storing of subscripts to pr
 THREADGBLDEF(trigger_compile_and_link,		boolean_t)			/* A trigger compilation/link is active */
 THREADGBLDEF(window_ident,			mstr)				/* current scanner mident from advancewindow */
 THREADGBLDEF(window_mval,			mval)				/* current scanner mval from advancewindow */
-THREADGBLDEF(window_token,			char)				/* current scanner token from advancewindow */
+THREADGBLDEF(window_token,			unsigned char)			/* current scanner token from advancewindow */
 THREADGBLDEF(xecute_literal_parse,		boolean_t)			/* flag TRUE when trying what its name says */
 THREADGBLDEF(fetch_control,			fetch_ctrl)			/* structure for managing lvn fetches */
 
 /* Database */
 THREADGBLDEF(dbinit_max_delta_secs,		uint4)				/* max time before we bail out in db_init */
-THREADGBLDEF(dollar_zmaxtptime, 		int4)				/* tp timeout in seconds */
+THREADGBLDEF(dollar_zmaxtptime, 		int4)				/* tp timeout in milliseconds */
 THREADGBLAR1DEF(save_xfer_root,		save_xfer_entry, DEFERRED_EVENTS)	/* array of deferred events with the zeroth
 										 * acting as the root of a queue identified by the
 										 * _ptr item immediately following
@@ -185,7 +185,8 @@ THREADGBLDEF(redo_rootsrch_ctxt,		redo_root_search_context)	/* context to be sav
 THREADGBLDEF(semwait2long,			volatile boolean_t)		/* Waited too long for a semaphore */
 THREADGBLDEF(skip_DB_exists_check,		boolean_t)			/* skip check for whether the DB file exists */
 THREADGBLDEF(skip_file_corrupt_check,		boolean_t)			/* skip file_corrupt check in grab_crit */
-THREADGBLDEF(tpnotacidtime,			mval)				/* limit for long non-ACID ops in transactions */
+THREADGBLDEF(tpnotacidtime,			int4)				/* millisecond limit 4 possibly long non-ACID ops */
+THREADGBLDEF(tpnotacidtries,			int4)				/* limit on tpnotacid events in TP */
 THREADGBLDEF(tp_restart_count,			uint4)				/* tp_restart counter */
 THREADGBLDEF(tp_restart_dont_counts,		int4)				/* tp_restart count adjustment; NOTE: DEBUG only */
 THREADGBLDEF(tp_restart_entryref,		mval)				/* tp_restart position for reporting */
@@ -237,7 +238,7 @@ THREADGBLDEF(arlink_loaded,			uint4)				/* Count of auto-relink enabled routines
 THREADGBLDEF(collseq_list,			collseq *)			/* list of pointers to currently mapped collation
 										 * algorithms - since this seems only used in
 										 * collseq.c -seems more like a STATICDEF */
-THREADGBLFPTR(create_fatal_error_zshow_dmp_fptr, void, 		(void))		/* Fptr for gtm_fatal_error* zshow dmp routine */
+THREADGBLFPTR(create_fatal_error_zshow_dmp_fptr, void, 		(int4))		/* Fptr for gtm_fatal_error* zshow dmp routine */
 THREADGBLDEF(disable_sigcont,			boolean_t)			/* indicates whether the SIGCONT signal
 										 * is allowed internally */
 THREADGBLDEF(dollar_zcompile,			mstr)				/* compiler qualifiers */
@@ -263,7 +264,8 @@ THREADGBLDEF(gtmdbgflags_freq,			int)
 THREADGBLDEF(gtmdbgflags_freq_cntr,		int)
 #endif
 THREADGBLDEF(gtm_env_init_started,		boolean_t)			/* gtm_env_init flag envvar processing */
-THREADGBLFPTR(gtm_env_xlate_entry,		int,		())		/* gtm_env_xlate() function pointer */
+										/* gtm_env_xlate() function pointer */
+THREADGBLFPTR(gtm_env_xlate_entry,		int,		(xc_string_t *, xc_string_t *, xc_string_t *, xc_string_t *))
 THREADGBLDEF(gtm_environment_init,		boolean_t)			/* indicates GT.M development environment rather
 										 * than a production environment */
 THREADGBLFPTR(gtm_sigusr1_handler,		void, 		(void))		/* SIGUSR1 signal handler function ptr */
@@ -279,6 +281,9 @@ THREADGBLDEF(gtmprompt,				mstr)				/* mstr pointing to prombuf containing the G
 										 * prompt */
 THREADGBLDEF(gtmsecshr_comkey,			unsigned int)			/* Hashed version key for gtmsecshr communications
 										 * eliminates cross-version issues */
+THREADGBLDEF(gvcst_incr_numoflow,		boolean_t)			/* set to TRUE by s2n if NUMOFLOW error encountered
+										 * during $INCREMENT to retry the operation, set to
+										 * FALSE in gvincr_compute_post_incr */
 THREADGBLDEF(gvcst_statsDB_open_ch_active,	boolean_t)			/* Condition handler is active */
 THREADGBLDEF(in_zwrite,				boolean_t)			/* ZWrite is active */
 THREADGBLDEF(is_socketpool,			boolean_t)			/* True when device-to-be-opened is socketpool */
@@ -508,6 +513,10 @@ THREADGBLDEF(nontp_jbuf_rsrv,			jbuf_rsrv_struct_t *)	/* Pointer to structure co
 									 * on the journal buffer for current non-TP transaction.
 									 */
 THREADGBLDEF(source_line,			int4)		/* keep track of line number in M file while compiling */
+THREADGBLDEF(defer_instance_freeze,		uint4)		/* system for deferring recognition of certain kinds of freezes
+								 * while a commit is in-flight so long as the commit does not
+								 * cause the freeze.
+								 */
 /* Debug values */
 #ifdef DEBUG
 THREADGBLDEF(donot_commit,			boolean_t)			/* debug-only - see gdsfhead.h for purpose */
@@ -551,6 +560,7 @@ THREADGBLDEF(fork_without_child_wait,		boolean_t)	/*  we did a FORK but did not
 THREADGBLDEF(in_bm_getfree_gdsfilext,           boolean_t)	/* bm_getfree() did a preemptive crit grab before doing a
 								 * file extension.
 								 */
+THREADGBLDEF(cur_cmt_step,			enum cmt_step)	/* Current step of commit. Intended to validate t_commit_cleanup */
 #endif	/* #ifdef DEBUG */
 /* (DEBUG_ONLY relevant points reproduced from the comment at the top of this file)
  *   5. It is important for ANY DEBUG_ONLY fields to go at the VERY END. Failure to do this breaks gtmpcat.
