@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -41,7 +41,8 @@ GBLREF	uint4			dollar_tlevel;
 GBLREF	sgm_info		*sgm_info_ptr;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
-GBLREF	boolean_t		mu_reorg_upgrd_dwngrd_in_prog, write_after_image;
+GBLREF	boolean_t		write_after_image;
+GBLREF	uint4			mu_upgrade_in_prog;
 GBLREF	unsigned int		t_tries;
 GBLREF	jnl_gbls_t		jgbl;
 
@@ -51,7 +52,6 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 	boolean_t	long_blk_id;
 	int4		offset, blk_id_sz, off_chain_sz;
 	off_chain	chain;
-	v6_off_chain	v6_chain;
 	sm_uc_ptr_t	ptr, ptrtop, c;
 	sm_ulong_t	n;
 	trans_num	blktn;
@@ -69,11 +69,11 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 	 * which holds crit for the entire duration
 	 */
 	assert((dba_bg != cs_data->acc_meth) || dollar_tlevel || !cs_addrs->now_crit || write_after_image || jgbl.onlnrlbk
-		|| mu_reorg_upgrd_dwngrd_in_prog || ((NULL != cse->recompute_list_head) && (gds_t_write == cse->mode)));
+		|| mu_upgrade_in_prog || ((NULL != cse->recompute_list_head) && (gds_t_write == cse->mode)));
 	assert((dba_mm != cs_data->acc_meth) || dollar_tlevel || cs_addrs->now_crit);
 	assert(cse->mode != gds_t_writemap);
 	assert(cse->ondsk_blkver);
-	array = (blk_segment *)cse->upd_addr;
+	array = cse->upd_addr.blk;
 	long_blk_id = BLK_ID_32_VER < cse->ondsk_blkver;
 	blk_id_sz = SIZEOF_BLK_ID(long_blk_id);
 	off_chain_sz = blk_id_sz; /* the off_chain struct should be the same size as the block_id in the current block */
@@ -188,13 +188,13 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 			chain.cw_index = cse->index;
 			assert((1LL << NEXT_OFF_MAX_BITS) > cse->next_off);
 			chain.next_off = cse->next_off;
-			WRITE_OFF_CHAIN(long_blk_id, &chain, &v6_chain, ptr);
+			WRITE_OFF_CHAIN(long_blk_id, &chain, ptr);
 			cse->index = 0;
 			cse->ins_off = 0;
 			cse->next_off = 0;
 		}
 #		ifdef DEBUG
-		if (offset = cse->first_off)
+		if ((offset = cse->first_off))
 		{	/* Verify the integrity of the TP chains within a newly created block.
 			 * If it is the first TP private build, the update array could have referenced
 			 * shared memory global buffers which could have been concurrently updated.
@@ -215,7 +215,6 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 					GET_USHORT(nRecLen, &((rec_hdr_ptr_t)ptr)->rsiz);
 					if (0 == nRecLen)
 					{
-						assert(NULL == input_base_addr);
 						integ_error_found = TRUE;
 						break;
 					}
@@ -228,7 +227,6 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 							;
 						if (c >= ptrtop)
 						{
-							assert(NULL == input_base_addr);
 							integ_error_found = TRUE;
 							break;
 						}
@@ -240,21 +238,18 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 						if (((ptr - off_chain_sz) != chainptr)
 							&& ((ptr - off_chain_sz - COLL_SPEC_LEN) != chainptr))
 						{
-							assert(NULL == input_base_addr);
 							integ_error_found = TRUE;
 						}
 						break;
 					}
 					if (c > chainptr)
 					{
-						assert(NULL == input_base_addr);
 						integ_error_found = TRUE;
 						break;
 					}
-					READ_OFF_CHAIN(long_blk_id, &chain, &v6_chain, c);
+					READ_OFF_CHAIN(long_blk_id, &chain, c);
 					if (chain.flag)
 					{
-						assert(NULL == input_base_addr);
 						integ_error_found = TRUE;
 						break;
 					}
@@ -263,7 +258,7 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 					break;
 				if (chainptr < ptrtop)
 				{
-					READ_OFF_CHAIN(long_blk_id, &chain, &v6_chain, chainptr);
+					READ_OFF_CHAIN(long_blk_id, &chain, chainptr);
 					assert(1 == chain.flag || (skip_block_chain_tail_check && (0 == chain.next_off)));
 					assert(chain.cw_index < sgm_info_ptr->cw_set_depth);
 					offset = chain.next_off;
