@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2004-2020 Fidelity National Information	*
+ * Copyright (c) 2004-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -27,6 +27,8 @@ void		db_init_err_cleanup(boolean_t retry_dbinit);
 void		gvcst_redo_root_search(void);
 gd_region	*dbfilopn (gd_region *reg);
 void		dbsecspc(gd_region *reg, sgmnt_data_ptr_t csd, gtm_uint64_t *sec_size);
+void		bg_sync_init(node_local_ptr_t cnl);
+
 unsigned char	gvcst_cre_autoDB(gd_region *reg);
 mint		gvcst_data(void);
 mint		gvcst_data2(void);
@@ -112,7 +114,7 @@ GBLREF	boolean_t	gv_play_duplicate_kills;
  */
 #define	IS_OK_TO_INVOKE_GVCST_KILL(GVT)											\
 (															\
-	DBG_ASSERT(!jgbl.forw_phase_recovery || gv_play_duplicate_kills GTMTRIG_ONLY(&& IS_EXPLICIT_UPDATE_NOASSERT))	\
+	DBG_ASSERT(!jgbl.forw_phase_recovery || (gv_play_duplicate_kills GTMTRIG_ONLY(&& IS_EXPLICIT_UPDATE_NOASSERT)))	\
 	DBG_ASSERT(!jgbl.forw_phase_recovery || gv_target->root || (JS_IS_DUPLICATE & jgbl.mur_jrec_nodeflags)		\
 			|| jgbl.mur_options_forward)									\
 	(GVT->root || (gv_play_duplicate_kills GTMTRIG_ONLY(&& IS_EXPLICIT_UPDATE_NOASSERT)))				\
