@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -103,6 +103,8 @@ GBLDEF int			muint_adj;
 GBLDEF int			mu_int_plen;
 GBLDEF int			mu_map_errs;
 GBLDEF int			trans_errors = 0;
+GBLDEF int4			max_data_blk_size;
+GBLDEF int4			max_index_blk_size;
 GBLDEF boolean_t		block = FALSE;
 GBLDEF boolean_t		muint_fast = FALSE;
 GBLDEF boolean_t		master_dir;
@@ -117,7 +119,7 @@ GBLDEF global_list		*trees;
 GBLDEF sgmnt_data		mu_int_data;
 GBLDEF unsigned char		*mu_int_master;
 GBLDEF trans_num		largest_tn;
-GBLDEF int4			mu_int_blks_to_upgrd;
+GBLDEF block_id			mu_int_blks_to_upgrd;
 GBLDEF span_node_integ		*sndata;
 GBLDEF boolean_t		null_coll_type_err = FALSE;
 GBLDEF boolean_t		null_coll_type;
@@ -190,14 +192,14 @@ void mupip_integ(void)
 	file_control		*fc;
 	gtm_uint64_t		blocks_free = MAXUINT8;
 	gtm_uint64_t		mu_int_tot[TOT_TYPE_MAX][CUM_TYPE_MAX], tot_blks, tot_recs;
-	tp_region		*rptr;
+	tp_region		*rptr = NULL;
 	sgmnt_addrs		*csa;
 	sgmnt_data_ptr_t	csd;
 	span_node_integ		span_node_data;
 	char			ss_filename[GTM_PATH_MAX];
 	unsigned short		ss_file_len = GTM_PATH_MAX;
 	unix_db_info		*udi;
-	gd_region		*baseDBreg, *reg;
+	gd_region		*baseDBreg, *reg = NULL;
 	sgmnt_addrs		*baseDBcsa;
 	node_local_ptr_t	baseDBnl;
 	sgmnt_addrs		*tcsa;
@@ -314,6 +316,8 @@ void mupip_integ(void)
 		keylen = SIZEOF(key_buff);
 		if (0 == cli_get_str("SUBSCRIPT", (char *)key_buff, &keylen))
 			mupip_exit(ERR_MUPCLIERR);
+		if (rptr)
+			gv_cur_region = rptr->reg;
 		if (FALSE == mu_getkey(key_buff, keylen))
 			mupip_exit(ERR_MUPCLIERR);
 		assert(mu_key);	/* or else "mu_getkey" call above would have returned FALSE */
@@ -327,6 +331,26 @@ void mupip_integ(void)
 		disp_map_errors = 0;
 		master_dir = FALSE;
 	}
+	if (CLI_PRESENT == cli_present("DMAXBLOCKSIZE"))
+	{
+		if (0 == cli_get_int("DMAXBLOCKSIZE", &max_data_blk_size))
+			max_data_blk_size = 0;
+		else if (max_data_blk_size < 0)
+			max_data_blk_size = 0;
+		else if (max_data_blk_size > MAX_DB_BLK_SIZE)
+			max_data_blk_size = MAX_DB_BLK_SIZE;
+	} else
+		max_data_blk_size = 0;
+	if (CLI_PRESENT == cli_present("IMAXBLOCKSIZE"))
+	{
+		if (0 == cli_get_int("IMAXBLOCKSIZE", &max_index_blk_size))
+			max_index_blk_size = 0;
+		else if (max_index_blk_size < 0)
+			max_index_blk_size = 0;
+		else if (max_index_blk_size > MAX_DB_BLK_SIZE)
+			max_index_blk_size = MAX_DB_BLK_SIZE;
+	} else
+		max_index_blk_size = 0;
 	muint_all_index_blocks = !(block || mu_key);
 	mu_int_master = malloc(MASTER_MAP_SIZE_MAX);
 	tn_reset_specified = (CLI_PRESENT == cli_present("TN_RESET"));
@@ -403,6 +427,7 @@ void mupip_integ(void)
 		sndata->sn_type = SN_NOT;
 		if (region)
 		{
+			assert(reg);
 			if (stats_specified)
 			{	/* -STATS has been specified. So only work on the stats region, not the base region.
 				 * "mu_getlst" would have added only the base region in the list. Replace that with
@@ -522,6 +547,7 @@ void mupip_integ(void)
 			{
 				if (region_was_frozen)
 				{
+					assert(reg);
 					region_freeze(reg, FALSE, FALSE, FALSE, FALSE, FALSE);
 					if (!reg->read_only)
 					{
@@ -699,14 +725,11 @@ void mupip_integ(void)
 			if (!muint_fast && (mu_int_blks_to_upgrd != csd->blks_to_upgrd))
 			{
 				gtm_putmsg_csa(CSA_ARG(csa)
-					VARLSTCNT(4) ERR_DBBTUWRNG, 2, &(csd->blks_to_upgrd), &mu_int_blks_to_upgrd);
+					VARLSTCNT(4) ERR_DBBTUWRNG, 2, &mu_int_blks_to_upgrd, &(csd->blks_to_upgrd));
 				if (gv_cur_region->read_only || mu_int_errknt)
 					mu_int_errknt++;
-				else
-				{
-					csd->blks_to_upgrd = mu_int_blks_to_upgrd;
+				else if (!ointeg_this_reg)
 					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(1) ERR_DBBTUFIXED);
-				}
 			}
 			if (((0 != mu_int_data.kill_in_prog) || (0 != mu_int_data.abandoned_kills)) && (!mu_map_errs) && !region
 				&& !gv_cur_region->read_only)
@@ -810,10 +833,11 @@ void mupip_integ(void)
 			{
 				if (MAXUINT8 != blocks_free)
 					csd->trans_hist.free_blocks = blocks_free;
-				if (!mu_int_errknt && muint_all_index_blocks && !muint_fast)
+				if ((!mu_int_errknt && muint_all_index_blocks && !muint_fast)
+						&& (mu_int_blks_to_upgrd != csd->blks_to_upgrd))
 				{
-					if (mu_int_blks_to_upgrd != csd->blks_to_upgrd)
-						csd->blks_to_upgrd = mu_int_blks_to_upgrd;
+					csd->blks_to_upgrd = mu_int_blks_to_upgrd;
+					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(1) ERR_DBBTUFIXED);
 				}
 				csd->span_node_absent = (sndata->sn_cnt) ? FALSE : TRUE;
 				csd->maxkeysz_assured = (maxkey_errors) ? FALSE : TRUE;
