@@ -1,6 +1,6 @@
 /***************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -69,6 +69,7 @@
 error_def(ERR_CONCURTRUNCINPROG);
 error_def(ERR_DBRDONLY);
 error_def(ERR_EXCLUDEREORG);
+error_def(ERR_GTMCURUNSUPP);
 error_def(ERR_MUNOACTION);
 error_def(ERR_MUNOFINISH);
 error_def(ERR_MUPCLIERR);
@@ -79,6 +80,7 @@ error_def(ERR_NOSELECT);
 error_def(ERR_NOEXCLUDE);
 error_def(ERR_REORGCTRLY);
 error_def(ERR_REORGINC);
+error_def(ERR_REORGUPCNFLCT);
 error_def(ERR_MUKEEPNOTRUNC);
 error_def(ERR_MUKEEPPERCENT);
 error_def(ERR_MUKEEPNODEC);
@@ -86,7 +88,9 @@ error_def(ERR_MUKEEPNODEC);
 GBLREF	bool			error_mupip;
 GBLREF	bool			mu_ctrlc_occurred;
 GBLREF	bool			mu_ctrly_occurred;
+GBLREF	boolean_t		debug_mupip;
 GBLREF	boolean_t		jnlpool_init_needed;
+GBLREF	boolean_t		mu_reorg_more_tries;
 GBLREF	boolean_t		mu_reorg_process;
 GBLREF	gd_region		*gv_cur_region;
 GBLREF	gv_key			*gv_currkey_next_reorg, *gv_currkey, *gv_altkey;
@@ -104,7 +108,7 @@ static readonly mval literal_poollimit =
 void mupip_reorg(void)
 {
 	boolean_t		resume, reorg_success = TRUE;
-	int			data_fill_factor, index_fill_factor;
+	int			data_fill_factor, index_fill_factor, min_level = 0;
 	int			reorg_op, reg_max_rec, reg_max_key, reg_max_blk, status;
 	char			cli_buff[MAX_LINE], *ptr, keep_value_buffer[MAX_LINE + 1];
 	glist			gl_head, exclude_gl_head, *gl_ptr, hasht_gl;
@@ -129,6 +133,8 @@ void mupip_reorg(void)
 	mu_outofband_setup();
 	truncate = FALSE;
 	reg_list = NULL;
+	/* DBG qualifier prints extra debug messages where applicable */
+	debug_mupip = (CLI_PRESENT == cli_present("DBG"));
 	if (CLI_PRESENT == cli_present("TRUNCATE"))
 		truncate = TRUE;
 	if (CLI_PRESENT == cli_present("KEEP"))
@@ -142,7 +148,8 @@ void mupip_reorg(void)
 			keep_mval_ptr = (&keep_mval);
 			if ('%' == keep_mval.str.addr[keep_mval.str.len - 1])
 			{
-				STRNCPY_STR(cli_buff, keep_mval.str.addr,keep_mval.str.len);
+				assert(sizeof(cli_buff) > keep_mval.str.len);
+				memcpy(cli_buff, keep_mval.str.addr,keep_mval.str.len);
 				cli_buff[keep_mval.str.len - 1] = '\0';
 				if (!cli_is_dcm(cli_buff))
 				{
@@ -170,8 +177,10 @@ void mupip_reorg(void)
 			mupip_exit(ERR_MUNOACTION);
 		}
 	}
-	if ((CLI_PRESENT == cli_present("UPGRADE")) || (CLI_PRESENT == cli_present("DOWNGRADE")))
-	{	/* Note that "mu_reorg_process" is not set to TRUE in case of MUPIP REORG -UPGRADE/DOWNGRADE.
+	if (CLI_PRESENT == cli_present("DOWNGRADE"))
+		mupip_exit(ERR_GTMCURUNSUPP);
+	if (CLI_PRESENT == cli_present("UPGRADE"))
+	{	/* Note that "mu_reorg_process" is not set to TRUE in case of MUPIP REORG -UPGRADE
 		 * This is intentional because we are not doing any REORG kind of processing.
 		 */
 		mu_reorg_upgrd_dwngrd();
@@ -257,6 +266,13 @@ void mupip_reorg(void)
 		index_fill_factor = data_fill_factor;
 	util_out_print("Fill Factor:: Index blocks !UL%: Data blocks !UL%", FLUSH, index_fill_factor, data_fill_factor);
 
+	if ((cli_status = cli_present("MIN_LEVEL")))
+	{
+		if (!cli_get_int("MIN_LEVEL", &min_level))
+			min_level = 0;
+		else if (min_level > MAX_BT_DEPTH)
+			min_level = MAX_BT_DEPTH;
+	}
 	n_len = SIZEOF(cli_buff);
 	memset(cli_buff, 0, n_len);
 	if (CLI_PRESENT != cli_present("EXCLUDE"))
@@ -291,7 +307,7 @@ void mupip_reorg(void)
 	}
 	TREF(want_empty_gvts) = FALSE;
 
-	mu_reorg_process = TRUE;
+	mu_reorg_more_tries = mu_reorg_process = TRUE;
 	assert(NULL == gv_currkey_next_reorg);
 	GVKEYSIZE_INIT_IF_NEEDED;	/* sets "gv_keysize", "gv_currkey" and "gv_altkey" (if not already done) */
 	GVKEY_INIT(gv_currkey_next_reorg, gv_keysize);
@@ -302,7 +318,7 @@ void mupip_reorg(void)
 	{
 		/* mu_reorg_process can't be set before gv_select above as the assert(!mu_reorg_process) fails in gvcst_search.
 		 * The option to set mu_reorg_process before calling gv_select and clearing it after being used in gvcst_init,
-		 *  can't be taken up as gvcst_init is called multiple times, once for each region.
+		 * can't be taken up as gvcst_init is called multiple times, once for each region.
 		 * This means gvcst_init() doesn't set csa->gbuff and we need to set csa->gbuff_limit explicitly now. */
 		if (0 == (TREF(gbuff_limit)).str.len)
 		{
@@ -328,7 +344,7 @@ void mupip_reorg(void)
 		 */
 		reorg_gv_target->gvname.var_name = GNAME(gl_ptr);
 		GTMTRIG_ONLY(assert(!IS_MNAME_HASHT_GBLNAME(reorg_gv_target->gvname.var_name));)
-		cur_success = mu_reorg(gl_ptr, &exclude_gl_head, &resume, index_fill_factor, data_fill_factor, reorg_op);
+		cur_success = mu_reorg(gl_ptr, &exclude_gl_head, &resume, index_fill_factor, data_fill_factor, reorg_op, min_level);
 		reorg_success &= cur_success;
 		SET_GV_CURRKEY_FROM_GVT(reorg_gv_target);
 		if (truncate)
@@ -368,7 +384,8 @@ void mupip_reorg(void)
 								REG_LEN_STR(gv_cur_region));
 							reorg_gv_target->gvname.var_name = gv_target->gvname.var_name;
 							cur_success = mu_reorg(&hasht_gl, &exclude_gl_head, &resume,
-										index_fill_factor, data_fill_factor, reorg_op);
+										index_fill_factor, data_fill_factor, reorg_op,
+										min_level);
 							reorg_success &= cur_success;
 						}
 					}
@@ -381,6 +398,18 @@ void mupip_reorg(void)
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REORGCTRLY);
 			mupip_exit(ERR_MUNOFINISH);
 		}
+		if ((0 != cs_addrs->nl->reorg_upgrade_pid) && (is_proc_alive(cs_addrs->nl->reorg_upgrade_pid, 0)))
+		{	/* REORG -UPGRADE cannot run concurrently with TRUNCATE and is higher priority. Stop */
+			send_msg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+					LEN_AND_LIT("REORG"),
+					LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+					cs_addrs->nl->reorg_upgrade_pid);
+			gtm_putmsg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+					LEN_AND_LIT("REORG"),
+					LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+					cs_addrs->nl->reorg_upgrade_pid);
+			mupip_exit(ERR_MUNOFINISH);
+		}
 	}
 	status = SS_NORMAL;
 	if (!reorg_success)
@@ -390,6 +419,26 @@ void mupip_reorg(void)
 		status = ERR_REORGINC;
 	} else if (truncate)
 	{
+		for (reg_iter = reg_list; reg_iter; reg_iter = reg_iter->next)
+		{
+			gv_cur_region = reg_iter->reg;
+			tp_change_reg();
+			csd = cs_data;
+			csa = cs_addrs;
+			cnl = csa->nl;
+			if ((0 != cnl->reorg_upgrade_pid) && (is_proc_alive(cnl->reorg_upgrade_pid, 0)))
+			{	/* REORG -UPGRADE cannot run concurrently with TRUNCATE and is higher priority. Stop */
+				send_msg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+						LEN_AND_LIT("REORG -TRUNCATE"),
+						LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+						cnl->reorg_upgrade_pid);
+				gtm_putmsg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+						LEN_AND_LIT("REORG -TRUNCATE"),
+						LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+						cnl->reorg_upgrade_pid);
+				mupip_exit(ERR_MUNOFINISH);
+			}
+		}
 		/* Move GVT ROOT blocks of all global names AFTER doing regular reorg on ALL global names.
 		 * This way we ensure one pass of reorg -truncate is enough to produce an optimally truncated file.
 		 */
@@ -401,19 +450,22 @@ void mupip_reorg(void)
 		{
 			gv_cur_region = reg_iter->reg;
 			tp_change_reg();
+			csd = cs_data;
+			csa = cs_addrs;
+			cnl = csa->nl;
 			SET_GVTARGET_TO_HASHT_GBL(cs_addrs);	/* sets gv_target */
 			SET_GV_CURRKEY_FROM_GVT(gv_target);
 			gv_target->root = 0; /* Recompute gv_target->root in case mu_reorg changed things around */
 			inctn_opcode = inctn_invalid_op;	/* needed for GVCST_ROOT_SEARCH */
 			GVCST_ROOT_SEARCH;			/* set gv_target->root */
-			if (0 == gv_target->root)
+			if ((0 == gv_target->root) || (0 != cnl->reorg_upgrade_pid))
 				continue;
 			hasht_gl.reg = gv_cur_region;
 			hasht_gl.gvt = gv_target;
 			mu_swap_root(&hasht_gl, &root_swap_statistic, 0);
 		}
 		util_out_print("Total root blocks moved: !UL", FLUSH, root_swap_statistic);
-		mu_reorg_process = FALSE;
+		mu_reorg_more_tries = mu_reorg_process = FALSE;
 		/* Default threshold is 0 i.e. we attempt to truncate no matter what free_blocks is. */
 		truncate_percent = 0;
 		cli_get_int("TRUNCATE", (int4 *)&truncate_percent);
@@ -439,6 +491,19 @@ void mupip_reorg(void)
 						REG_LEN_STR(gv_cur_region));
 				continue;
 			}
+			if ((0 != cnl->reorg_upgrade_pid) && (is_proc_alive(cnl->reorg_upgrade_pid, 0)))
+			{	/* REORG -UPGRADE cannot run concurrently with TRUNCATE and is higher priority. Stop */
+				rel_crit(gv_cur_region);
+				send_msg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+						LEN_AND_LIT("REORG -TRUNCATE"),
+						LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+						cnl->reorg_upgrade_pid);
+				gtm_putmsg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+						LEN_AND_LIT("REORG -TRUNCATE"),
+						LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+						cnl->reorg_upgrade_pid);
+				mupip_exit(ERR_MUNOFINISH);
+			}
 			cnl->trunc_pid = process_id;
 			cnl->highest_lbm_with_busy_blk = 0;
 			rel_crit(gv_cur_region);
