@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2020 Fidelity National Information	*
+ * Copyright (c) 2003-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -34,6 +34,7 @@
 #include "hashtab_int8.h"	/* needed for muprec.h */
 #include "hashtab_mname.h"	/* needed for muprec.h */
 #include "muprec.h"
+#include "mutex.h"
 #include "iosp.h"
 #include "tp_change_reg.h"
 #include "gds_rundown.h"
@@ -95,7 +96,10 @@ GBLREF	uint4			process_id;
 GBLREF	void			(*call_on_signal)();
 #ifdef DEBUG
 GBLREF	boolean_t		prin_out_dev_failure;
+GBLREF	bool			only_usr_jnlpool_flush;
 #endif
+GBLREF int			in_rlbk;
+GBLREF boolean_t		repl_inst_rlbk_fd;
 
 error_def(ERR_FILERENAME);
 error_def(ERR_JNLACTINCMPLT);
@@ -151,6 +155,7 @@ boolean_t mur_close_files(void)
 #	ifdef DEBUG
 	int		semval;
 #	endif
+	boolean_t		local_ftok_counter_halted = FALSE;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -457,7 +462,7 @@ boolean_t mur_close_files(void)
 				jctl = rctl->jctl_turn_around;
 				head_jnl_fn_len = jctl->jnl_fn_len;
 				head_jnl_fn = fn;
-				memcpy(head_jnl_fn, jctl->jnl_fn, head_jnl_fn_len);
+				memcpy((void *)head_jnl_fn, jctl->jnl_fn, head_jnl_fn_len);
 				/* reset jctl->jfh->recover_interrupted field in all recover created jnl files to signal
 				 * that a future recover should not consider this recover as an interrupted recover.
 				 */
@@ -594,6 +599,7 @@ boolean_t mur_close_files(void)
 	 * (due to some other process still accessing the instance file and/or db/jnl). In that case "clean_exit" should be FALSE.
 	 */
 	assert(!mur_options.rollback || mur_options.forward || murgbl.repl_standalone || !murgbl.clean_exit);
+	local_ftok_counter_halted = FALSE;
 	if (mur_options.rollback)
 	{
 		if (mur_options.forward && murgbl.clean_exit && murgbl.consist_jnl_seqno && strm_seqno_nonzero)
@@ -682,9 +688,10 @@ boolean_t mur_close_files(void)
 						assert(anticipatory_freeze_available);
 						assert(!csa->hold_onto_crit);
 						grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, ASSERT_NO_ONLINE_ROLLBACK);
-					}
+					} else
+						was_crit = FALSE;
 					last_histinfo_seqno = repl_inst_histinfo_truncate(murgbl.consist_jnl_seqno);
-					if ((NULL != jnlpool->jnlpool_ctl) && !was_crit)
+					if ((NULL != jnlpool->jnlpool_ctl) && !was_crit && !csa->hold_onto_crit)
 						rel_lock(jnlpool->jnlpool_dummy_reg);
 					/* The above also updates "repl_inst_filehdr->jnl_seqno". If regular rollback, it also
 					 * updates "repl_inst_filehdr->crash" to FALSE. For online rollback, we have to update
@@ -700,7 +707,8 @@ boolean_t mur_close_files(void)
 						jnlpool->jnlpool_ctl->jnl_seqno = murgbl.consist_jnl_seqno;
 						jnlpool->jnlpool_ctl->start_jnl_seqno = murgbl.consist_jnl_seqno;
 						jnlpool->jnlpool_ctl->rsrv_write_addr = jnlpool->jnlpool_ctl->write_addr = 0;
-						jnlpool->jnlpool_ctl->rsrv_write_addr = 0;
+						jnlpool->jnlpool_ctl->contig_addr = 0;
+						rollback_mutex_cln_ctl(murgbl.consist_jnl_seqno);
 						assert(jnlpool->jnlpool_ctl->phase2_commit_index1
 							== jnlpool->jnlpool_ctl->phase2_commit_index2);
 						jnlpool->jnlpool_ctl->lastwrite_len = 0;
@@ -745,6 +753,15 @@ boolean_t mur_close_files(void)
 				repl_inst_read(udi->fn, (off_t)REPL_INST_HDR_SIZE, (sm_uc_ptr_t)gtmsrc_lcl_array, GTMSRC_LCL_SIZE);
 				for (idx = 0; idx < NUM_GTMSRC_LCL; idx++)
 				{	/* Check if the slot is being used and only then check the resync_seqno */
+#					ifdef DEBUG
+					if (jnlpool)
+					{
+						gtmsourcelocal_ptr = &jnlpool->gtmsource_local_array[idx];
+						latch = &gtmsourcelocal_ptr->gtmsource_srv_latch;
+					}
+					assert(murgbl.repl_standalone
+							|| (jnlpool && (process_id == latch->u.parts.latch_pid)));
+#					endif
 					if ('\0' != gtmsrc_lcl_array[idx].secondary_instname[0])
 					{
 						if (gtmsrc_lcl_array[idx].resync_seqno > murgbl.consist_jnl_seqno)
@@ -753,7 +770,9 @@ boolean_t mur_close_files(void)
 							gtmsrc_lcl_array[idx].connect_jnl_seqno = murgbl.consist_jnl_seqno;
 					}
 				}
+				DEBUG_ONLY(only_usr_jnlpool_flush = murgbl.repl_standalone;)
 				repl_inst_write(udi->fn, (off_t)REPL_INST_HDR_SIZE, (sm_uc_ptr_t)gtmsrc_lcl_array, GTMSRC_LCL_SIZE);
+				DEBUG_ONLY(only_usr_jnlpool_flush = false;)
 			}
 			if (((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl)) && jgbl.onlnrlbk)
 			{	/* Remove any locks that we acquired in mur_open_files.
@@ -775,6 +794,8 @@ boolean_t mur_close_files(void)
 					}
 				}
 				csa->hold_onto_crit = FALSE;
+				/* Stash value so that we have it after detaching the journal pool */
+				local_ftok_counter_halted = jnlpool->jnlpool_ctl->ftok_counter_halted;
 			}
 		}
 	}
@@ -869,7 +890,7 @@ boolean_t mur_close_files(void)
 			 */
 			assert(INVALID_SHMID == repl_instance.recvpool_shmid || jgbl.onlnrlbk);
 			/* Check frozen state before detaching the journal pool */
-			inst_frozen = IS_REPL_INST_FROZEN;
+			inst_frozen = IS_REPL_INST_FROZEN(RECOGNIZE_FREEZES);
 			/* Ensure that no new processes have attached to the journal pool */
 			if (-1 == shmctl(repl_instance.jnlpool_shmid, IPC_STAT, &shm_buf))
 			{
@@ -920,6 +941,7 @@ boolean_t mur_close_files(void)
 		 * later (for flushing the sem-id fields). But, these should have negligible penalty as the file system caches the
 		 * disk reads/writes.
 		 */
+		in_rlbk = RLBKEXITING; /* indicate to re-create repl inst file if needed */
 		repl_inst_write(udi->fn, (off_t)0, (sm_uc_ptr_t)&repl_instance, SIZEOF(repl_inst_hdr));
 		assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_dummy_reg));
 		got_ftok = ftok_sem_lock(jnlpool->jnlpool_dummy_reg, TRUE); /* immediate=TRUE */
@@ -946,10 +968,16 @@ boolean_t mur_close_files(void)
 		 * to error out because the semaphore should still exist in the system
 		 */
 		assert(udi->counter_ftok_incremented || jgbl.onlnrlbk || anticipatory_freeze_available);
-		if (!ftok_sem_lock(jnlpool->jnlpool_dummy_reg, FALSE)
-				|| !ftok_sem_release(jnlpool->jnlpool_dummy_reg, udi->counter_ftok_incremented, FALSE))
-			wrn_count++;
+		if (!local_ftok_counter_halted && udi->counter_ftok_incremented)
+		{
+			if (!ftok_sem_lock(jnlpool->jnlpool_dummy_reg, FALSE)
+					|| !ftok_sem_release(jnlpool->jnlpool_dummy_reg, udi->counter_ftok_incremented, FALSE))
+				wrn_count++;
+		}
 	}
+	in_rlbk = RLBKNOTACTIVE;
+	if (FD_INVALID != repl_inst_rlbk_fd) /* No need to check status below as we are exiting */
+		CLOSEFILE_RESET(repl_inst_rlbk_fd, status);    /* resets "fd" to FD_INVALID */
 	if (jgbl.onlnrlbk)
 	{	/* Signal completion */
 		assert(NULL != jnlpool);
