@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2022 Fidelity National Information	*
+ * Copyright (c) 2003-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -143,6 +143,11 @@ GBLREF	int4			strm_index;
 GBLREF	jnl_gbls_t		jgbl;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	uint4			process_id;
+#ifdef DEBUG
+GBLREF	bool			only_usr_jnlpool_flush;
+#endif
+GBLREF int			in_rlbk;
+GBLREF boolean_t		repl_inst_rlbk_fd;
 
 error_def(ERR_CRITSEMFAIL);
 error_def(ERR_DBFILOPERR);
@@ -170,7 +175,7 @@ error_def(ERR_NOTALLJNLEN);
 error_def(ERR_NOTALLREPLON);
 error_def(ERR_ORLBKFRZOVER);
 error_def(ERR_ORLBKFRZPROG);
-error_def(ERR_ORLBKNOV4BLK);
+error_def(ERR_ORLBKDBUPGRDREQ);
 error_def(ERR_ORLBKSTART);
 error_def(ERR_REPLSTATEOFF);
 error_def(ERR_RLBKNOBIMG);
@@ -283,7 +288,7 @@ int4 mur_open_files(boolean_t retry)
 	freeze_status			reg_frz_status;
 	intrpt_state_t			prev_intrpt_state;
 	onln_rlbk_reg_list		*reglist = NULL, *rl, *rl_last, *save_rl, *rl_new;
-	boolean_t			x_lock, wait_for_kip, replinst_file_corrupt = FALSE, inst_requires_rlbk;
+	boolean_t			x_lock, wait_for_kip, replinst_file_corrupt = FALSE, inst_requires_rlbk = FALSE;
 	boolean_t			jnlpool_sem_created;
 	sgmnt_addrs			*tmpcsa;
 	sgmnt_data			*tmpcsd;
@@ -295,13 +300,15 @@ int4 mur_open_files(boolean_t retry)
 	unix_db_info			*udi;
 	char				time_str[CTIME_BEFORE_NL + 2]; /* for GET_CUR_TIME macro */
 	const char			*verbose_ptr;
-	gtmsource_local_ptr_t		gtmsourcelocal_ptr;
+	gtmsource_local_ptr_t		gtmsourcelocal_ptr = NULL;
 	DEBUG_ONLY(int			semval;)
 	DEBUG_ONLY(jnl_buffer_ptr_t	jb;)
 	boolean_t			recov_interrupted;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
+	repl_inst_rlbk_fd = FD_INVALID;
+	in_rlbk = RLBKACTIVE;
 	jnlpool_init_needed = !mur_options.update;
 	if (!retry)
 	{
@@ -454,12 +461,13 @@ int4 mur_open_files(boolean_t retry)
 				TP_CHANGE_REG(rctl->gd);
 				if (jgbl.onlnrlbk)
 				{
-					if (!cs_data->fully_upgraded)
+					if (!cs_data->fully_upgraded && (BLK_ID_32_VER > cs_data->desired_db_format))
 					{
-						gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) ERR_ORLBKNOV4BLK, 4,
+						gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) ERR_ORLBKDBUPGRDREQ, 4,
 							       REG_LEN_STR(gv_cur_region), DB_LEN_STR(gv_cur_region));
 						return FALSE;
 					}
+					/* else TODO: online_specified && V7m version - this should not be a problem */
 					/* Only select epoch_interval values (for timed epochs) which are sane. */
 					if (MAX_EPOCH_INTERVAL >= cs_data->epoch_interval)
 						max_epoch_interval = MAX(cs_data->epoch_interval, max_epoch_interval);
@@ -522,7 +530,8 @@ int4 mur_open_files(boolean_t retry)
 					assertpro(FALSE); /* should not reach here due to rts_error in the above function */
 
 			}
-		}
+		} else
+			csa = NULL;
 		/* For online rollback we need to grab crit on all the regions. But, this has to be done in the ftok order. To
 		 * setup the regions in the ftok order invoke insert_region.  Note that all we need is a list of regions sorted by
 		 * the inode/device numbers. So, a bubble sort would have worked just fine. But, since most of the code uses
@@ -586,6 +595,7 @@ int4 mur_open_files(boolean_t retry)
 					{
 						send_msg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ORLBKREL);
 						gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ORLBKREL);
+						assert(gtmsourcelocal_ptr);
 						release_all_locks(udi, gtmsourcelocal_ptr, gld_db_files, max_reg_total, rl_last);
 						locks_released = TRUE;
 					}
@@ -611,6 +621,7 @@ int4 mur_open_files(boolean_t retry)
 		 */
 		if ((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl))
 		{	/* Validate the journal pool is accessible and the offsets of various structures within it are intact */
+			assert(csa);
 			grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, GRAB_LOCK_ONLY);
 			csa->hold_onto_crit = TRUE;	/* No more unconditional rel_lock() */
 			assert(jnlpool->repl_inst_filehdr->crash); /* since we haven't removed the journal pool */
@@ -623,7 +634,9 @@ int4 mur_open_files(boolean_t retry)
 				JPL_TRACE_PRO(jnlpool->jnlpool_ctl, jnl_pool_write_sleep);
 				SLEEP_USEC(1, FALSE);
 			}
+			DEBUG_ONLY(only_usr_jnlpool_flush = murgbl.repl_standalone;)
 			repl_inst_flush_jnlpool(FALSE, FALSE);
+			DEBUG_ONLY(only_usr_jnlpool_flush = false;)
 			assert((0 == jnlpool->jnlpool_ctl->onln_rlbk_pid)
 				|| !is_proc_alive(jnlpool->jnlpool_ctl->onln_rlbk_pid, 0));
 			jnlpool->jnlpool_ctl->onln_rlbk_pid = process_id;
@@ -675,7 +688,7 @@ int4 mur_open_files(boolean_t retry)
 			cs_addrs->nl->onln_rlbk_pid = process_id;
 #			ifdef DEBUG
 			/* ensure that we don't have any pending journal writes or flushes */
-			assert(!JNL_ALLOWED(cs_data) || (NULL != cs_addrs->jnl) && (NULL != cs_addrs->jnl->jnl_buff));
+			assert(!JNL_ALLOWED(cs_data) || ((NULL != cs_addrs->jnl) && (NULL != cs_addrs->jnl->jnl_buff)));
 			if (NULL != cs_addrs->jnl)
 			{
 				jb = cs_addrs->jnl->jnl_buff;
@@ -686,11 +699,11 @@ int4 mur_open_files(boolean_t retry)
 			if (cs_data->kill_in_prog)
 				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) ERR_MUKILLIP, 4, DB_LEN_STR(reg),
 					       LEN_AND_LIT("ONLINE ROLLBACK"));
-			/* Ensure that inhibit_kills is ZERO at this point. This is because, we hold crit at this point and anyone
-			 * who wants to set inhibit_kills need crit. The reason this is important is because t_end/tp_tend has
-			 * logic to restart if inhibit_kills is set to TRUE and we don't want online rollback to restart
+			/* Set inhibit_kills to ZERO because t_end and tp_tend have logic to restart if inhibit_kills is
+			 * NON-ZERO and we don't want online rollback to restart. We can do this because we hold crit at
+			 * this point and anybody else that wants to inhibit kills needs crit.
 			 */
-			assert(0 == cs_addrs->nl->inhibit_kills);
+			cs_addrs->nl->inhibit_kills = 0;
 		}
 		TREF(donot_write_inctn_in_wcs_recover) = FALSE;
 	}
@@ -902,13 +915,12 @@ int4 mur_open_files(boolean_t retry)
 					return FALSE;	/* mur_fopen() would have done the appropriate gtm_putmsg() */
 				if (SS_NORMAL != (jctl->status = mur_fread_eof(jctl, rctl)))
 				{
-					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(9) ERR_JNLBADRECFMT, 3, jctl->jnl_fn_len,
+					gtm_putmsg_csa(CSA_ARG(rctl->csa) VARLSTCNT(9) ERR_JNLBADRECFMT, 3, jctl->jnl_fn_len,
 						       jctl->jnl_fn, jctl->rec_offset, ERR_TEXT, 2,
 						       LEN_AND_LIT("mur_fread_eof failed"));
 					return FALSE;
 				}
-				assert((csa == rctl->csa) || !mur_options.update);
-				assert(!jgbl.onlnrlbk || (csa == &udi->s_addrs));
+				udi = FILE_INFO(rctl->gd);
 				if (jgbl.onlnrlbk && jctl->jfh->crash
 						&& !udi->shm_created && !jctl->jfh->recover_interrupted && !inst_requires_rlbk)
 				{	/* If the journal file is crashed, mur_fread_eof invokes mur_fread_eof_crash to read the
@@ -943,7 +955,9 @@ int4 mur_open_files(boolean_t retry)
 				}
 				if (!is_file_identical((char *)jctl->jfh->data_file_name, (char *)rctl->gd->dyn.addr->fname))
 				{
-					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_DBJNLNOTMATCH, 6, DB_LEN_STR(rctl->gd),
+					csa = &udi->s_addrs;
+					assert((csa == rctl->csa) || !mur_options.update);
+					gtm_putmsg_csa(csa, VARLSTCNT(8) ERR_DBJNLNOTMATCH, 6, DB_LEN_STR(rctl->gd),
 						jctl->jnl_fn_len, jctl->jnl_fn,
 						jctl->jfh->data_file_name_length, jctl->jfh->data_file_name);
 					return FALSE;
