@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -15,9 +15,7 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <errno.h>
-#ifdef USE_POLL
-# include <sys/poll.h>
-#endif
+#include "gtm_poll.h"
 
 #include "gtm_stdio.h"
 #include "gtm_string.h"
@@ -27,7 +25,6 @@
 #include "gtm_fcntl.h"
 #include "gtm_unistd.h"
 #include "gtm_stat.h"
-#include "gtm_select.h"
 #include "gtm_time.h"
 #include "eintr_wrappers.h"
 
@@ -64,11 +61,11 @@
  * # calls to repl_send with unaligned buffer
  * # bytes repl_send called with, distribute into buckets
  * % of input bytes repl_send actually sent, or # bytes actually sent distributed into buckets
- * # calls to select, and timeout distributed into buckets
+ * # calls to poll, and timeout distributed into buckets
  * # calls to send
- * # calls to select that were interrupted (EINTR)
- * # calls to select that were unsuccessful due to system resource shortage (EAGAIN)
- * # calls to select that timed out
+ * # calls to poll that were interrupted (EINTR)
+ * # calls to poll that were unsuccessful due to system resource shortage (EAGAIN)
+ * # calls to poll that timed out
  * # calls to send that were interrupted (EINTR)
  * # calls to send that failed due to the message size being too big (EMSGSIZE)
  * # calls to send that would have blocked (EWOULDBLOCK)
@@ -78,11 +75,11 @@
  * # calls to repl_recv with unaligned buffer
  * # bytes repl_recv called with, distribute into buckets
  * % of input length repl_recv actually received, or # bytes actuall received distributed into buckets
- * # calls to select, and timeout distributed into buckets
+ * # calls to poll, and timeout distributed into buckets
  * # calls to recv
- * # calls to select that were interrupted (EINTR)
- * # calls to select that were unsuccessful due to system resource shortage (EAGAIN)
- * # calls to select that timed out
+ * # calls to poll that were interrupted (EINTR)
+ * # calls to poll that were unsuccessful due to system resource shortage (EAGAIN)
+ * # calls to poll that timed out
  * # calls to recv that were interrupted (EINTR)
  * # calls to recv that failed due to the connection reset (bytes received == 0)
  * # calls to recv that would have blocked (EWOULDBLOCK)
@@ -136,17 +133,17 @@ error_def(ERR_TLSCONNINFO);
 {													\
 	if (IO_SIZE > MAX_TRACE_SIZE)									\
 	{												\
-		memcpy(TRACE_BUFF, IO_BUFF + IO_SIZE - MAX_TRACE_SIZE, MAX_TRACE_SIZE);			\
+		memcpy((void *)TRACE_BUFF, IO_BUFF + IO_SIZE - MAX_TRACE_SIZE, MAX_TRACE_SIZE);		\
 		TRACE_BUFF_POS = 0;									\
 	} else												\
 	{												\
 		int space_to_end = MAX_TRACE_SIZE - TRACE_BUFF_POS;					\
 		if (IO_SIZE > space_to_end)								\
 		{											\
-			memcpy(TRACE_BUFF + TRACE_BUFF_POS, IO_BUFF, space_to_end);			\
-			memcpy(TRACE_BUFF, IO_BUFF + space_to_end, IO_SIZE - space_to_end);		\
+			memcpy((void *)TRACE_BUFF + TRACE_BUFF_POS, IO_BUFF, space_to_end);		\
+			memcpy((void *)TRACE_BUFF, IO_BUFF + space_to_end, IO_SIZE - space_to_end);	\
 		} else											\
-			memcpy(TRACE_BUFF + TRACE_BUFF_POS, IO_BUFF, IO_SIZE);				\
+			memcpy((void *)TRACE_BUFF + TRACE_BUFF_POS, IO_BUFF, IO_SIZE);			\
 		TRACE_BUFF_POS = (TRACE_BUFF_POS + IO_SIZE) % MAX_TRACE_SIZE;				\
 	}												\
 }
@@ -154,31 +151,12 @@ error_def(ERR_TLSCONNINFO);
 int fd_ioready(int sock_fd, int poll_direction, int timeout)
 {
 	int		save_errno, status, EAGAIN_cnt = 0, ENOMEM_cnt = 0, REPL_MAXPOLLFAIL_cnt = 0;
-#	ifdef USE_POLL
 	struct pollfd	fds;
-#	else
-	fd_set		fds, *readfds, *writefds;
-	struct timeval	timeout_spec;
-#	endif
 
-	assert(timeout < MILLISECS_IN_SEC);
-	SELECT_ONLY(timeout = timeout * 1000);		/* Convert to microseconds (~ 1sec) */
-	assert((timeout >= 0) && (timeout < POLL_ONLY(MILLISECS_IN_SEC) SELECT_ONLY(MICROSECS_IN_SEC)));
-#	ifdef USE_POLL
+	assert((timeout >= 0) && (timeout < MILLISECS_IN_SEC));
 	fds.fd = sock_fd;
 	fds.events = (REPL_POLLIN == poll_direction) ? POLLIN : POLLOUT;
-#	else
-	readfds = writefds = NULL;
-	timeout_spec.tv_sec = 0;
-	timeout_spec.tv_usec = timeout;
-	assertpro(FD_SETSIZE > sock_fd);
-	FD_ZERO(&fds);
-	FD_SET(sock_fd, &fds);
-	writefds = (REPL_POLLOUT == poll_direction) ? &fds : NULL;
-	readfds = (REPL_POLLIN == poll_direction) ? &fds : NULL;
-#	endif
-	POLL_ONLY(while (-1 == (status = poll(&fds, 1, timeout))))
-	SELECT_ONLY(while (-1 == (status = select(sock_fd + 1, readfds, writefds, NULL, &timeout_spec))))
+	while (-1 == (status = poll(&fds, 1, timeout)))
 	{
 		save_errno = ERRNO;
 		if (EINTR == save_errno)
@@ -190,7 +168,7 @@ int fd_ioready(int sock_fd, int poll_direction, int timeout)
 			if (0 == ++EAGAIN_cnt % REPL_COMM_LOG_EAGAIN_INTERVAL)
 			{
 				repl_log(stderr, TRUE, TRUE, "Communication subsytem warning: System appears to be resource "
-						"starved. EAGAIN returned from select()/poll() %d times\n", EAGAIN_cnt);
+						"starved. EAGAIN returned from poll() %d times\n", EAGAIN_cnt);
 			}
 			rel_quant();	/* this seems legit */
 		} else if (ENOMEM == save_errno)
@@ -198,16 +176,10 @@ int fd_ioready(int sock_fd, int poll_direction, int timeout)
 			if (0 == ++ENOMEM_cnt % REPL_COMM_LOG_ENOMEM_INTERVAL)
 			{
 				repl_log(stderr, TRUE, TRUE, "Communication subsytem warning: No memory available for "
-						"polling. ENOMEM returned from select()/poll() %d times\n", ENOMEM_cnt);
+						"polling. ENOMEM returned from poll() %d times\n", ENOMEM_cnt);
 			}
 			return -1;
 		}
-		/* Restore fd_set. Also, on Linux, select() modifies timeout, so restore timeout to the GT.M controlled timeout */
-		SELECT_ONLY(
-			assert(0 == timeout_spec.tv_sec);
-			timeout_spec.tv_usec = timeout;
-			FD_SET(sock_fd, &fds);
-		)
 		if (REPL_MAXPOLLFAIL < ++REPL_MAXPOLLFAIL_cnt)
 			return -1;
 	}
@@ -341,7 +313,7 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTLS_ONLY_COMMA(int *poll_direction))
 {
 	int		status, max_recv_len, io_ready, save_errno;
-	ssize_t		bytes_recvd;
+	ssize_t		bytes_recvd = 0;
 	char const	*errptr;
 
 	if (!repl_recv_trace_buff)
@@ -376,14 +348,17 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 	{
 		while (TRUE)
 		{
+#ifdef DEBUG
 			if ((WBTEST_ENABLED(WBTEST_REPL_TLS_RECONN)) && is_rcvr_server)
 			{
 				GTM_WHITE_BOX_TEST(WBTEST_REPL_TLS_RECONN, bytes_recvd, -1);
 				if (-1 == bytes_recvd)
 					gtm_wbox_input_test_case_count = 11; /* Do not go to white box again*/
 			}
+#endif
 			bytes_recvd = GTMTLS_ONLY(repl_tls.enabled ? gtm_tls_recv(repl_tls.sock, (char *)buff, max_recv_len)
 							 : ) recv(sock_fd, (char *)buff, max_recv_len, 0);	/* BYPASSOK */
+#ifdef DEBUG
 			if (((WBTEST_ENABLED(WBTEST_FETCHCOMM_ERR)) || (WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO)))
 				 && !(is_rcvr_server || is_src_server))
 			{	/* Induce the test only in mupip rollback process*/
@@ -395,6 +370,7 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 				LONG_SLEEP(5);
 				repl_log(stdout, TRUE, TRUE, "bytes_recvd: %d\n",bytes_recvd);
 			}
+#endif
 			if (0 < bytes_recvd)
 			{
 				*recv_len = (int)bytes_recvd;
@@ -464,12 +440,14 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 								&& !(is_rcvr_server || is_src_server))
 		{
 			repl_log(stdout, TRUE, TRUE, "Changing save_errno\n");
+#ifdef DEBUG
 			GTM_WHITE_BOX_TEST(WBTEST_FETCHCOMM_ERR, save_errno, ECONNRESET);
 			if (WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO) && (-1 == bytes_recvd))
 			{
 				save_errno = ECONNRESET;
 				gtm_wbox_input_test_case_count = 9; /* Do not go to white box again*/
 			}
+#endif
 		}
 		return save_errno;
 	} else if (!io_ready)	/* Select call timedout with no errors */
@@ -607,10 +585,10 @@ void repl_log_tls_info(FILE *logfp, gtm_tls_socket_t *socket)
 	struct tm		*localtm;
 	gtm_tls_conn_info	conn_info;
 
-	if (0 != gtm_tls_get_conn_info(repl_tls.sock, &conn_info))
+	if (0 != gtm_tls_get_conn_info(socket, &conn_info))
 	{
 		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TLSCONNINFO, 0,
-				ERR_TEXT, 2, LEN_AND_STR(gtm_tls_get_error(repl_tls.sock)));
+				ERR_TEXT, 2, LEN_AND_STR(gtm_tls_get_error(socket)));
 		return;
 	}
 	repl_log(logfp, FALSE, TRUE, "TLS/SSL Session:\n");
@@ -628,7 +606,9 @@ void repl_log_tls_info(FILE *logfp, gtm_tls_socket_t *socket)
 		expiry = asctime(localtm);	/* BYPASSOK to prevent workcheck wanting to convert *time(..) to GTM_*TIME(..)*/
 	}
 	repl_log(logfp, FALSE, TRUE, "  Session Expiry: %s", expiry);
-	repl_log(logfp, FALSE, TRUE, "  Secure Renegotiation %s supported\n", conn_info.secure_renegotiation ? "IS" : "IS NOT");
+	if (gtm_tls_does_renegotiate(socket))	/* Announce only if renegotiation is supported */
+		repl_log(logfp, FALSE, TRUE, "  Secure Renegotiation %s supported\n",
+				(conn_info.secure_renegotiation) ? "IS" : "IS NOT");
 	repl_log(logfp, FALSE, TRUE, "Peer Certificate Information:\n");
 	repl_log(logfp, FALSE, TRUE, "  Asymmetric Algorithm: %s (%d bit)\n", conn_info.cert_algo, conn_info.cert_nbits);
 	repl_log(logfp, FALSE, TRUE, "  Subject: %s\n", conn_info.subject);
@@ -709,12 +689,51 @@ int repl_do_tls_handshake(FILE *logfp, int sock_fd, boolean_t do_accept, int *po
 		return status;
 	} else
 	{	/* TLS/SSL handshake succeeded. Log information about the peer's certificate and the TLS/SSL connection. */
-		repl_log(logfp, TRUE, TRUE, "Secure communication enabled using TLS/SSL protocol\n");
-		repl_log_tls_info(logfp, repl_tls.sock);
+		if (!gtm_tls_has_post_hand_shake(repl_tls.sock))
+			repl_log_tls_info(logfp, repl_tls.sock);
 		return SS_NORMAL;
 	}
 	assert(SS_NORMAL != save_errno);
 	repl_errno = do_accept ? EREPL_RECV : EREPL_SEND;
 	return save_errno;
 }
+
+int repl_do_tls_post_handshake(FILE *logfp, int sock_fd)
+{
+	int		io_ready, save_errno, status;
+	const char	*errp;
+	struct pollfd	fds;
+
+	assert(REPL_TLS_REQUESTED);
+	assert(NULL != tls_ctx);
+	assert(NULL != repl_tls.sock);
+	assert(!repl_tls.enabled);
+	/* Do the post handshake auth */
+	if (0 != (status = gtm_tls_do_post_hand_shake(repl_tls.sock)))
+	{
+		while ((GTMTLS_WANT_READ == status) || (GTMTLS_WANT_WRITE == status))
+		{	/* Repeat the handshake in order to send the CertificateRequest */
+			if (0 >= (io_ready = fd_ioready(sock_fd, (GTMTLS_WANT_READ == status) ? POLLIN : POLLOUT, REPL_POLL_WAIT)))
+			{
+				if (!io_ready)
+					continue;
+				save_errno = errno;
+				repl_errno = EREPL_SELECT;
+				return save_errno;
+			}
+			status = gtm_tls_repeat_hand_shake(repl_tls.sock);
+		}
+	}
+	if (status)
+	{	/* Post handshake auth failure */
+		errp = gtm_tls_get_error(repl_tls.sock);
+		repl_log(logfp, TRUE, TRUE, "Post-handshake authentication failure: %s\n", errp);
+		repl_errno = EREPL_RECV;
+		if (GTMTLS_OP_PHA_EXT_NOT_RECEIVED & repl_tls.sock->flags)
+			return -2;	/* Source Server did not send the PHA extension */
+		else
+			return -1;
+	}
+	return SS_NORMAL;
+}
 #endif
