@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2007-2021 Fidelity National Information	*
+ * Copyright (c) 2007-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -64,6 +64,7 @@
 #include "add_inter.h"
 #include "wcs_write_in_progress_wait.h"
 #include "gvt_inline.h"
+#include "mu_updwn_ver_inline.h"
 
 /* Include prototypes */
 #include "send_msg.h"
@@ -96,20 +97,6 @@ error_def(ERR_GBLOFLOW);
 error_def(ERR_TEXT);
 error_def(ERR_WCBLOCKED);
 
-/* Set the cr->ondsk_blkver to the csd->desired_db_format */
-#define SET_ONDSK_BLKVER(cr, csd, ctn)											\
-{															\
-	/* Note that even though the corresponding blks_to_uprd adjustment for this cache-record happened in phase1 	\
-	 * while holding crit, we are guaranteed that csd->desired_db_format did not change since then because the 	\
-	 * function that changes this ("desired_db_format_set") waits for all phase2 commits to	complete before 	\
-	 * changing the format. Before resetting cr->ondsk_blkver, ensure db_format in file header did not change in 	\
-	 * between phase1 (inside of crit) and phase2 (outside of crit). This is needed to ensure the correctness of 	\
-	 * the blks_to_upgrd counter.											\
-	 */														\
-	assert((ctn > csd->desired_db_format_tn) || ((ctn == csd->desired_db_format_tn) && (1 == ctn)));		\
-	cr->ondsk_blkver = csd->desired_db_format;									\
-}
-
 #define MAX_CYCLES	2
 
 #define DO_FAST_INTEG_CHECK(old_block, cs_addrs, cs, lcl_ss_ctx, blkid, write_to_snapshot_file)					\
@@ -143,7 +130,6 @@ error_def(ERR_WCBLOCKED);
 GBLREF	boolean_t		block_saved, certify_all_blocks, dse_running, is_src_server, is_updproc, unhandled_stale_timer_pop,
 				write_after_image;
 GBLREF	boolean_t		mu_reorg_nosafejnl;		/* TRUE if NOSAFEJNL explicitly specified */
-GBLREF	boolean_t		mu_reorg_upgrd_dwngrd_in_prog;	/* TRUE if MUPIP REORG UPGRADE/DOWNGRADE is in progress */
 GBLREF	cache_rec_ptr_t		cr_array[((MAX_BT_DEPTH * 2) - 1) * 2]; /* Maximum number of blocks that can be in transaction */
 GBLREF	char			*update_array, *update_array_ptr;
 GBLREF	cw_set_element		cw_set[];
@@ -160,6 +146,7 @@ GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	uint4			mu_reorg_encrypt_in_prog;	/* non-zero if MUPIP REORG is in progress */
 GBLREF	uint4			dollar_tlevel, process_id, update_array_size;
+GBLREF	uint4			mu_upgrade_in_prog;		/* non-zero if MUPIP REORG UPGRADE/DOWNGRADE is in progress */
 GBLREF	unsigned char		cw_set_depth;
 GBLREF	unsigned int		cr_array_index, t_tries;
 GBLREF	volatile boolean_t	in_mutex_deadlock_check;
@@ -175,7 +162,9 @@ void fileheader_sync(gd_region *reg)
 	size_t			flush_len, sync_size, rounded_flush_len;
 	int4			save_errno;
 	unix_db_info		*udi;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	udi = FILE_INFO(reg);
 	csa = &udi->s_addrs;
 	csd = csa->hdr;
@@ -250,10 +239,10 @@ void bm_update(cw_set_element *cs, sm_uc_ptr_t lclmap, boolean_t is_mm)
 	cti->free_blocks -= reference_cnt;
 	change_bmm = FALSE;
 	/* assert that cs->reference_cnt is 0 if we are in MUPIP REORG UPGRADE/DOWNGRADE */
-	assert(!mu_reorg_upgrd_dwngrd_in_prog || !mu_reorg_encrypt_in_prog || (0 == reference_cnt));
+	assert(!mu_upgrade_in_prog || !mu_reorg_encrypt_in_prog || (0 == reference_cnt));
 	/* assert that if cs->reference_cnt is 0, then we are in MUPIP REORG UPGRADE/DOWNGRADE or DSE MAPS or DSE CHANGE -BHEAD
 	 * or MUPIP REORG -TRUNCATE */
-	assert(mu_reorg_upgrd_dwngrd_in_prog || mu_reorg_encrypt_in_prog || dse_running || (0 != reference_cnt)
+	assert(mu_upgrade_in_prog || mu_reorg_encrypt_in_prog || dse_running || (0 != reference_cnt)
 		|| (NULL != csa->nl && process_id == csa->nl->trunc_pid));
 	if (0 < reference_cnt)
 	{	/* Blocks were allocated in this bitmap. Check if local bitmap became full as a result. If so update mastermap. */
@@ -279,9 +268,20 @@ void bm_update(cw_set_element *cs, sm_uc_ptr_t lclmap, boolean_t is_mm)
 			/* Bitmap block should be the only block updated in this transaction. The only exception is if the
 			 * previous cw-set-element is of type gds_t_busy2free (which does not go through bg_update) */
 			assert((1 == cw_set_depth)
-				|| (2 == cw_set_depth) && (gds_t_busy2free == (cs-1)->old_mode));
-			if (0 != inctn_detail.blknum_struct.blknum)
+				|| ((2 == cw_set_depth) && (gds_t_busy2free == (cs-1)->old_mode)));
+			/* When deleting pre-V7m index blocks, decrement blks_to_upgrd */
+			if (!mu_upgrade_in_prog && (0 != inctn_detail.blknum_struct.blknum)
+					&& (!cs_data->fully_upgraded && (GDSV6 < cs_data->desired_db_format))
+					&& (GDSV7m > (cs-1)->ondsk_blkver) && !(cs-1)->done)
+			{
 				DECR_BLKS_TO_UPGRD(csa, csd, 1);
+#ifdef				DEBUG_BLKS_TO_UPGRD
+				util_out_print("!UL - 0x0 !AD:0x!@XQ:!UL:!UL 0x!@XQ:!UL:!UL", TRUE, cs_data->blks_to_upgrd,
+						REG_LEN_STR(gv_cur_region),
+						&(cs-1)->blk, (cs-1)->ondsk_blkver, (cs-1)->level,
+						&(cs)->blk, (cs)->ondsk_blkver, (cs)->level);
+#endif
+			}
 		}
 	}
 	/* else cs->reference_cnt is 0, this means no free/busy state change in non-bitmap blocks, hence no mastermap change */
@@ -299,7 +299,6 @@ enum cdb_sc mm_update(cw_set_element *cs, trans_num ctn, trans_num effective_tn,
 	block_id		blkid;
 	cw_set_element		*cs_ptr, *nxt;
 	off_chain		chain;
-	v6_off_chain		v6_chain;
 	sm_uc_ptr_t		chain_ptr, db_addr[2];
 	boolean_t		write_to_snapshot_file, long_blk_id;
 	int4			blk_id_sz;
@@ -356,7 +355,7 @@ enum cdb_sc mm_update(cw_set_element *cs, trans_num ctn, trans_num effective_tn,
 	{
 		assert(0 == (blkid & (BLKS_PER_LMAP - 1)));
 		if (FALSE == cs->done)
-			gvcst_map_build((block_id *)cs->upd_addr, db_addr[0], cs, effective_tn);
+			gvcst_map_build(cs->upd_addr.map, db_addr[0], cs, effective_tn);
 		else
 		{	/* It has been built; Update tn in the block and copy from private memory to shared space. */
 			assert(write_after_image);
@@ -367,6 +366,39 @@ enum cdb_sc mm_update(cw_set_element *cs, trans_num ctn, trans_num effective_tn,
 	} else
 	{	/* either it is a non-local bit-map or we are in dse_maps or MUPIP RECOVER writing an AIMG record */
 		assert((0 != (blkid & (BLKS_PER_LMAP - 1))) || write_after_image);
+		if (!cs_data->fully_upgraded)
+		{	/* Transitional DB format */
+			assert(GDSV6 < cs_data->desired_db_format);
+			if (GDSV7m != cs->ondsk_blkver)
+			{
+				if (gds_t_acquired == cs->mode)
+				{	/* Increment blks_to_upgrd for new V6p idx blocks */
+					INCR_BLKS_TO_UPGRD(cs_addrs, cs_data, 1);
+#ifdef					DEBUG_BLKS_TO_UPGRD
+					util_out_print("!UL + 0x!@XQ !AD:0x!@XQ:!UL:!UL:!UL", TRUE, cs_data->blks_to_upgrd, &ctn,
+							REG_LEN_STR(gv_cur_region), &cs->blk, cs->ondsk_blkver,
+							cs->level, cs->mode);
+#endif
+				} else if (!mu_upgrade_in_prog && (0 == cs->level) && (GDSV6 == cs->ondsk_blkver)
+						&& (gds_t_write_recycled != cs->mode) && !cs->done)
+				{	/* Level zero block, update the header and decrement blks_to_upgrd */
+					DECR_BLKS_TO_UPGRD(cs_addrs, cs_data, 1);
+#ifdef					DEBUG_BLKS_TO_UPGRD
+					util_out_print("!UL - 0x!@XQ !AD:0x!@XQ:!UL:!UL:!UL", TRUE, cs_data->blks_to_upgrd, &ctn,
+							REG_LEN_STR(gv_cur_region), &cs->blk, cs->ondsk_blkver,
+							cs->level, cs->mode);
+#endif
+					cs->ondsk_blkver = cs_data->desired_db_format;
+				}
+			} else if ((GDSV7m == cs->ondsk_blkver) && mu_upgrade_in_prog)
+			{					/* Decrement blks_to_upgrd for new V7m blocks */
+				DECR_BLKS_TO_UPGRD(cs_addrs, cs_data, 1);
+#ifdef				DEBUG_BLKS_TO_UPGRD
+				util_out_print("!UL - 0x!@XQ !AD:0x!@XQ:!UL:!UL:!UL", TRUE, cs_data->blks_to_upgrd, &ctn,
+						REG_LEN_STR(gv_cur_region), &cs->blk, cs->ondsk_blkver, cs->level, cs->mode);
+#endif
+			}
+		}
 		if (FALSE == cs->done)
 		{	/* if the current block has not been built (from being referenced in TP) */
 			if (NULL != cs->new_buff)
@@ -381,11 +413,11 @@ enum cdb_sc mm_update(cw_set_element *cs, trans_num ctn, trans_num effective_tn,
 				((blk_hdr_ptr_t)db_addr[0])->tn = ((blk_hdr_ptr_t)cs->new_buff)->tn = ctn;
 			memcpy(db_addr[0], cs->new_buff, ((blk_hdr_ptr_t)cs->new_buff)->bsiz);
 		}
-		long_blk_id = IS_64_BLK_ID(db_addr[0]);
-		blk_id_sz = SIZEOF_BLK_ID(long_blk_id);
 		assert(SIZEOF(blk_hdr) <= ((blk_hdr_ptr_t)db_addr[0])->bsiz);
 		assert((int)(((blk_hdr_ptr_t)db_addr[0])->bsiz) > 0);
 		assert((int)(((blk_hdr_ptr_t)db_addr[0])->bsiz) <= cs_data->blk_size);
+		long_blk_id = IS_64_BLK_ID(db_addr[0]);
+		blk_id_sz = SIZEOF_BLK_ID(long_blk_id);
 		if (!dollar_tlevel)
 		{
 			if (0 != cs->ins_off)
@@ -412,7 +444,7 @@ enum cdb_sc mm_update(cw_set_element *cs, trans_num ctn, trans_num effective_tn,
 			{	/* TP resolve pointer references to new blocks */
 				for (chain_ptr = db_addr[0] + cs->first_off; ; chain_ptr += chain.next_off)
 				{
-					READ_OFF_CHAIN(long_blk_id, &chain, &v6_chain, chain_ptr);
+					READ_OFF_CHAIN(long_blk_id, &chain, chain_ptr);
 					assert(1 == chain.flag);
 					assert((int)(chain_ptr - db_addr[0] + chain.next_off + blk_id_sz)
 							<= (int)(((blk_hdr_ptr_t)db_addr[0])->bsiz));
@@ -467,9 +499,6 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 	SETUP_THREADGBL_ACCESS;
 	csa = cs_addrs;		/* Local access copies */
 	csd = csa->hdr;
-	v7_db_mode = (0 == MEMCMP_LIT(csd->label, GDS_LABEL));
-	v6_db_mode = (0 == MEMCMP_LIT(csd->label, V6_GDS_LABEL));
-	assert((v7_db_mode || v6_db_mode) && (v7_db_mode != v6_db_mode));
 	cnl = csa->nl;
 	assert(csd == cs_data);
 	mode = cs->mode;
@@ -477,6 +506,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 	assert(0 != ctn);
 	assert(csa->now_crit);
 	blkid = cs->blk;
+	twinning_on = TWINNING_ON(csd);
 	/* assert changed to assertpro 2/15/2012. can be changed back once reorg truncate has been running for say 3 to 4 years */
 	assert((0 <= blkid) && (blkid < csa->ti->total_blks));
 	INCR_DB_CSH_COUNTER(csa, n_bgmm_updates, 1);
@@ -499,7 +529,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 		if (NULL == cr)
 		{	/* no cache_rec associated with the block */
 			assert(((gds_t_acquired == mode) && (!read_before_image || (NULL == cs->old_block)))
-					|| (gds_t_acquired != mode) && (NULL != cs->new_buff));
+					|| ((gds_t_acquired != mode) && (NULL != cs->new_buff)));
 			INCR_DB_CSH_COUNTER(csa, n_bg_update_creates, 1);
 			cr = db_csh_getn(blkid);
 #			ifdef DEBUG
@@ -552,7 +582,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 			 *	b) the buffer has already been constructed in private memory (cse->new_buff is non-NULL)
 			 */
 			assert(((gds_t_acquired == mode) && (!read_before_image || (NULL == cs->old_block)))
-					|| (gds_t_acquired != mode) && (NULL != cs->new_buff));
+					|| ((gds_t_acquired != mode) && (NULL != cs->new_buff)));
 			read_finished = wcs_read_in_progress_wait(cr, WBTEST_BG_UPDATE_READINPROGSTUCK1);
 			if (!read_finished)
 			{
@@ -605,7 +635,11 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 		BG_TRACE(new_buff);
 		cr->bt_index = GDS_ABS2REL(bt);
 		bt->cache_index = (int4)GDS_ABS2REL(cr);
-		cr->backup_cr_is_twin = FALSE;
+		/* Avoid writing to cr and causing false sharing if possible. Cost of branch infrequently taken (never in typical
+		 * non-AIO case) should be smaller overall than total cost to all processes of an incrementally busier cacheline.
+		 */
+		if (EXPECT_NOAIO(csd, twinning_on, FALSE))
+			cr->backup_cr_is_twin = FALSE;
 	} else	/* end of if else on cr NOTVALID */
 	{
 		cr = (cache_rec_ptr_t)GDS_REL2ABS(cr);
@@ -616,10 +650,11 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 		{	/* Wait for another process in bg_update_phase2 to stop overlaying the buffer (possible in case of)
 			 *	a) reuse of a killed block that's still in the cache
 			 *	b) the buffer has already been constructed in private memory (cse->new_buff is non-NULL)
+			 *	c) MUPIP REORG -UPGRADE in process - no TP and does not have a new_buff
 			 */
 			assert(process_id != cr->in_tend);
 			assert(((gds_t_acquired == mode) && (!read_before_image || (NULL == cs->old_block)))
-					|| (gds_t_acquired != mode) && (NULL != cs->new_buff));
+					|| ((gds_t_acquired != mode) && ((NULL != cs->new_buff) || mu_upgrade_in_prog)));
 			intend_finished = wcs_phase2_commit_wait(csa, cr);
 			GTM_WHITE_BOX_TEST(WBTEST_BG_UPDATE_INTENDSTUCK, intend_finished, 0);
 			if (!intend_finished)
@@ -645,9 +680,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 		 */
 		LOCK_BUFF_FOR_UPDATE(cr, n, &cnl->db_latch);
 		assert((LATCH_CONFLICT >= n) && (LATCH_CLEAR <= n));
-		twinning_on = TWINNING_ON(csd);
-		cr->backup_cr_is_twin = FALSE;
-		if (!twinning_on)
+		if (EXPECT_NOAIO(csd, !twinning_on, TRUE))
 		{	/* TWINNING is not active because of asyncio=OFF */
 			if (!OWN_BUFF(n))
 			{
@@ -668,6 +701,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 				wait_for_rip = TRUE;
 		} else
 		{	/* TWINNING is enabled because of asyncio=ON. Create TWIN instead of waiting for write to complete */
+			cr->backup_cr_is_twin = FALSE;
 			if (0 == cr->dirty)		/* Free, move to active queue */
 			{	/* If "asyncio" is turned ON, "wcs_wtfini" is the only one that will reset "cr->dirty". And
 				 * that runs inside crit. Since we hold crit now, wcs_wtfini could not have run after we
@@ -849,7 +883,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 			 *	(b) the buffer has already been constructed in private memory
 			 */
 			assert(((gds_t_acquired == mode) && (!read_before_image || (NULL == cs->old_block)))
-					|| (gds_t_acquired != mode) && (NULL != cs->new_buff));
+					|| ((gds_t_acquired != mode) && (NULL != cs->new_buff)));
 			read_finished = wcs_read_in_progress_wait(cr, WBTEST_BG_UPDATE_READINPROGSTUCK2);
 			if (!read_finished)
 			{
@@ -869,7 +903,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 		 * or writing an AIMG record (possible by either DSE or MUPIP JOURNAL RECOVER),
 		 * or this is a newly created block, or we have an in-memory copy.
 		 */
-		 assert(dse_running || write_after_image || mu_reorg_upgrd_dwngrd_in_prog
+		 assert(dse_running || write_after_image || mu_upgrade_in_prog
 			 || ((gds_t_acquired != mode) ? (0 != cs->new_buff): (!read_before_image || (NULL == cs->old_block))));
 		if (!dollar_tlevel)		/* stuff it in the array before setting in_cw_set */
 		{
@@ -881,7 +915,7 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 	assert(0 == cr->data_invalid);
 	if (0 != cr->r_epid)
 	{	/* must have got it with a db_csh_getn */
-		if (gds_t_acquired != mode)
+		if ((gds_t_acquired != mode) || mu_upgrade_in_prog)
 		{	/* Not a newly created block, yet we have got it with a db_csh_getn. This means we have an in-memory
 			 * copy of the block already built. In that case, cr->ondsk_blkver is uninitialized. Copy it over
 			 * from cs->ondsk_blkver which should hold the correct value.
@@ -895,63 +929,47 @@ enum cdb_sc bg_update_phase1(cw_set_element *cs, trans_num ctn, sgm_info *si)
 		TREF(block_now_locked) = NULL;
 	}
 	/* Update csd->blks_to_upgrd while we have crit */
-	/* cs->ondsk_blkver is what gets filled in the PBLK record header as the pre-update on-disk block format.
-	 * cr->ondsk_blkver is what is used to update the blks_to_upgrd counter in the file-header whenever a block is updated.
-	 * They both better be the same. Note that PBLK is written if "read_before_image" is TRUE and cs->old_block is non-NULL.
-	 * For created blocks that have NULL cs->old_blocks, t_create should have set format to GDSVCURR. Assert that too.
-	 */
-	assert(!read_before_image || (NULL == cs->old_block) || (cs->ondsk_blkver == cr->ondsk_blkver));
-	assert((gds_t_acquired != mode) || (NULL != cs->old_block)
-		|| (v7_db_mode && ((GDSV7 == cs->ondsk_blkver) || (GDSV7m == cs->ondsk_blkver)))
-		|| (v6_db_mode && ((GDSV6 == cs->ondsk_blkver) || (GDSV6p == cs->ondsk_blkver))));
 	desired_db_format = csd->desired_db_format;
 	/* assert that appropriate inctn journal records were written at the beginning of the commit in t_end */
-	assert((inctn_blkupgrd_fmtchng != inctn_opcode) || ((GDSV6 == cr->ondsk_blkver) && (GDSV7m == desired_db_format)));
-	assert((inctn_blkdwngrd_fmtchng != inctn_opcode) || ((GDSV7m == cr->ondsk_blkver) && (GDSV6 == desired_db_format)));
+	assert((inctn_blkupgrd_fmtchng != inctn_opcode) || ((GDSV6p == cr->ondsk_blkver) && (GDSV7m == desired_db_format)));
 	assert(!(JNL_ENABLED(csa) && csa->jnl_before_image) || !mu_reorg_nosafejnl
 		|| (inctn_blkupgrd != inctn_opcode) || (cr->ondsk_blkver == desired_db_format));
-	assert(!mu_reorg_upgrd_dwngrd_in_prog || !mu_reorg_encrypt_in_prog || (gds_t_acquired != mode));
-	/* RECYCLED blocks could be converted by MUPIP REORG UPGRADE/DOWNGRADE. In this case do NOT update blks_to_upgrd */
-	assert((gds_t_write_recycled != mode) || mu_reorg_upgrd_dwngrd_in_prog || mu_reorg_encrypt_in_prog);
-	if (gds_t_acquired == mode)
-	{	/* It is a created block. It should inherit the desired db format. This is done as a part of call to
-		 * SET_ONDSK_BLKVER in bg_update_phase1 and bg_update_phase2.
-		 * Also, if this is a V7 DB and that format is V6, increase blks_to_upgrd.
-		 */
-		if ((GDSV6 == desired_db_format) && v7_db_mode)
+	assert(!mu_upgrade_in_prog || !mu_reorg_encrypt_in_prog || (gds_t_acquired != mode));
+	assert((gds_t_write_recycled != mode) || mu_upgrade_in_prog || mu_reorg_encrypt_in_prog);
+	if (!cs_data->fully_upgraded)
+	{
+		assert(GDSV6 < cs_data->desired_db_format);
+		if (GDSV7m != cs->ondsk_blkver)
 		{
-			INCR_BLKS_TO_UPGRD(csa, csd, 1);
-		}
-	} else if (cr->ondsk_blkver != desired_db_format)
-	{	/* Some sort of state change in the block format is occurring */
-		switch(desired_db_format)
+			if  (gds_t_acquired == mode)
+			{	/* Increment blks_to_upgrd for new V6p idx blocks */
+				INCR_BLKS_TO_UPGRD(csa, csd, 1);	/* Increment blks_to_upgrd for new V6p index blocks */
+#ifdef				DEBUG_BLKS_TO_UPGRD
+				util_out_print("!UL + 0x!@XQ !AD:0x!@XQ:!UL:!UL:!UL", TRUE, cs_data->blks_to_upgrd, &ctn,
+						REG_LEN_STR(gv_cur_region), &cs->blk, cs->ondsk_blkver, cs->level, cs->mode);
+#endif
+			} else if (!mu_upgrade_in_prog && (0 == cs->level) && (GDSV6 == cs->ondsk_blkver)
+					&& (gds_t_write_recycled != cs->mode) && !cs->done)
+			{	/* Level zero block, update the header and decrement blks_to_upgrd */
+				DECR_BLKS_TO_UPGRD(cs_addrs, cs_data, 1);
+#ifdef				DEBUG_BLKS_TO_UPGRD
+				util_out_print("!UL - 0x!@XQ !AD:0x!@XQ:!UL:!UL:!UL", TRUE, cs_data->blks_to_upgrd, &ctn,
+						REG_LEN_STR(gv_cur_region), &cs->blk, cs->ondsk_blkver, cs->level, cs->mode);
+#endif
+				cr->ondsk_blkver = cs->ondsk_blkver = cs_data->desired_db_format;
+			}
+		} else if ((GDSV7m == cs->ondsk_blkver) && mu_upgrade_in_prog)
 		{
-			case GDSV7m:
-				/* V6 -> V7 transition */
-				if (gds_t_write_recycled != mode)
-					if (v7_db_mode)
-						DECR_BLKS_TO_UPGRD(csa, csd, 1);
-					else
-						assert(FALSE);
-				break;
-			case GDSV6:
-				/* V4 -> V5 transition */
-				if (gds_t_write_recycled != mode)
-					if (v7_db_mode)
-						INCR_BLKS_TO_UPGRD(csa, csd, 1);
-					/*else
-						DECR_BLKS_TO_UPGRD(csa, csd, 1);*/
-				break;
-			case GDSV6p:		/* this is only the case for upgrade_dir_tree */
-				DECR_BLKS_TO_UPGRD(csa, csd, 1);
-				break;
-			default:
-				assertpro(FALSE);
+			DECR_BLKS_TO_UPGRD(csa, csd, 1);	/* Decrement in transition block */
+#ifdef			DEBUG_BLKS_TO_UPGRD
+			util_out_print("!UL - 0x!@XQ !AD:0x!@XQ:!UL:!UL:!UL", TRUE, cs_data->blks_to_upgrd, &ctn,
+					REG_LEN_STR(gv_cur_region), &cs->blk, cs->ondsk_blkver, cs->level, cs->mode);
+#endif
 		}
 	}
 	/* generic dse_running variable below is used for caller == dse_maps */
-	assert((gds_t_writemap != mode) || dse_running || mu_reorg_upgrd_dwngrd_in_prog
-		|| cr->twin || (CR_BLKEMPTY == cs->cr->blk) || (cs->cr == cr) && (cs->cycle == cr->cycle));
+	assert((gds_t_writemap != mode) || dse_running || mu_upgrade_in_prog
+		|| cr->twin || (CR_BLKEMPTY == cs->cr->blk) || ((cs->cr == cr) && (cs->cycle == cr->cycle)));
 	/* Before marking this cache-record dirty, record the value of cr->dirty into cr->tn.
 	 * This is used in phase2 to determine "recycled".
 	 */
@@ -1003,7 +1021,6 @@ enum cdb_sc bg_update_phase2(cw_set_element *cs, trans_num ctn, trans_num effect
 {
 	int4			n, blk_id_sz;
 	off_chain		chain;
-	v6_off_chain		v6_chain;
 	sm_uc_ptr_t		blk_ptr, backup_blk_ptr, chain_ptr;
 	cw_set_element		*cs_ptr, *nxt;
 	cache_rec_ptr_t		cr, backup_cr;
@@ -1069,7 +1086,7 @@ enum cdb_sc bg_update_phase2(cw_set_element *cs, trans_num ctn, trans_num effect
 	/* Take backup of block in phase2 (outside of crit). */
 	if (!WAS_FREE(cs->blk_prior_state)) /* don't do before image write for backup for FREE blocks */
 	{
-		if (!cr->backup_cr_is_twin)
+		if (EXPECT_NOAIO(csd, !TWINNING_ON(csd), TRUE) || !cr->backup_cr_is_twin)
 		{
 			backup_cr = cr;
 			backup_blk_ptr = blk_ptr;
@@ -1108,7 +1125,7 @@ enum cdb_sc bg_update_phase2(cw_set_element *cs, trans_num ctn, trans_num effect
 		assert(csa->now_crit);	/* at this point, bitmap blocks are built while holding crit */
 		assert(0 == (blkid & (BLKS_PER_LMAP - 1)));
 		if (FALSE == cs->done)
-			gvcst_map_build((block_id *)cs->upd_addr, blk_ptr, cs, effective_tn);
+			gvcst_map_build(cs->upd_addr.map, blk_ptr, cs, effective_tn);
 		else
 		{	/* It has been built; Update tn in the block and copy from private memory to shared space */
 			assert(write_after_image);
@@ -1130,7 +1147,7 @@ enum cdb_sc bg_update_phase2(cw_set_element *cs, trans_num ctn, trans_num effect
 		 * only exception to this is ONLINE ROLLBACK or MUPIP TRIGGER -UPGRADE which holds crit for the entire duration
 		 */
 		assert(!csa->now_crit || cs->recompute_list_head || dse_running || jgbl.onlnrlbk || TREF(in_trigger_upgrade)
-			|| mu_reorg_upgrd_dwngrd_in_prog);
+			|| mu_upgrade_in_prog);
 		if (FALSE == cs->done)
 		{	/* if the current block has not been built (from being referenced in TP) */
 			if (NULL != cs->new_buff)
@@ -1171,12 +1188,12 @@ enum cdb_sc bg_update_phase2(cw_set_element *cs, trans_num ctn, trans_num effect
 				}
 			}
 		} else
-		{
+		{	/* TP */
 			if (0 != cs->first_off)
 			{	/* TP - resolve pointer references to new blocks */
 				for (chain_ptr = blk_ptr + cs->first_off; ; chain_ptr += chain.next_off)
 				{
-					READ_OFF_CHAIN(long_blk_id, &chain, &v6_chain, chain_ptr);
+					READ_OFF_CHAIN(long_blk_id, &chain, chain_ptr);
 					assert(1 == chain.flag);
 					assert((int)(chain_ptr - blk_ptr + chain.next_off + blk_id_sz)
 							<= (int)((blk_hdr_ptr_t)blk_ptr)->bsiz);
@@ -1309,12 +1326,13 @@ void wcs_timer_start(gd_region *reg, boolean_t io_ok)
 	/* assume defaults for flush_target and buffs_per_flush */
 	flush_target = csd->flush_trigger;
 	buffs_per_flush = 0;
-	if ((0 != csd->epoch_taper) && (0 != cnl->wcs_active_lvl) &&
-			JNL_ENABLED(csd) && (0 != cnl->jnl_file.u.inode) && csd->jnl_before_image)
+	if ((0 != csd->epoch_taper) && (0 != cnl->wcs_active_lvl) && JNL_ENABLED(csd) && (0 != cnl->jnl_file.u.inode) &&
+		 csd->jnl_before_image && (cnl->epoch_taper_last_pid != process_id))
 	{
 		EPOCH_TAPER_IF_NEEDED(csa, csd, cnl, reg, TRUE, buffs_per_flush, flush_target);
+		cnl->epoch_taper_last_pid = process_id;
 	}
-	if ((flush_target  <= cnl->wcs_active_lvl) && !FROZEN_CHILLED(csa))
+	if ((flush_target <= cnl->wcs_active_lvl) && !FROZEN_CHILLED(csa))
 	{	/* Already in need of a good flush */
 		BG_TRACE_PRO_ANY(csa, active_lvl_trigger);
 		wtstart_errno = wcs_wtstart(reg, buffs_per_flush, NULL, NULL);
@@ -1340,10 +1358,11 @@ void wcs_stale(TID tid, int4 hd_len, gd_region **region)
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-	save_region = gv_cur_region;		/* Certain debugging calls expect gv_cur_region to be correct */
-	save_csaddrs = cs_addrs;
-	save_csdata = cs_data;
 	save_jnlpool = jnlpool;
+	save_csdata = cs_data;
+	save_csaddrs = cs_addrs;
+	COMPILER_FENCE(__ATOMIC_ACQUIRE);
+	save_region = gv_cur_region;		/* Certain debugging calls expect gv_cur_region to be correct */
 	check_csaddrs = (NULL == save_region || FALSE == save_region->open) ? NULL : &FILE_INFO(save_region)->s_addrs;
 		/* Save to see if we are in crit anywhere */
 	reg = *region;
@@ -1374,13 +1393,10 @@ void wcs_stale(TID tid, int4 hd_len, gd_region **region)
 		csa->timer = FALSE;		/* No timer set for this region by this process anymore */
 		csa->canceled_flush_timer = TRUE;
 		/* Restore region */
-		if (save_region != gv_cur_region)
-		{
-			gv_cur_region = save_region;
-			cs_addrs = save_csaddrs;
-			cs_data = save_csdata;
-			jnlpool = save_jnlpool;
-		}
+		gv_cur_region = save_region;
+		cs_addrs = save_csaddrs;
+		cs_data = save_csdata;
+		jnlpool = save_jnlpool;
 		return;
 	}
 	BG_TRACE_ANY(csa, stale_timer_pop);
@@ -1422,8 +1438,7 @@ void wcs_stale(TID tid, int4 hd_len, gd_region **region)
 					/* If there is no dirty buffer left in the active queue, then no need for new timer */
 					if (0 == csa->acc_meth.bg.cache_state->cacheq_active.fl)
 						need_new_timer = FALSE;
-				}
-				else
+				} else
 				{	/* We can't flush to the file, but we can flush the journal */
 					jnl_wait(reg);
 				}
@@ -1484,7 +1499,7 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 {
 	blk_hdr_ptr_t		old_block;
 	blk_segment		*bs1, *bs_ptr;
-	boolean_t		new_rec;
+	boolean_t		new_rec = FALSE;
 	/*cache_rec_ptr_t		cr; calculate instead of pass in ?*/
 	char			*va;
 	enum cdb_sc		status;
@@ -1495,7 +1510,7 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 	int			tmp_cmpc;
 	key_cum_value		*kv, *kvhead;			/* kvhead tp only */
 	mstr			value;
-	off_chain		chain1;				/* tp only */
+	block_ref		chain1;				/* tp only */
 	rec_hdr_ptr_t		curr_rec_hdr, next_rec_hdr, rp;
 	sgmnt_addrs		*csa;
 	sm_uc_ptr_t		cp1, buffaddr;
@@ -1514,7 +1529,6 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 	assert(dollar_tlevel ? (NULL != sgm_info_ptr) : (NULL == cse->new_buff));
 	if (dba_bg == csa->hdr->acc_meth)
 	{	/* For BG method, modify history with uptodate cache-record, buffer and cycle information.
-		* Also modify cse->old_block and cse->ondsk_blkver to reflect the updated buffer.
 		* This is necessary in case history contains an older twin cr or a cr which has since been recycled
 		*/
 		if (dollar_tlevel)
@@ -1541,17 +1555,22 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 		bh->cr = cr;
 		bh->cycle = cr->cycle;
 		cse->old_block = (sm_uc_ptr_t)GDS_REL2ABS(cr->buffaddr);
-		cse->ondsk_blkver = cr->ondsk_blkver;
 		/* old_block needs to be repointed to the NEW buffer but the fact that this block was free does not change in this
 		 * entire function. So cse->blk_prior_state's free_status can stay as it is.
 		 */
 		bh->buffaddr = (sm_uc_ptr_t)GDS_REL2ABS(cr->buffaddr);
 	}
 	buffaddr = bh->buffaddr;
+	if ((NULL != cse->old_block) && (cse->ondsk_blkver < ((blk_hdr_ptr_t)cse->old_block)->bver))
+	{	/* Adjust the ondsk_blkver as needed */
+		assert(!cs_data->fully_upgraded);
+		assert(GDSV7m == ((blk_hdr_ptr_t)cse->old_block)->bver);
+		cse->ondsk_blkver = ((blk_hdr_ptr_t)cse->old_block)->bver;
+	}
 	assert(NULL != cse->recompute_list_head);
 	for (kvhead = kv = cse->recompute_list_head; (NULL != kv); kv = kv->next)
 	{
-		pKey = (gv_key *)&kv->keybuf.key;
+		pKey = (gv_key *)&kv->keybuf.nobase;
 		value = kv->value;
 		target_key_size = pKey->end + 1;
 		if (kvhead != kv)
@@ -1616,7 +1635,8 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 			next_rec_shrink = 0;
 		}
 		blk_size = cs_data->blk_size;	/* "blk_size" is also used by the BLK_FINI macro below */
-		blk_fill_size = (blk_size * gv_fillfactor) / 100 - cs_data->reserved_bytes;
+		blk_fill_size = (blk_size * gv_fillfactor) / 100 -
+			(cse->level ? cs_data->i_reserved_bytes : cs_data->reserved_bytes);
 		if ((cur_blk_size + delta) > blk_fill_size)
 		{
 			BG_TRACE_PRO_ANY(csa, recompute_upd_array_blk_split);
@@ -1626,8 +1646,8 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 		if (dollar_tlevel)
 		{
 			assertpro(0 == rc_set_fragment);
-			chain1 = *(off_chain *)&bh->blk_num;
-			assert(0 == chain1.flag);
+			chain1.id = bh->blk_num;
+			assert(0 == chain1.chain.flag);
 			segment_update_array_size = UA_NON_BM_SIZE(cs_data);
 			ENSURE_UPDATE_ARRAY_SPACE(segment_update_array_size);
 		} else
@@ -1647,7 +1667,7 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 			 * array space we have used is for the current (and only) cw_set_element "cse" and hence can reuse space
 			 * by resetting update_array_ptr
 			 */
-			assert(ROUND_UP2((INTPTR_T)update_array, UPDATE_ELEMENT_ALIGN_SIZE) == (INTPTR_T)cse->upd_addr);
+			assert(ROUND_UP2((INTPTR_T)update_array, UPDATE_ELEMENT_ALIGN_SIZE) == (INTPTR_T)cse->upd_addr.ptr);
 			RESET_UPDATE_ARRAY; /* no CHECK_AND_RESET_UPDATE_ARRAY because we are resetting an active update array */
 		}
 		BLK_INIT(bs_ptr, bs1);
@@ -1686,7 +1706,7 @@ enum cdb_sc	t_recompute_upd_array(srch_blk_status *bh, struct cw_set_element_str
 			BG_TRACE_PRO_ANY(csa, recompute_upd_array_blk_fini);
 			return cdb_sc_mkblk;
 		}
-		cse->upd_addr = (unsigned char *)bs1;
+		cse->upd_addr.blk = bs1;
 		cse->done = FALSE;
 		assert(NULL != gv_target);
 	}
