@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -113,6 +113,107 @@ error_def(ERR_CRYPTBADCONFIG);
 error_def(ERR_DBFILERR);
 error_def(ERR_GVPUTFAIL);
 
+void	bg_read_complete(node_local_ptr_t cnl);
+void	bg_read_wait(sgmnt_addrs *csa, node_local_ptr_t cnl);
+
+void bg_read_complete(node_local_ptr_t cnl)
+{
+#	if PTHREAD_MUTEX_ROBUST_SUPPORTED
+	int4			status;
+	intrpt_state_t		prev_intrpt_state;
+
+	DEFER_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+	status = pthread_mutex_lock(&cnl->read_completed_ctl);
+	if (0 != status)
+	{
+		if (EOWNERDEAD == status)
+		{
+#			if PTHREAD_MUTEX_CONSISTENT_SUPPORTED
+			status = pthread_mutex_consistent(&cnl->read_completed_ctl);
+			if (0 != status)
+			{
+				ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_consistent"),
+							CALLFROM, status, 0);
+			}
+#			endif
+		} else
+		{
+			ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_lock"),
+						CALLFROM, status, 0);
+		}
+	}
+	status = pthread_cond_broadcast(&cnl->read_completed);
+	if (0 != status)
+	{
+		pthread_mutex_unlock(&cnl->read_completed_ctl);
+		ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_cond_broadcast"), CALLFROM, status, 0);
+	}
+	status = pthread_mutex_unlock(&cnl->read_completed_ctl);
+	ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+	if (0 != status)
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_unlock"), CALLFROM, status, 0);
+#	endif
+}
+
+void bg_read_wait(sgmnt_addrs *csa, node_local_ptr_t cnl)
+{
+#	if PTHREAD_MUTEX_ROBUST_SUPPORTED
+	int4			status;
+	struct timespec		waketime;
+	intrpt_state_t		prev_intrpt_state;
+
+	status = clock_gettime(CLOCK_REALTIME, &waketime);
+	assert(0 == status);
+	waketime.tv_nsec += 1000 * NANOSECS_IN_USEC; /* Match the old sleep; consider reducing. */
+	if (NANOSECS_IN_SEC <= waketime.tv_nsec)
+	{
+		waketime.tv_sec++;
+		waketime.tv_nsec -= NANOSECS_IN_SEC;
+	}
+	DEFER_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+	status = pthread_mutex_lock(&cnl->read_completed_ctl);
+	csa->in_read_wait = TRUE;
+	if (0 != status)
+	{
+		if (EOWNERDEAD == status)
+		{
+#			if PTHREAD_MUTEX_CONSISTENT_SUPPORTED
+			status = pthread_mutex_consistent(&cnl->read_completed_ctl);
+			if (0 != status)
+			{
+				csa->in_read_wait = FALSE;
+				ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_consistent"),
+						CALLFROM, status, 0);
+			}
+#			endif
+		} else
+		{
+			csa->in_read_wait = FALSE;
+			ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_lock"), CALLFROM, status,
+					0);
+		}
+	}
+	status = pthread_cond_timedwait(&cnl->read_completed, &cnl->read_completed_ctl, &waketime);
+	if ((0 != status) && (ETIMEDOUT != status))
+	{
+		pthread_mutex_unlock(&cnl->read_completed_ctl);
+		csa->in_read_wait = FALSE;
+		ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_cond_timedwait"), CALLFROM, status, 0);
+	}
+	status = pthread_mutex_unlock(&cnl->read_completed_ctl);
+	csa->in_read_wait = FALSE;
+	ENABLE_INTERRUPTS(INTRPT_IN_RIP_NOTIFY, prev_intrpt_state);
+	if (0 != status)
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("pthread_mutex_unlock"), CALLFROM, status, 0);
+#	endif
+}
+
 /**
  * Returns pointer to the global buffer containing block blk.
  *
@@ -134,7 +235,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 	boolean_t		clustered, hold_onto_crit, was_crit, issued_db_init_crypt_warning, sync_needed;
 	int			dummy, exceed_sleep_count, lcnt, ocnt;
 	cw_set_element		*cse;
-	off_chain		chain1;
+	block_ref		chain1;
 	register sgmnt_addrs	*csa;
 	register sgmnt_data_ptr_t	csd;
 	enum db_ver		ondsk_blkver;
@@ -150,9 +251,11 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 	gd_segment		*seg;
 	uint4			buffs_per_flush, flush_target;
 	enc_info_t		*encr_ptr;
+	boolean_t		twinning_on;
 #	ifdef DEBUG
 	cache_rec_ptr_t		cr_lo, bt_cr;
 #	endif
+	unsigned char		tmp_levl;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -164,31 +267,32 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 	csd = csa->hdr;
 	INCR_DB_CSH_COUNTER(csa, n_t_qreads, 1);
 	is_mm = (dba_mm == csd->acc_meth);
+	twinning_on = TWINNING_ON(csd);
 	/* We better hold crit in the final retry (TP & non-TP). Only exception is journal recovery */
-	assert((t_tries < CDB_STAGNATE) || csa->now_crit || mupip_jnl_recover);
+#	ifdef DEBUG
+	assert((t_tries < CDB_STAGNATE) || csa->now_crit || mupip_jnl_recover || mu_reorg_encrypt_in_prog);
 	if (TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped))
 	{
 		assert(!dollar_tlevel);
-#		ifdef DEBUG
 		if (WBTEST_ENABLED(WBTEST_MUINTEG_TQREAD))
 		{
 			util_out_print("Whitebox test : t_qread returns NULL", TRUE);
 			return (sm_uc_ptr_t)NULL;
 		}
-#		endif
 	}
+#	endif
 	if (dollar_tlevel)
 	{
 		assert(sgm_info_ptr);
 		if (0 != sgm_info_ptr->cw_set_depth)
 		{
-			PUT_BLK_ID(&chain1, blk);
-			if (1 == chain1.flag)
+			PUT_BLK_ID(&chain1.id, blk);
+			if (1 == chain1.chain.flag)
 			{
 				assert(sgm_info_ptr->cw_set_depth);
 				assert((SIZEOF(int) * 8) >= CW_INDEX_MAX_BITS);
-				if ((int)chain1.cw_index < sgm_info_ptr->cw_set_depth)
-					tp_get_cw(sgm_info_ptr->first_cw_set, (int)chain1.cw_index, &cse);
+				if ((int)chain1.chain.cw_index < sgm_info_ptr->cw_set_depth)
+					tp_get_cw(sgm_info_ptr->first_cw_set, (int)chain1.chain.cw_index, &cse);
 				else
 				{
 					assert(FALSE == csa->now_crit);
@@ -205,7 +309,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				cse = first_tp_srch_status ? first_tp_srch_status->cse : NULL;
 			}
 			assert(!cse || !cse->high_tlevel);
-			assert(!chain1.flag || cse);
+			assert(!chain1.chain.flag || cse);
 			if (cse)
 			{	/* transaction has modified the sought after block  */
 				if ((gds_t_committed != cse->mode) || (n_gds_t_op < cse->old_mode))
@@ -232,7 +336,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 						 */
 						gvcst_blk_build(cse, (uchar_ptr_t)cse->new_buff, 0);
 						assert(NULL != cse->blk_target);
-						if (!already_built && !chain1.flag)
+						if (!already_built && !chain1.chain.flag)
 						{
 							buffaddr = first_tp_srch_status->buffaddr;
 							cr = first_tp_srch_status->cr;
@@ -272,7 +376,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 					 * have had an old_mode of kill_t_create in which case we would not have come into this
 					 * else block. Assert accordingly.
 					 */
-					assert(!chain1.flag);
+					assert(!chain1.chain.flag);
 					first_tp_srch_status = NULL;	/* do not use any previous srch_hist information */
 				}
 			}
@@ -330,7 +434,8 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 		 * positive value you can represent in a signed value of the same length.
 		 */
 		assert((&FILE_INFO(gv_cur_region)->s_addrs == csa) && (csd == cs_data));
-		assert(!csa->now_crit);
+		DEBUG_ONLY(PUT_BLK_ID(&chain1, blk));
+		assert((!csa->now_crit) || (1 == chain1.chain.flag));
 		rdfail_detail = cdb_sc_blknumerr;
 		return (sm_uc_ptr_t)NULL;
 	}
@@ -338,7 +443,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 	{
 		*cycle = CYCLE_SHRD_COPY;
 		*cr_out = 0;
-		return (sm_uc_ptr_t)(mm_read(blk));
+		return (sm_uc_ptr_t)(mm_read(blk, lcl_blk_free));
 	}
 	was_crit = csa->now_crit;
 	cnl = csa->nl;
@@ -433,11 +538,13 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				/* assume defaults for flush_target and buffs_per_flush */
 				flush_target = csd->flush_trigger;
 				buffs_per_flush = 0;
-				if ((0 != csd->epoch_taper) && (FALSE == gv_cur_region->read_only) && JNL_ENABLED(csd) &&
+				if ((0 != csd->epoch_taper) && (cnl->epoch_taper_last_pid != process_id) &&
+						(FALSE == gv_cur_region->read_only) && JNL_ENABLED(csd) &&
 						(0 != cnl->wcs_active_lvl) && (NOJNL != csa->jnl->channel) &&
 						(0 != cnl->jnl_file.u.inode) && csd->jnl_before_image)
 				{
 					EPOCH_TAPER_IF_NEEDED(csa, csd, cnl, (gd_region *) 0, FALSE, buffs_per_flush, flush_target);
+					cnl->epoch_taper_last_pid = process_id;
 				}
 				if ((flush_target <= cnl->wcs_active_lvl) && (FALSE == gv_cur_region->read_only))
 					JNL_ENSURE_OPEN_WCS_WTSTART(csa, gv_cur_region, buffs_per_flush, NULL, FALSE, dummy_errno);
@@ -546,7 +653,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				assert(0 <= cr->read_in_progress);
 				assert(0 == cr->dirty);
 				/* Only set in cache if read was success */
-				cr->ondsk_blkver = (lcl_blk_free ? csd->desired_db_format : ondsk_blkver);
+				cr->ondsk_blkver = ondsk_blkver;
 				cr->r_epid = 0;
 				RELEASE_BUFF_READ_LOCK(cr);
 				TREF(block_now_locked) = NULL;
@@ -555,8 +662,14 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				assert(was_crit == csa->now_crit);
 				if (reset_first_tp_srch_status)
 					RESET_FIRST_TP_SRCH_STATUS(first_tp_srch_status, cr, *cycle);
+#ifdef				DEBUG_DB_CSH_COUNTER
+				tmp_levl = ((blk_hdr_ptr_t) (buffaddr))->levl;
+				if ((0 < tmp_levl) && (255 != tmp_levl))
+					INCR_DB_CSH_COUNTER(csa, n_idxblk_csh_miss, 1);
+#endif
+				bg_read_complete(cnl);
 				return buffaddr;
-			} else  if (!was_crit && (BAD_LUCK_ABOUNDS > ocnt))
+			} else if (!was_crit && (BAD_LUCK_ABOUNDS > ocnt))
 			{
 				assert(!hold_onto_crit);
 				assert(TRUE == csa->now_crit);
@@ -609,7 +722,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				 * we depend on the the fact that "bg_update" sets cr->bt_index to 0 before incrementing cr->cycle.
 				 * Given that order, cr->bt_index can be guaranteed to be 0 if we read the incremented cycle.
 				 */
-				if (cr->twin && (0 == cr->bt_index))
+				if (twinning_on && cr->twin && (0 == cr->bt_index))
 					break;
 				if (cr->blk != blk)
 					break;
@@ -716,6 +829,11 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				 * corresponds to "blk" passed in. It is crucial to get an accurate value for both the fields
 				 * since "tp_hist" relies on this for its intermediate validation.
 				 */
+#ifdef				DEBUG_DB_CSH_COUNTER
+				tmp_levl = ((blk_hdr_ptr_t) GDS_ANY_REL2ABS(csa, cr->buffaddr))->levl;
+				if ((0 < tmp_levl) && (255 != tmp_levl))
+					INCR_DB_CSH_COUNTER(csa, n_idxblk_csh_hit, 1);
+#endif
 				return (sm_uc_ptr_t)GDS_ANY_REL2ABS(csa, cr->buffaddr);
 			}
 			if (blk != cr->blk)
@@ -812,7 +930,11 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 					 * 					 					 */
 					INCR_GVSTATS_COUNTER(csa, cnl, n_wait_read_long, 1);
 				}
+#				if PTHREAD_MUTEX_ROBUST_SUPPORTED
+				bg_read_wait(csa, cnl);
+#				else
 				wcs_sleep(lcnt);
+#				endif
 				sleep_invoked = TRUE;
 				exceed_sleep_count++;
 			}
