@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -69,6 +69,7 @@ GBLREF	gd_region		*gv_cur_region;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	uint4			dollar_tlevel;
 GBLREF	uint4			gtmDebugLevel;
+GBLREF	uint4			mu_upgrade_in_prog;
 GBLREF	uint4			process_id;
 GBLREF	unsigned int		cr_array_index;
 GBLREF	volatile boolean_t	in_wcs_recover;	/* TRUE if in "wcs_recover" */
@@ -119,7 +120,7 @@ void wcs_recover(gd_region *reg)
 	sgmnt_data_ptr_t	csd;
 	node_local_ptr_t	cnl;
 	int4			dummy_errno, blk_size;
-	uint4			jnl_status, epid, r_epid;
+	uint4			jnl_status, epid = 0, r_epid;
 	int4			bt_buckets;
 	inctn_opcode_t		save_inctn_opcode;
 	unsigned int		bplmap, lcnt, total_rip_wait;
@@ -149,7 +150,7 @@ void wcs_recover(gd_region *reg)
 	TP_CHANGE_REG(reg);		/* which are needed by called routines such as wcs_wtstart and wcs_mm_recover */
 	if (dba_mm == reg->dyn.addr->acc_meth)	 /* MM uses wcs_recover to remap the database in case of a file extension */
 	{
-		wcs_mm_recover(reg);
+		wcs_mm_recover(reg, 0);
 		TP_CHANGE_REG(save_reg);
 		jnlpool = save_jnlpool;
 		TREF(wcs_recover_done) = TRUE;
@@ -213,6 +214,10 @@ void wcs_recover(gd_region *reg)
 		 * in case there are orphaned phase2 jnl writes still lying around. Take this opportunity to do that.
 		 */
 		jnl_phase2_cleanup(csa, jbp);
+	} else
+	{
+		jpc = NULL;
+		jbp = NULL;
 	}
 	asyncio = csd->asyncio;
 	twinning_on = TWINNING_ON(csd);
@@ -321,6 +326,7 @@ void wcs_recover(gd_region *reg)
 					 */
 					assert(WBTEST_ENABLED(WBTEST_CRASH_SHUTDOWN_EXPECTED)
 						|| WBTEST_ENABLED(WBTEST_MURUNDOWN_KILLCMT06));
+					assert(epid);
 					if (!((0 == r_epid) || (epid == r_epid)))
 					{
 						send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_INVALIDRIP, 2, DB_LEN_STR(reg));
@@ -599,8 +605,7 @@ void wcs_recover(gd_region *reg)
 		{	/* Whichever cache record (previous or current one) has cr->epid non-zero is the WIP one.
 			 * The other is the more recent one.
 			 */
-			assert(asyncio);
-			assert(twinning_on);
+			assert(WBTEST_ENABLED(WBTEST_EXPECT_IO_HANG) || (asyncio && twinning_on));
 			cr_alt = (cache_rec_ptr_t)GDS_ANY_REL2ABS(csa, bt->cache_index);
 			/* These two crs should be twins so one of them should have epid set. Only exception is a kill -9
 			 * in a small window in wcs_wtstart after LOCK_BUFF_FOR_WRITE but before csr->epid = process_id.
@@ -672,6 +677,7 @@ void wcs_recover(gd_region *reg)
 	if (!TREF(donot_write_inctn_in_wcs_recover) && JNL_ENABLED(csd))
 	{
 		assert(&FILE_INFO(jpc->region)->s_addrs == csa);
+		assert(jpc);
 		if (!jgbl.dont_reset_gbl_jrec_time)
 		{
 			SET_GBL_JREC_TIME; /* needed for jnl_ensure_open, jnl_write_pini and jnl_write_inctn_rec */
@@ -719,7 +725,7 @@ void wcs_recover(gd_region *reg)
 }
 
 #ifdef MM_FILE_EXT_OK
-void	wcs_mm_recover(gd_region *reg)
+void	wcs_mm_recover(gd_region *reg, unsigned int orig_vbn_offset)
 {
 	int			save_errno;
 	gtm_uint64_t		mmap_sz;
@@ -751,7 +757,8 @@ void	wcs_mm_recover(gd_region *reg)
 	cs_addrs->db_addrs[0] = NULL;
 	syscall = MEM_UNMAP_SYSCALL;
 #	ifdef _AIX
-	status = shmdt(old_db_addrs[0] - BLK_ZERO_OFF(cs_data->start_vbn));
+	assert(!mu_upgrade_in_prog || (0 == orig_vbn_offset) || (mu_upgrade_in_prog && (cs_data->start_vbn > orig_vbn_offset)));
+	status = shmdt(old_db_addrs[0] - BLK_ZERO_OFF(cs_data->start_vbn - orig_vbn_offset));
 #	else
 	status = (INTPTR_T)munmap((caddr_t)old_db_addrs[0], (size_t)(old_db_addrs[1] - old_db_addrs[0]));
 #	endif
