@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2004 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -18,19 +19,14 @@
 #include "cmmdef.h"
 #include "gvcmz.h"
 
+error_def(ERR_UNSOLCNTERR);
+
 void gvcmz_neterr_set(struct CLB *c)
 {
-	error_def(ERR_UNSOLCNTERR);
-
-	VMS_ONLY(
-		if (c->ios.status == MSG$_CONNECT || c->ios.status == MSG$_INTMSG || c->ios.status == MSG$_CONFIRM)
-		        return;
-	)
-
 	if (((link_info*)(c->usr))->lck_info  & (REMOTE_ZALLOCATES | REMOTE_LOCKS | LREQUEST_SENT | ZAREQUEST_SENT))
 	{
 		((link_info *)(c->usr))->neterr = TRUE;
-		rts_error(VARLSTCNT(1) ERR_UNSOLCNTERR);
+		RTS_ERROR_ABT(VARLSTCNT(1) ERR_UNSOLCNTERR);
 	}
 	return;
 }
