@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -55,12 +55,13 @@
 #define CONDSTK_MAX_STACK	512	/* Actual max is approx 504 due to arithmetic progression */
 #define CONDSTK_RESERVE		4	/* Reserve 4 frames for when process_exiting */
 
-#define CONDITION_HANDLER(name)	ch_ret_type name(int arg)
+#define CONDITION_HANDLER(name)	ch_ret_type name(ch_arg_type arg)
 
 /* Count of arguments the TPRETRY error will make available for tp_restart to use */
 #define TPRESTART_ARG_CNT 6
 
 typedef void	ch_ret_type;
+typedef int	ch_arg_type;
 
 /* Note that the condition_handler structure layout is relied upon by assembly code (see chnd_size, chnd_* in error.si).
  * Any changes here need corresponding changes in error.si.
@@ -74,7 +75,7 @@ typedef struct condition_handler_struct
 								 * simple (it keeps track of offsets of members in structures
 								 * and we dont want it to be conditional on PRO vs DBG).
 								 */
-	ch_ret_type			(*ch)();		/* Condition handler address */
+	ch_ret_type			(*ch)(ch_arg_type);	/* Condition handler address */
 	jmp_buf				jmp;			/* setjmp/longjmp buffer associated with ESTABLISH point */
 	intrpt_state_t			intrpt_ok_state;	/* intrpt_ok_state at time of ESTABLISH_RET/ESTABLISH */
 } condition_handler;
@@ -118,11 +119,11 @@ GBLREF int			process_exiting;
 #define FATALFAILURE(X)		(X > 4)
 #define CHANDLER_EXISTS (active_ch >= &chnd[0])
 
-#define MAKE_MSG_WARNING(x)	((x) & ~SEV_MSK | WARNING)
-#define MAKE_MSG_SUCCESS(x)	((x) & ~SEV_MSK | SUCCESS)
-#define MAKE_MSG_ERROR(x)	((x) & ~SEV_MSK | ERROR)
-#define MAKE_MSG_INFO(x)	((x) & ~SEV_MSK | INFO)
-#define MAKE_MSG_SEVERE(x)	((x) & ~SEV_MSK | SEVERE)
+#define MAKE_MSG_WARNING(x)	(((x) & ~SEV_MSK) | WARNING)
+#define MAKE_MSG_SUCCESS(x)	(((x) & ~SEV_MSK) | SUCCESS)
+#define MAKE_MSG_ERROR(x)	(((x) & ~SEV_MSK) | ERROR)
+#define MAKE_MSG_INFO(x)	(((x) & ~SEV_MSK) | INFO)
+#define MAKE_MSG_SEVERE(x)	(((x) & ~SEV_MSK) | SEVERE)
 
 #define ERROR_RTN		error_return
 
@@ -187,7 +188,8 @@ void ch_trace_point() {return;}
  * disable/enable of interrupts this routine can add an assert that interrupts are disabled.
  */
 #define GTM_ASM_ESTABLISH	{	/* So named because gtm_asm_establish does exactly this */		\
-					GBLREF uint4 dollar_tlevel;						\
+					GBLREF uint4	dollar_tlevel;						\
+					GBLREF int	process_exiting;					\
 														\
 					assert(IS_PTHREAD_LOCKED_AND_HOLDER);					\
 					CHTRACEPOINT;								\
@@ -257,50 +259,50 @@ void ch_trace_point() {return;}
 # define ESTABLISH(x, ret)	ESTABLISH_RET(x, ret)
 #else
 # define ESTABLISH(x)		{										\
-					intrpt_state_t		prev_intrpt_state;				\
+					volatile intrpt_state_t		prev_intrpt_state_est;			\
 														\
-					DEFER_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state);			\
+					DEFER_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state_est);		\
 					ESTABLISH_NOJMP(x);							\
 					/* See ESTABLISH_RET macro comment for below assert */			\
-					ctxt->intrpt_ok_state = prev_intrpt_state;				\
+					ctxt->intrpt_ok_state = prev_intrpt_state_est;				\
 					if (0 != setjmp(ctxt->jmp))						\
 					{									\
-						prev_intrpt_state = ctxt->intrpt_ok_state;			\
+						prev_intrpt_state_est = ctxt->intrpt_ok_state;			\
 						REVERT;								\
 						/* See ESTABLISH_RET macro comment for below assert */		\
 						if (!multi_thread_in_use)					\
 						{								\
-							assert(INTRPT_OK_TO_INTERRUPT <= prev_intrpt_state);	\
-							assert(INTRPT_NUM_STATES > prev_intrpt_state);		\
-							intrpt_ok_state = prev_intrpt_state;			\
+							assert(INTRPT_OK_TO_INTERRUPT <= prev_intrpt_state_est);\
+							assert(INTRPT_NUM_STATES > prev_intrpt_state_est);	\
+							intrpt_ok_state = prev_intrpt_state_est;		\
 						}								\
 						return;								\
 					} else									\
-						ENABLE_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state);	\
+						ENABLE_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state_est);	\
 				}
 # define ESTABLISH_NORET(x, did_long_jump)									\
 				{										\
-					intrpt_state_t		prev_intrpt_state;				\
-														\
-					DEFER_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state);			\
-					did_long_jump = FALSE;							\
-					ESTABLISH_NOJMP(x);							\
-					/* See ESTABLISH_RET macro comment for below assert */			\
-					ctxt->intrpt_ok_state = prev_intrpt_state;				\
-					if (0 != setjmp(ctxt->jmp))						\
-					{									\
-						prev_intrpt_state = ctxt->intrpt_ok_state;			\
-						did_long_jump = TRUE;						\
-						/* See ESTABLISH_RET macro comment for below assert */		\
-						assert(INTRPT_OK_TO_INTERRUPT <= prev_intrpt_state);		\
-						assert(INTRPT_NUM_STATES > prev_intrpt_state);			\
-						/* Assert "intrpt_ok_state" and "prev_intrpt_state" are same in	\
-						 * "dbg" but restore "intrpt_ok_state" in "pro" just in case	\
-						 */								\
-						assert(prev_intrpt_state == intrpt_ok_state);			\
-						intrpt_ok_state = prev_intrpt_state;				\
-					} else									\
-						ENABLE_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state);	\
+					intrpt_state_t		prev_intrpt_state_estnr;				\
+					\
+					DEFER_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state_estnr);			\
+					did_long_jump = FALSE;								\
+					ESTABLISH_NOJMP(x);								\
+					/* See ESTABLISH_RET macro comment for below assert */				\
+					ctxt->intrpt_ok_state = prev_intrpt_state_estnr;				\
+					if (0 != setjmp(ctxt->jmp))							\
+					{										\
+						prev_intrpt_state_estnr = ctxt->intrpt_ok_state;			\
+						did_long_jump = TRUE;							\
+						/* See ESTABLISH_RET macro comment for below assert */			\
+						assert(INTRPT_OK_TO_INTERRUPT <= prev_intrpt_state_estnr);		\
+						assert(INTRPT_NUM_STATES > prev_intrpt_state_estnr);			\
+						/* Assert "intrpt_ok_state" and "prev_intrpt_state_estnr" are same in	\
+						 * "dbg" but restore "intrpt_ok_state" in "pro" just in case		\
+						 */									\
+						assert(prev_intrpt_state_estnr == intrpt_ok_state);			\
+						intrpt_ok_state = prev_intrpt_state_estnr;				\
+					} else										\
+						ENABLE_INTERRUPTS(INTRPT_IN_CONDSTK, prev_intrpt_state_estnr);		\
 				}
 
 # define WITH_CH(HANDLER, OP, ERR_OP)										\
@@ -321,6 +323,7 @@ MBSTART {													\
 #endif
 
 #define REVERT			{										\
+					GBLREF	int		process_exiting;				\
 					intrpt_state_t		prev_intrpt_state;				\
 														\
 					assert(IS_PTHREAD_LOCKED_AND_HOLDER);					\
@@ -493,7 +496,7 @@ MBSTART {													\
 #define MDB_START
 
 void stop_image(void);
-void stop_image_conditional_core(void);
+CONDITION_HANDLER(stop_image_conditional_core);
 void stop_image_no_core(void);
 
 #define TERMINATE		{						\
@@ -586,6 +589,7 @@ CONDITION_HANDLER(mupip_upgrade_ch);
 CONDITION_HANDLER(omi_dbms_ch);
 CONDITION_HANDLER(rc_dbms_ch);
 CONDITION_HANDLER(read_source_ch);
+CONDITION_HANDLER(relinkctl_handler);
 CONDITION_HANDLER(source_ch);
 #ifdef GTM_TRIGGER
 CONDITION_HANDLER(gtm_trigger_ch);
