@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -34,9 +34,10 @@
 	(((UINTPTR_T)(DIO_BUFF).aligned + (SIZE)) <= ((UINTPTR_T)(DIO_BUFF).unaligned + (DIO_BUFF).unaligned_size))
 
 /* DIO buffer alignment is needed in almost all cases of writing to the database file. The only exception is if
- * we are about to create the database file AND we should not have opened the fd with O_DIRECT. Take that into account.
+ * we are about to create the database file (note that UPGRADE takes some liberties with an otherwise empty database)
+ * AND we should not have opened the fd with O_DIRECT. Take that into account.
  */
-#define	ASSERT_NO_DIO_ALIGN_NEEDED(UDI)	assert(in_mu_cre_file && !(UDI)->fd_opened_with_o_direct)
+#define	ASSERT_NO_DIO_ALIGN_NEEDED(UDI)	assert((in_mu_cre_file || mu_upgrade_in_prog) && !(UDI)->fd_opened_with_o_direct)
 
 #define	DBG_CHECK_DIO_ALIGNMENT(udi, offset, buff, size)							\
 MBSTART {													\
