@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -92,6 +92,7 @@ GBLREF	boolean_t		is_src_server, is_updproc;
 GBLREF	gd_region		*ftok_sem_reg, *gv_cur_region;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
+GBLREF	uint4			mu_upgrade_in_prog;
 GBLREF	uint4			process_id;
 GBLREF	ipcs_mesg		db_ipcs;
 GBLREF	jnl_process_vector	*prc_vec;
@@ -168,7 +169,9 @@ int4 gds_rundown(boolean_t cleanup_udi)
 	node_local_ptr_t	baseDBnl;
 	sgm_info		*si;
 	DEBUG_ONLY(boolean_t	orig_we_are_last_writer = FALSE);
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	jnl_status = 0;
 	reg = gv_cur_region;			/* Local copy */
 	/* Early out for cluster regions
@@ -186,6 +189,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 	udi = FILE_INFO(reg);
 	csa = &udi->s_addrs;
 	csd = csa->hdr;
+	cnl = csa->nl;
 	assert((csa == cs_addrs) && (csd == cs_data));	/* relied upon by "jnl_ensure_open" calls below */
 	if ((reg->open) && (dba_usr == csd->acc_meth))
 	{
@@ -242,6 +246,8 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		udi->semid = udi->ftok_semid = INVALID_SEMID;
 		return EXIT_NRM;
 	}
+	if (mu_upgrade_in_prog)
+		csa->nl->reorg_upgrade_pid = 0;
 	csa->regcnt--;
 	if (csa->regcnt)
 	{	/* There is at least one more region pointing to the same db file as this region.
@@ -262,7 +268,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 	if (err_caught)
 	{
 		REVERT;
-		WITH_CH(gds_rundown_ch, gds_rundown_err_cleanup(have_standalone_access), 0);
+		WITH_CH(gds_rundown_ch, gds_rundown_err_cleanup(have_standalone_access), if(FALSE));
 		ENABLE_INTERRUPTS(INTRPT_IN_GDS_RUNDOWN, prev_intrpt_state);
 		DEBUG_ONLY(ok_to_UNWIND_in_exit_handling = FALSE);
 		return EXIT_ERR;
@@ -284,7 +290,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 	canceled_dbsync_timer = FALSE;
 	CANCEL_DB_TIMERS(reg, csa, canceled_dbsync_timer);
 	we_are_last_user = FALSE;
-	inst_is_frozen = IS_REPL_INST_FROZEN && REPL_ALLOWED(csa->hdr);
+	inst_is_frozen = IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)) && REPL_ALLOWED(csa->hdr);
 	if (FREEZE_LATCH_HELD(csa))
 		rel_latch(&cnl->freeze_latch);
 	if (!csa->persistent_freeze)
@@ -294,7 +300,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		mlk_pvtctl	pctl;
 
 		MLK_PVTCTL_INIT(pctl, reg);
-		REL_LOCK_CRIT(pctl, FALSE);
+		REL_LOCK_CRIT(&pctl, FALSE);
 	}
 	if (!was_crit)
 	{
@@ -310,7 +316,6 @@ int4 gds_rundown(boolean_t cleanup_udi)
 	/* If we have standalone access, then ensure that a concurrent online rollback cannot be running at the same time as it
 	 * needs the access control lock as well. The only expection is we are online rollback and currently running down.
 	 */
-	cnl = csa->nl;
 	onln_rlbk_pid = cnl->onln_rlbk_pid;
 	assert(!have_standalone_access || mupip_jnl_recover || !onln_rlbk_pid || !is_proc_alive(onln_rlbk_pid, 0));
 	if (!have_standalone_access)
@@ -614,10 +619,12 @@ int4 gds_rundown(boolean_t cleanup_udi)
 			 */
 			db_needs_flushing = (cnl->last_wcsflu_tn < csa->ti->curr_tn);
 			if (db_needs_flushing)
+			{
 				if (!FROZEN_CHILLED(csa))
 					wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_SYNC_EPOCH);
 				else
 					jnl_wait(reg);
+			}
 			/* Same as above "wcs_flu" */
 			IF_LIBAIO(aio_shim_destroy(udi->owning_gd);)
 			assert(is_mm || (csd == cs_data));
@@ -638,6 +645,11 @@ int4 gds_rundown(boolean_t cleanup_udi)
 				assert(FALSE);
 				COMPSWAP_UNLOCK(&jbp->io_in_prog_latch, process_id, 0, LOCK_AVAILABLE, 0);
 			}
+			if (jbp->phase2_commit_latch.u.parts.latch_pid == process_id)
+			{
+				FIXUP_JBP_FREE_IF_NEEDED(jbp);
+				COMPSWAP_UNLOCK(&jbp->phase2_commit_latch, process_id, 0, LOCK_AVAILABLE, 0);
+			}
 			/* If we are last writer, it is possible cnl->remove_shm is set to FALSE from the "wcs_flu" call
 			 * above (e.g. we are source server and "wcs_flu" noticed a phase2 commit that need to be cleaned up
 			 * which needs a "wcs_recover" call but that is a no-op for the source server). So check that
@@ -892,6 +904,14 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		assert(!csa->now_crit);
 		rel_crit(reg);
 	}
+#	if PTHREAD_MUTEX_ROBUST_SUPPORTED
+	if (csa->in_read_wait)
+	{	/* Release the read control mutex before detaching shared memory, allowing waiters to retry first. */
+		PTHREAD_COND_BROADCAST(&cnl->read_completed, status);
+		pthread_mutex_unlock(&cnl->read_completed_ctl);
+		csa->in_read_wait = FALSE;
+	}
+#	endif
 	/* Dereferencing nl or hdr+friends after detach is not right; Nullify ahead of the detach operation so that concurrent
 	 * code, e.g. signal handlers, can test before a dereference the occurs in the middle of a detach. */
 	csa->nl = NULL;
