@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -40,8 +40,6 @@
 #include "send_msg.h"
 #include "gt_timer.h"
 #include "mmseg.h"
-#include "gdsblk.h"		/* needed for gds_blk_downgrade.h */
-#include "gds_blk_downgrade.h"	/* for IS_GDS_BLK_DOWNGRADE_NEEDED macro */
 #include "wbox_test_init.h"
 #include "anticipatory_freeze.h"
 /* Include prototypes */
@@ -133,7 +131,7 @@ STATICFNDEF int extend_wait_for_write(unix_db_info *udi, int blk_size, off_t new
 
 int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 {
-	sm_uc_ptr_t		old_base[2], mmap_retaddr;
+	sm_uc_ptr_t		old_base[2] = { NULL, NULL }, mmap_retaddr;
 	boolean_t		was_crit, is_mm;
 	int			fd, result, save_errno, status, to_msg, to_wait, wait_period;
 	DEBUG_ONLY(int		first_save_errno);
@@ -145,7 +143,7 @@ int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 	unix_db_info		*udi;
 	inctn_opcode_t		save_inctn_opcode;
 	block_id		prev_extend_blks_to_upgrd;
-	jnl_private_control	*jpc;
+	jnl_private_control	*jpc = NULL;
 	jnl_buffer_ptr_t	jbp;
 	cache_rec_ptr_t		cr;
 	jnlpool_addrs_ptr_t	local_jnlpool;	/* needed by INST_FREEZE_ON_NOSPC_ENABLED */
@@ -156,7 +154,6 @@ int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 	assert(!IS_DSE_IMAGE);
 	assert((cs_addrs->nl == NULL) || (process_id != cs_addrs->nl->trunc_pid)); /* mu_truncate shouldn't extend file... */
 	assert(!process_exiting);
-	DEBUG_ONLY(old_base[0] = old_base[1] = NULL);
 	assert(!gv_cur_region->read_only);
 	udi = FILE_INFO(gv_cur_region);
 	is_mm = (dba_mm == cs_addrs->hdr->acc_meth);
@@ -263,10 +260,10 @@ int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			if (FROZEN_CHILLED(cs_addrs))
 				DO_CHILLED_AUTORELEASE(cs_addrs, cs_data);
 			assert(FROZEN(cs_data) || !cs_addrs->jnlpool || (cs_addrs->jnlpool == jnlpool));
-			if (!FROZEN(cs_data) && !IS_REPL_INST_FROZEN)
+			if (!FROZEN(cs_data) && !IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)))
 				break;
 			rel_crit(gv_cur_region);
-			while (FROZEN(cs_data) || IS_REPL_INST_FROZEN)
+			while (FROZEN(cs_data) || IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)))
 			{
 				hiber_start(1000);
 				if (FROZEN_CHILLED(cs_addrs) && CHILLED_AUTORELEASE(cs_addrs))
@@ -283,7 +280,7 @@ int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 	} else
 		WAIT_FOR_REGION_TO_UNCHILL(cs_addrs, cs_data);
 	assert(!cs_addrs->jnlpool || (cs_addrs->jnlpool == jnlpool));
-	if (IS_REPL_INST_FROZEN && trans_in_prog)
+	if (IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)) && trans_in_prog)
 	{
 		assert((CDB_STAGNATE <= t_tries) || TREF(in_bm_getfree_gdsfilext));
 		GDSFILEXT_CLNUP;
@@ -441,9 +438,9 @@ int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 		assert(FALSE); /* Should be killed before that */
 	}
 	DEBUG_ONLY(prev_extend_blks_to_upgrd = cs_data->blks_to_upgrd;)
-	/* inctn_detail.blks_to_upgrd_delta holds the increase in "csd->blks_to_upgrd" due to the file extension */
-	inctn_detail.blks2upgrd_struct.blks_to_upgrd_delta =
-		(IS_GDS_BLK_DOWNGRADE_NEEDED(cs_data->desired_db_format) ? new_bit_maps : 0);
+	/* Previously, inctn_detail.blks_to_upgrd_delta hekd the increase in "csd->blks_to_upgrd" due to the file
+	 * extension. Because a downgrade is not possible, extensions do not increase blks_to_upgrd */
+	inctn_detail.blks2upgrd_struct.blks_to_upgrd_delta = 0;
 	if (JNL_ENABLED(cs_data))
 	{
 		save_inctn_opcode = inctn_opcode;
@@ -451,6 +448,7 @@ int4 gdsfilext(block_id blocks, block_id filesize, boolean_t trans_in_prog)
 			inctn_opcode = inctn_gdsfilext_mu_reorg;
 		else
 			inctn_opcode = inctn_gdsfilext_gtm;
+		assert(jpc);
 		if (0 == jpc->pini_addr)
 			jnl_write_pini(cs_addrs);
 		jnl_write_inctn_rec(cs_addrs);
