@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2021 Fidelity National Information	*
+ * Copyright (c) 2006-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -61,8 +61,10 @@
 #include "gtmcrypt.h"
 #include "gtmdbgflags.h"
 #include "anticipatory_freeze.h"
+#ifdef DEBUG
+#include "util.h"
+#endif
 
-#define LSEEK_ERR_STR		"Error in lseek"
 #define READ_ERR_STR		"Error in read"
 #define UNKNOWN_ERR_STR		"Error unknown"
 
@@ -77,13 +79,13 @@
 MBSTART {													\
 	assert(MAX_SEQNO_ADDR <= MAX_SEQNO_EOF_ADDR);								\
 	assert(SKIP_EOF_ADDR_CHECK || (MAX_SEQNO_EOF_ADDR <= CTL->repl_buff->fc->eof_addr));			\
-	assert(ctl->max_seqno <= MAX_SEQNO);									\
-	ctl->max_seqno = MAX_SEQNO;										\
-	assert(ctl->max_seqno_dskaddr <= MAX_SEQNO_ADDR);							\
-	ctl->max_seqno_dskaddr = MAX_SEQNO_ADDR;								\
+	assert(MAX_SEQNO >= CTL->max_seqno);									\
+	CTL->max_seqno = MAX_SEQNO;										\
+	assert(MAX_SEQNO_ADDR >= CTL->max_seqno_dskaddr);							\
+	CTL->max_seqno_dskaddr = MAX_SEQNO_ADDR;								\
 	assert(MAX_SEQNO_EOF_ADDR >= MAX_SEQNO_ADDR);								\
-	assert(ctl->max_seqno_eof_addr <= MAX_SEQNO_EOF_ADDR);							\
-	ctl->max_seqno_eof_addr = MAX_SEQNO_EOF_ADDR;								\
+	assert(MAX_SEQNO_EOF_ADDR >= CTL->max_seqno_eof_addr);							\
+	CTL->max_seqno_eof_addr = MAX_SEQNO_EOF_ADDR;								\
 } MBEND
 
 #define BUNCHING_TIME	(8 * MILLISECS_IN_SEC)
@@ -174,6 +176,13 @@ static	int repl_read_file(repl_buff_t *rb)
 	int			eof_change;
 	uint4			start_addr;
 	uint4			end_addr;
+#	ifdef DEBUG
+	ssize_t			rc;
+	struct timespec		ts1, ts2;
+	uint4			len;
+	char			*jnl_buffer;
+	unsigned long long	lseek_time, pread_time;
+#	endif
 
 	fc = rb->fc;
 	b = &rb->buff[rb->buffindex];
@@ -215,14 +224,29 @@ static	int repl_read_file(repl_buff_t *rb)
 	}
 	start_addr = ROUND_DOWN2(b->readaddr, fc->fs_block_size);
 	end_addr = ROUND_UP2(b->readaddr + b->buffremaining - read_less, fc->fs_block_size);
-	if ((off_t)-1 == lseek(fc->fd, (off_t)start_addr, SEEK_SET))
-	{
-		repl_errno = EREPL_JNLFILESEEK;
-		return (errno);
-	}
-	READ_FILE(fc->fd, b->base + REPL_BLKSIZE(rb) - b->buffremaining - (b->readaddr - start_addr),
-		  end_addr - start_addr, nb);
+	PREAD_FILE(fc->fd, b->base + REPL_BLKSIZE(rb) - b->buffremaining - (b->readaddr - start_addr),
+										end_addr - start_addr, start_addr, nb);
 	status = errno;
+#	ifdef DEBUG
+	if (WBTEST_ENABLED(WBTEST_JNL_PREAD))
+	{
+		len = end_addr - start_addr;
+		jnl_buffer = (char *)malloc(len+1);
+		clock_gettime(CLOCK_REALTIME, &ts1);
+		if ((off_t)-1 == lseek(fc->fd, (off_t)start_addr, SEEK_SET))
+			util_out_print("TEST-E-TIME : lseek() error", TRUE);
+		READ_FILE(fc->fd, jnl_buffer, len, rc);
+		clock_gettime(CLOCK_REALTIME, &ts2);
+		lseek_time = (((ts2.tv_sec - ts1.tv_sec) * NANOSECS_IN_SEC) + ts2.tv_nsec) - ts1.tv_nsec;
+		memset(jnl_buffer, 0, len+1);
+		clock_gettime(CLOCK_REALTIME, &ts1);
+		PREAD_FILE(fc->fd, jnl_buffer, len, start_addr, rc);
+		clock_gettime(CLOCK_REALTIME, &ts2);
+		pread_time = (((ts2.tv_sec - ts1.tv_sec) * NANOSECS_IN_SEC) + ts2.tv_nsec) - ts1.tv_nsec;
+		util_out_print("WBTEST_JNL_PREAD : pread !UL : lseek !UL", TRUE, pread_time, lseek_time);
+		free(jnl_buffer);
+	}
+#	endif
 	if (nb < (b->readaddr - start_addr))
 	{	/* This case means that we didn't read enough bytes to get from the alignment point in the disk file
 		 * to the start of the actual desired read (the offset we did the lseek to above).  This can't happen
@@ -292,9 +316,7 @@ static	int repl_next(repl_buff_t *rb)
 			}
 		} else
 		{
-			if (repl_errno == EREPL_JNLFILESEEK)
-				MEMCPY_LIT(err_string, LSEEK_ERR_STR);
-			else if (repl_errno == EREPL_JNLFILEREAD)
+			if (repl_errno == EREPL_JNLFILEREAD)
 				MEMCPY_LIT(err_string, READ_ERR_STR);
 			else
 				MEMCPY_LIT(err_string, UNKNOWN_ERR_STR);
@@ -524,7 +546,7 @@ static	int update_eof_addr(repl_ctl_element *ctl, int *eof_change)
 	if (is_gdid_gdid_identical(&fc->id, JNL_GDID_PTR(csa)))
 	{
 		new_eof_addr = csa->jnl->jnl_buff->dskaddr;
-		REPL_DPRINT3("Update EOF : New EOF addr from SHM for %s is %u\n", ctl->jnl_fn, new_eof_addr);
+		REPL_DPRINT4("Update EOF : New EOF addr from SHM for %s is %u, was %u\n", ctl->jnl_fn, new_eof_addr, prev_eof_addr);
 	} else
 	{
 		REPL_DPRINT2("Update EOF : New EOF addr will be found from jnl file hdr for %s\n", ctl->jnl_fn);
@@ -1276,8 +1298,8 @@ static	tr_search_state_t do_binary_search(repl_ctl_element *ctl, uint4 lo_addr,
 
 			case TR_FIND_WOULD_BLOCK:
 				assert(mid == ROUND_DOWN(high, REPL_BLKSIZE(rb))); /* must be the last block for this retval */
-				assert(ctl->file_state == JNL_FILE_OPEN || /* file that is yet to grow, or truncated file */
-					ctl->file_state == JNL_FILE_CLOSED && 0 != fc->jfh->prev_recov_end_of_data);
+				assert((ctl->file_state == JNL_FILE_OPEN) /* file that is yet to grow, or truncated file */
+					|| ((ctl->file_state == JNL_FILE_CLOSED) && (0 != fc->jfh->prev_recov_end_of_data)));
 				if (srch_status->seqno != 0)
 				{ /* journal flush yet to be done, or truncated file's end_of_data reached, can't locate seqno */
 					assert(read_seqno > srch_status->seqno);
@@ -1328,7 +1350,8 @@ static	tr_search_state_t position_read(repl_ctl_element *ctl, seq_num read_seqno
 	repl_buff_t		*rb;
 	repl_buff_desc		*b;
 	uint4			lo_addr, hi_addr;
-	tr_search_state_t	found, (*srch_func)();
+	tr_search_state_t	found, (*srch_func)(repl_ctl_element *ctl, uint4 lo_addr, uint4 max_readaddr, seq_num read_seqno,
+							tr_search_status_t *srch_status);
 	tr_search_status_t	srch_status;
 	uint4			willnotbefound_addr = 0, willnotbefound_stop_at = 0;
 	int			eof_change;
@@ -1690,7 +1713,7 @@ static	int read_regions(unsigned char **buff, int *buff_avail,
 					{
 						csa = &FILE_INFO(ctl->reg)->s_addrs;
 						freeaddr = csa->jnl->jnl_buff->rsrv_freeaddr;
-						if ((ctl->repl_buff->fc->eof_addr == freeaddr) || (!JNL_ENABLED(csa->hdr)))
+						if ((ctl->max_seqno_eof_addr == freeaddr) || (!JNL_ENABLED(csa->hdr)))
 						{	/* No more pending updates in the journal file. Next update to the
 							 * journal file will take the seqno jctl->jnl_seqno which will be
 							 * greater than read_jnl_seqno
@@ -1787,9 +1810,9 @@ static	int read_regions(unsigned char **buff, int *buff_avail,
 							ctl->prev->jnl_fn, INT8_PRINT(ctl->prev->max_seqno));
 					REPL_DPRINT3(") and %s (min seqno "INT8_FMT, ctl->jnl_fn, INT8_PRINT(ctl->min_seqno));
 					REPL_DPRINT2(") found while looking for "INT8_FMT"\n", INT8_PRINT(read_jnl_seqno));
-					assert((JNL_FILE_CLOSED == ctl->prev->file_state)
-						&& (ctl->prev->max_seqno < read_jnl_seqno)
-							|| (JNL_FILE_EMPTY == ctl->prev->file_state));
+					assert(((JNL_FILE_CLOSED == ctl->prev->file_state)
+							&& (ctl->prev->max_seqno < read_jnl_seqno))
+						|| (JNL_FILE_EMPTY == ctl->prev->file_state));
 					found = TR_WILL_NOT_BE_FOUND;
 					continue;
 				}
@@ -1923,7 +1946,7 @@ static	int read_regions(unsigned char **buff, int *buff_avail,
 
 int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, boolean_t read_multiple)
 {
-	size_t			read_size, read_state, first_tr_len, tot_tr_len, loopcnt;
+	size_t			read_size, first_tr_len, tot_tr_len, loopcnt;
 	unsigned char		*orig_msgp, seq_num_str[32], *seq_num_ptr;  /* INT8_PRINT */
 	jnlpool_ctl_ptr_t	jctl;
 	gtmsource_local_ptr_t	gtmsource_local;
@@ -1931,6 +1954,7 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 	qw_num			read_addr;
 	gtm_uint64_t		jnlpool_size;
 	boolean_t		file2pool;
+	enum src_read_state	read_state;
 	unsigned int		start_heartbeat;
 	boolean_t		stop_bunching;
 	gtmsource_state_t	gtmsource_state_sav;
@@ -1979,10 +2003,10 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 			tot_tr_len += read_size;
 			REPL_DPRINT5("File read seqno : %llu Tr len : %d Total tr len : %d Maxbufflen : %d\n", read_jnl_seqno - 1,
 					read_size - REPL_MSG_HDRLEN, tot_tr_len, maxbufflen);
-			if (gtmsource_save_read_jnl_seqno < read_jnl_seqno)
+			if ((gtmsource_save_read_jnl_seqno < read_jnl_seqno) && (JNLPOOL_DATA == gtmsource_local->read_algo))
 			{
 				read_addr += read_size;
-				if (jnlpool_size >= (jctl->rsrv_write_addr - read_addr))
+				if ((read_addr >= jctl->contig_addr) && (jnlpool_size >= (jctl->rsrv_write_addr - read_addr)))
 				{	/* No more overflow, switch to READ_POOL.  To avoid the expense of memory barrier
 					 * in jnlpool_hasnt_overflowed(), we use a possibly stale value of rsrv_write_addr
 					 * to check if we can switch back to pool. The consequence is that we may switch
@@ -2065,7 +2089,7 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 			maxbufflen = gtmsource_msgbufsiz - REPL_MSG_HDRLEN;
 		}
 	} while (TRUE);
-	if (file2pool && !gtmsource_local->jnlfileonly)
+	if (file2pool && (JNLPOOL_DATA == gtmsource_local->read_algo))
 	{
 		/* Ahead of the transition to pool, force repl_phase2_cleanup() when write_addr is behind read_addr. This
 		 * condition happens frequently with replicating instances and instances with infrequent updates.
