@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -401,7 +401,6 @@ error_def(ERR_HOSTCONFLICT);
 error_def(ERR_INVOPNSTATSDB);
 error_def(ERR_INVSTATSDB);
 error_def(ERR_JNLBUFFREGUPD);
-error_def(ERR_MMNODYNUPGRD);
 error_def(ERR_NLMISMATCHCALC);
 error_def(ERR_NLRESTORE);
 error_def(ERR_REQRECOV);
@@ -411,6 +410,7 @@ error_def(ERR_STATSDBNOTSUPP);
 error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
 error_def(ERR_VERMISMATCH);
+error_def(ERR_SEMUNDOOVERFLOW);
 
 gd_region *dbfilopn(gd_region *reg)
 {
@@ -424,7 +424,7 @@ gd_region *dbfilopn(gd_region *reg)
 	int			status;
 	boolean_t		raw;
 	boolean_t		open_read_only;
-	int			stat_res, rc, save_errno;
+	int			stat_res, rc, save_errno = 0;
 	sgmnt_addrs		*csa;
 	sgmnt_data		tsdbuff;
 	sgmnt_data_ptr_t        tsd;
@@ -647,7 +647,7 @@ gd_region *dbfilopn(gd_region *reg)
 		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), save_errno);
         }
 	set_gdid_from_stat(&udi->fileid, &buf);
-	if (prev_reg = gv_match(reg))
+	if ((prev_reg = gv_match(reg)))
 	{
 		CLOSEFILE_RESET(udi->fd, rc);	/* resets "udi->fd" to FD_INVALID */
 		free(seg->file_cntl->file_info);
@@ -674,7 +674,7 @@ void dbsecspc(gd_region *reg, sgmnt_data_ptr_t csd, gtm_uint64_t *sec_size)
 	assert(0 == SHMPOOL_SECTION_SIZE % OS_PAGE_SIZE);
 	assert(0 == CACHE_CONTROL_SIZE(csd) % OS_PAGE_SIZE);
 	/* First compute the size based on sections common to both MM and BG */
-	csd->free_space = (BLK_ZERO_OFF(csd->start_vbn) - SIZEOF_FILE_HDR(csd));
+	assert((BLK_ZERO_OFF(csd->start_vbn) - SIZEOF_FILE_HDR(csd)) == csd->free_space);
 	tmp_sec_size = NODE_LOCAL_SPACE(csd) + JNL_SHARE_SIZE(csd) + LOCK_SPACE_SIZE(csd) + SHMPOOL_SECTION_SIZE
 		+ SIZEOF_FILE_HDR(csd);
 	/* Now, add sections specific to MM and BG */
@@ -684,6 +684,66 @@ void dbsecspc(gd_region *reg, sgmnt_data_ptr_t csd, gtm_uint64_t *sec_size)
 	return;
 }
 
+void bg_sync_init(node_local_ptr_t cnl)
+{	/* Initialize mutex/cond for notifying processes of completed reads */
+#	if PTHREAD_MUTEX_ROBUST_SUPPORTED
+	int			status;
+	pthread_mutexattr_t	read_completed_ctl_attr;
+	pthread_condattr_t	read_completed_attr;
+
+	status = pthread_mutexattr_init(&read_completed_ctl_attr);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_mutexattr_init"), CALLFROM, status, 0);
+	}
+	status = pthread_mutexattr_settype(&read_completed_ctl_attr, PTHREAD_MUTEX_ERRORCHECK);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_mutexattr_settype"), CALLFROM, status, 0);
+	}
+	status = pthread_mutexattr_setpshared(&read_completed_ctl_attr, PTHREAD_PROCESS_SHARED);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_mutexattr_setpshared"), CALLFROM, status, 0);
+	}
+#	if PTHREAD_MUTEX_ROBUST_SUPPORTED
+	status = pthread_mutexattr_setrobust(&read_completed_ctl_attr, PTHREAD_MUTEX_ROBUST);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_mutexattr_setrobust"), CALLFROM, status, 0);
+	}
+#	endif
+	status = pthread_mutex_init(&cnl->read_completed_ctl, &read_completed_ctl_attr);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_mutex_init"), CALLFROM, status, 0);
+	}
+	status = pthread_condattr_init(&read_completed_attr);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_condattr_init"), CALLFROM, status, 0);
+	}
+	status = pthread_condattr_setpshared(&read_completed_attr, PTHREAD_PROCESS_SHARED);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_condattr_setpshared"), CALLFROM, status, 0);
+	}
+	status = pthread_cond_init(&cnl->read_completed, &read_completed_attr);
+	if (0 != status)
+	{
+		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
+			LEN_AND_LIT("pthread_cond_init"), CALLFROM, status, 0);
+	}
+#	endif
+}
+
 int db_init(gd_region *reg, boolean_t ok_to_bypass)
 {
 	boolean_t		is_bg, read_only, need_stacktrace, have_standalone_access;
@@ -733,6 +793,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 	node_local_ptr_t	baseDBnl;
 	int			slp_cnt = 0;
 	boolean_t		is_logged = FALSE;
+	boolean_t		got_undo_overflow = FALSE;
 #	ifdef DEBUG
 	int			i;
 	char			*ptr;
@@ -1030,7 +1091,14 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 			if (-1 != status)
 				break;
 			save_errno = errno;
-			assert(!udi->sem_created); /* If we created the semaphore, we should be able to do the semop */
+
+			/* If we got EINVAL, we might have encountered SEM_UNDO overflow */
+			got_undo_overflow = FALSE;
+			if (EINVAL == save_errno)
+				got_undo_overflow = is_sem_undo_overflow(udi->semid, sop, sopcnt);
+
+			/* If we created the semaphore, we should be able to do the semop unless we got SEM_UNDO overflow */
+			assert((!udi->sem_created) || (got_undo_overflow));
 			if ((EAGAIN == save_errno) || (ERANGE == save_errno))
 			{
 				if ((EAGAIN == save_errno) && (NO_SEMWAIT_ON_EAGAIN == TREF(dbinit_max_delta_secs)))
@@ -1083,6 +1151,16 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 				RTS_ERROR(VARLSTCNT(12) ERR_CRITSEMFAIL, 2, DB_LEN_STR(reg), ERR_SYSCALL, 5,
 						RTS_ERROR_LITERAL("semop()"), CALLFROM, save_errno);
 			}
+
+			/* If we get here, either our sem was removed, or we have SEM_UNDO overflow
+			 * If we have overflow, we treat it as fatal since there is nothing to be done
+			 * but we log an informative message */
+			if (got_undo_overflow)
+			{
+				RTS_ERROR(VARLSTCNT(12) ERR_SEMUNDOOVERFLOW, 2, DB_LEN_STR(reg), ERR_SYSCALL, 5,  \
+					RTS_ERROR_LITERAL("semop()"), CALLFROM, save_errno);
+			}
+
 			assert(SEM_REMOVED(save_errno));
 			if ((MAX_ACCESS_SEM_RETRIES - 1) == loopcnt)
 				RTS_ERROR(VARLSTCNT(12) ERR_CRITSEMFAIL, 2, DB_LEN_STR(reg), ERR_SYSCALL, 5,
@@ -1185,7 +1263,8 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		dbsecspc(reg, tsd, &sec_size); 	/* Find db segment size */
 		/* Create new shared memory using IPC_PRIVATE. System guarantees a unique id */
 		GTM_WHITE_BOX_TEST(WBTEST_FAIL_ON_SHMGET, sec_size, GTM_UINT64_MAX);
-		if (-1 == (status_l = udi->shmid = gtm_shmget(IPC_PRIVATE, sec_size, RWDALL | IPC_CREAT, TRUE)))
+		if (-1 == (status_l = udi->shmid = gtm_shmget(IPC_PRIVATE, sec_size, RWDALL | IPC_CREAT, TRUE,
+									DATABASE_FILE, (char*)reg->dyn.addr->fname)))
 		{
 			udi->shmid = (int)INVALID_SHMID;
 			status_l = INVALID_SHMID;
@@ -1307,7 +1386,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		if (-1 == stat_res)
 			RTS_ERROR(VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), errno);
 		csd = csa->hdr = (sgmnt_data_ptr_t)((sm_uc_ptr_t)csa->mlkctl + csa->mlkctl_len);
-		mmap_sz = stat_buf.st_size - BLK_ZERO_OFF(tsd->start_vbn) - csd->free_space;
+		mmap_sz = stat_buf.st_size - BLK_ZERO_OFF(tsd->start_vbn);
 		assert(0 < mmap_sz);
 		CHECK_LARGEFILE_MMAP(reg, mmap_sz); /* can issue rts_error MMFILETOOLARGE */
 #		ifdef _AIX
@@ -1356,6 +1435,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 			cnl->cache_off = -CACHE_CONTROL_SIZE(csd);
 			db_csh_ini(csa);
 			bt_malloc(csa);
+			bg_sync_init(cnl);
 		}
 		db_csh_ref(csa, TRUE);
 		shmpool_buff_init(reg);
