@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -91,7 +91,7 @@ void iott_use(io_desc *iod, mval *pp)
 	struct sigaction	act;
 	struct termios		t;
 	mstr			chset_mstr;
-	gtm_chset_t		temp_chset, old_ochset, old_ichset;
+	gtm_chset_t		temp_chset = -1, old_ochset, old_ichset;
 	uint4			mask_in;
 	unsigned char		ch, len;
 	boolean_t		ch_set;
@@ -107,6 +107,7 @@ void iott_use(io_desc *iod, mval *pp)
 	if (*(pp->str.addr + p_offset) != iop_eol)
 	{
 		if (tt_ptr->mupintr)
+		{
 			if (dollar_zininterrupt)
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(1) ERR_ZINTRECURSEIO);
 			else
@@ -115,6 +116,7 @@ void iott_use(io_desc *iod, mval *pp)
 				tt_ptr->tt_state_save.who_saved = ttwhichinvalid;
 				io_find_mvstent(iod, TRUE);
 			}
+		}
 		status = tcgetattr(tt_ptr->fildes, &t);
 		if (0 != status)
 		{
