@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -150,8 +150,9 @@ STATICFNDEF void mu_rndwn_all_helper(shm_parms *parm_buff, char *fname, int *exi
 		if (SS_NORMAL == *tmp_exit_status)
 		{	/* shm still exists */
 			mu_gv_cur_reg_init();
-			gv_cur_region->dyn.addr->fname_len = strlen(fname);
+			STRNLEN(fname, MAX_FN_LEN, gv_cur_region->dyn.addr->fname_len);
 			STRNCPY_STR(gv_cur_region->dyn.addr->fname, fname, gv_cur_region->dyn.addr->fname_len);
+			gv_cur_region->dyn.addr->fname[gv_cur_region->dyn.addr->fname_len] = '\0';
 			if (mu_rndwn_file(gv_cur_region, FALSE))
 			{	/* If this is a statsdb, "mu_rundwn_file" would have invoked rundown on basedb which would
 				 * in turn invoke rundown on the statsdb. The basedb rundown would print the MUFILRNDWNSUC
@@ -250,8 +251,9 @@ boolean_t validate_db_shm_entry(shm_parms *parm_buff, char *fname, int *exit_sta
 {
 	boolean_t		remove_shmid;
 	file_control		*fc;
-	int			fname_len, save_errno, status, shmid;
+	int			save_errno, status, shmid;
 	node_local_ptr_t	nl_addr;
+	size_t			fname_len;
 	sm_uc_ptr_t		start_addr;
 	struct stat		st_buff;
 	struct shmid_ds		shmstat;
@@ -335,8 +337,9 @@ boolean_t validate_db_shm_entry(shm_parms *parm_buff, char *fname, int *exit_sta
 	} else
 	{
 		mu_gv_cur_reg_init();
-		gv_cur_region->dyn.addr->fname_len = strlen(fname);
+		gv_cur_region->dyn.addr->fname_len = MIN(fname_len, MAX_FN_LEN);
 		STRNCPY_STR(gv_cur_region->dyn.addr->fname, fname, gv_cur_region->dyn.addr->fname_len);
+		gv_cur_region->dyn.addr->fname[gv_cur_region->dyn.addr->fname_len] = '\0';
 		fc = gv_cur_region->dyn.addr->file_cntl;
 		fc->op = FC_OPEN;
 		status = dbfilop(fc);
