@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2021 Fidelity National Information	*
+ * Copyright (c) 2012-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -30,28 +30,30 @@
 #include "muextr.h"
 #include "mu_reorg.h"
 /* Include prototypes */
-#include "t_end.h"
-#include "t_retry.h"
-#include "mupip_reorg.h"
-#include "util.h"
-#include "t_begin.h"
-#include "op.h"
+#include "add_inter.h"
+#include "change_reg.h"
+#include "gdsbml.h"
 #include "gvcst_protos.h"	/* for gvcst_rtsib,gvcst_search prototype */
 #include "gvcst_bmp_mark_free.h"
 #include "gvcst_kill_sort.h"
 #include "gtmmsg.h"
-#include "add_inter.h"
-#include "t_abort.h"
-#include "sleep_cnt.h"
-#include "wcs_sleep.h"
-#include "memcoherency.h"
-#include "gdsbml.h"
+#include "is_proc_alive.h"
 #include "jnl_get_checksum.h"
-#include "t_qread.h"
+#include "op.h"
+#include "memcoherency.h"
+#include "mupip_exit.h"
+#include "mupip_reorg.h"
+#include "sleep_cnt.h"
+#include "t_abort.h"
+#include "t_begin.h"
 #include "t_create.h"
+#include "t_end.h"
+#include "t_qread.h"
+#include "t_retry.h"
 #include "t_write_map.h"
 #include "t_write.h"
-#include "change_reg.h"
+#include "util.h"
+#include "wcs_sleep.h"
 
 GBLREF	boolean_t		mu_reorg_process;
 GBLREF	boolean_t		need_kip_incr;
@@ -63,7 +65,6 @@ GBLREF	gv_key			*gv_currkey, *gv_altkey;
 GBLREF	gv_namehead		*gv_target;
 GBLREF	gv_namehead		*reorg_gv_target;
 GBLREF	inctn_opcode_t		inctn_opcode;
-GBLREF	inctn_opcode_t		inctn_opcode;
 GBLREF	kill_set		*kill_set_tail;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_addrs		*kip_csa;
@@ -79,8 +80,10 @@ GBLREF	unsigned int		t_tries;
 error_def(ERR_DBRDONLY);
 error_def(ERR_GBLNOEXIST);
 error_def(ERR_MAXBTLEVEL);
+error_def(ERR_MUNOFINISH);
 error_def(ERR_MUREORGFAIL);
 error_def(ERR_MUTRUNCNOTBG);
+error_def(ERR_REORGUPCNFLCT);
 
 #define RETRY_SWAP		(0)
 #define ABORT_SWAP		(-1)
@@ -104,10 +107,10 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-	assert(mu_reorg_process);	/* TODO: use upg_mv_block? */
+	assert(mu_reorg_process || upg_mv_block);
 	gv_target = gl_ptr->gvt;
 	gv_target->clue.end = 0;		/* reset clue since reorg action on later globals might have invalidated it */
-	reorg_gv_target->gvname.var_name = gv_target->gvname.var_name;	/* needed by SAVE_ROOTSRCH_ENTRY_STATE */
+	reorg_gv_target->gvname.var_name = gv_target->gvname.var_name;  /* needed by SAVE_ROOTSRCH_ENTRY_STATE */
 	dir_hist_ptr = gv_target->alt_hist;
 	gvt_hist_ptr = &(gv_target->hist);
 	inctn_opcode = inctn_invalid_op;
@@ -115,12 +118,24 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 	{	/* set gv_target/gv_currkey/gv_cur_region/cs_addrs/cs_data to correspond to <globalname,reg> in gl_ptr */
 		gv_target->root = 0;		/* reset root so we recompute it in DO_OP_GVNAME below */
 		DO_OP_GVNAME(gl_ptr);
-	}
+	} /* else V6 to V7 upgrade where gen_hist_for_blk() has already done the name lookup */
 	csa = cs_addrs;
 	cnl = csa->nl;
 	csd = cs_data;	/* keep csd up to date; with MM, cs_data can change, and, dereferencing an older copy, cause SIG-11 */
-	if (gv_cur_region->read_only)			/* TODO: ensure such a check exists early on in upgrade logic */
+	if (gv_cur_region->read_only)
 		return;					/* Cannot proceed for read-only data files */
+	if ((0 == upg_mv_block) && (0 != cs_addrs->nl->reorg_upgrade_pid) && (is_proc_alive(cs_addrs->nl->reorg_upgrade_pid, 0)))
+	{	/* REORG -UPGRADE cannot run concurrently with TRUNCATE and is higher priority. Stop */
+		send_msg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+				LEN_AND_LIT("REORG -TRUNCATE"),
+				LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+				cs_addrs->nl->reorg_upgrade_pid);
+		gtm_putmsg_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+				LEN_AND_LIT("REORG -TRUNCATE"),
+				LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+				cs_addrs->nl->reorg_upgrade_pid);
+		mupip_exit(ERR_MUNOFINISH);
+	}
 	killed_global = FALSE;
 	while (0 == gv_target->root)
 	{	/* Global does not "exist" */
@@ -152,7 +167,7 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 			gv_target->clue.end = 0;
 			if (cdb_sc_normal != (status = gvcst_search(gv_altkey, dir_hist_ptr)))
 			{	/* Assign directory tree path to dir_hist_ptr */
-				assert(t_tries < CDB_STAGNATE);
+				assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 				gv_target->root = save_root;
 				t_retry(status);
 				continue;
@@ -161,7 +176,7 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 			gv_target->clue.end = 0;
 			if (cdb_sc_normal != (gvcst_search(gv_currkey, NULL)))
 			{	/* Assign global variable tree path to gvt_hist_ptr */
-				assert(t_tries < CDB_STAGNATE);
+				assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 				t_retry(status);
 				continue;
 			}
@@ -179,10 +194,10 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 		*root_swap_statistic_ptr += 1;
 		break;
 	}
-	if (DIR_ROOT == upg_mv_block)
-		return;
+	if (0 != upg_mv_block)	/* MUPIP UPGRADE Moving a GVT root from the top down, all done */
+		return;		/* UPGRADE is standalone and does not do REORG's deferred exit check */
 	/* ------------ Swap blocks in branch of directory tree --------- */
-	for (level = 0; (0 == upg_mv_block) && (level <= MAX_BT_DEPTH); level++)
+	for (level = 0; level <= MAX_BT_DEPTH; level++)
 	{
 		t_begin(ERR_MUREORGFAIL, UPDTRNS_DB_UPDATED_MASK);
 		for (;;)
@@ -194,7 +209,7 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 			gv_target->clue.end = 0;
 			if (cdb_sc_normal != (status = gvcst_search(gv_altkey, dir_hist_ptr)))
 			{	/* assign branch path of directory tree into dir_hist_ptr */
-				assert(t_tries < CDB_STAGNATE);
+				assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 				gv_target->root = save_root;
 				t_retry(status);
 				continue;
@@ -225,8 +240,6 @@ void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr, block_id upg_mv_b
 			inctn_opcode = inctn_mu_reorg;
 			assert(1 == kill_set_list.used);
 			need_kip_incr = TRUE;
-			if (!csa->now_crit)
-				WAIT_ON_INHIBIT_KILLS(cnl, MAXWAIT2KILL);
 			DEBUG_ONLY(lcl_t_tries = t_tries);
 			TREF(in_mu_swap_root_state) = MUSWP_DIRECTORY_SWAP;
 			if ((trans_num)0 == (ret_tn = t_end(dir_hist_ptr, NULL, TN_NOT_SPECIFIED)))
@@ -274,10 +287,9 @@ block_id mu_swap_root_blk(glist *gl_ptr, srch_hist *gvt_hist_ptr, srch_hist *dir
 	assert(ABORT_SWAP < free_blk_id);
 	update_trans = UPDTRNS_DB_UPDATED_MASK;
 	inctn_opcode = inctn_mu_reorg;
+	/* There is either a kill/set list OR the code is moving the root block */
 	assert((1 == kill_set_list->used) || ((DIR_ROOT == gv_target->root) && (1 == gv_target->hist.h[1].blk_num)));
 	need_kip_incr = TRUE;
-	if (!cs_addrs->now_crit)
-		WAIT_ON_INHIBIT_KILLS(cs_addrs->nl, MAXWAIT2KILL);
 	DEBUG_ONLY(lcl_t_tries = t_tries);
 	TREF(in_mu_swap_root_state) = MUSWP_INCR_ROOT_CYCLE;
 	assert(!TREF(in_gvcst_redo_root_search));
@@ -296,7 +308,7 @@ block_id mu_swap_root_blk(glist *gl_ptr, srch_hist *gvt_hist_ptr, srch_hist *dir
 	}
 	TREF(in_mu_swap_root_state) = MUSWP_NONE;
 	/* Note that this particular process's csa->root_search_cycle is now behind cnl->root_search_cycle.
-	 * This forces a cdb_sc_gvtrootmod2 restart in gvcst_bmp_mark_free below. TODO: how to handle after refactor???
+	 * This forces a cdb_sc_gvtrootmod2 restart in gvcst_bmp_mark_free below.
 	 */
 	assert(cs_addrs->nl->root_search_cycle > cs_addrs->root_search_cycle);
 	if (0 != upg_mv_block)
@@ -352,7 +364,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	if (NULL == (bmlhist.buffaddr = t_qread(bmlhist.blk_num, (sm_int_ptr_t)&bmlhist.cycle, &bmlhist.cr)))
 	{	/* WARNING: assignment above */
 		assert(0 == upg_mv_block);
-		assert(t_tries < CDB_STAGNATE);
+		assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 		t_retry((enum cdb_sc)rdfail_detail);
 		return RETRY_SWAP;
 	}
@@ -362,11 +374,11 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	maxbitsthismap = (master_bit != (num_local_maps - 1)) ? BLKS_PER_LMAP : (int4)(total_blks - bmlhist.blk_num);
 	free_bit = bm_find_blk(hint_bit, bmlhist.buffaddr + SIZEOF(blk_hdr), maxbitsthismap, &free_blk_recycled);
 	free_blk_id = bmlhist.blk_num + free_bit;
-	assert((0 == upg_mv_block) || (upg_mv_block + 1 == free_blk_id));
+	assert((0 == upg_mv_block) || (upg_mv_block <= free_blk_id));
 	if (DIR_ROOT >= free_blk_id)
 	{	/* Bitmap block 0 and directory tree root block 1 should always be marked busy. */
 		assert(0 == upg_mv_block);
-		assert(t_tries < CDB_STAGNATE);
+		assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 		t_retry(cdb_sc_badbitmap);
 		return RETRY_SWAP;
 	}
@@ -390,7 +402,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 		freeblkhist.blk_num = free_blk_id;
 		if (NULL == (freeblkhist.buffaddr = t_qread(free_blk_id, (sm_int_ptr_t)&freeblkhist.cycle, &freeblkhist.cr)))
 		{
-			assert(t_tries < CDB_STAGNATE);
+			assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 			t_retry((enum cdb_sc)rdfail_detail);
 			return RETRY_SWAP;
 		}
@@ -403,16 +415,17 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	assert(blk_seg_cnt == child_blk_size);
 	if (!BLK_FINI(bs_ptr, bs1))
 	{
-		assert(t_tries < CDB_STAGNATE);
+		assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 		t_retry(cdb_sc_blkmod);
 		return RETRY_SWAP;
 	}
 	tmpcse = &cw_set[cw_set_depth];
 	(free_blk_recycled) ? BIT_SET_RECYCLED_AND_CLEAR_FREE(tmpcse->blk_prior_state)
 			    : BIT_CLEAR_RECYCLED_AND_SET_FREE(tmpcse->blk_prior_state);
-	t_create(free_blk_id, (unsigned char *)bs1, 0, 0, child_blk_lvl);
+	t_create(free_blk_id, bs1, 0, 0, child_blk_lvl);
 	tmpcse->mode = gds_t_acquired;
-	if (!free_blk_recycled || !cs_data->db_got_to_v5_once)
+	tmpcse->ondsk_blkver = ((blk_hdr_ptr_t)child_blk_ptr)->bver; /* Retain block version */
+	if (!free_blk_recycled)
 		tmpcse->old_block = NULL;
 	else
 	{
@@ -444,7 +457,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 		GET_RSIZ(rec_size1, (parent_blk_ptr + curr_offset));
 		if ((parent_blk_size < rec_size1 + curr_offset) || (bstar_rec_size(parent_long_blk_id) > rec_size1))
 		{
-			assert(t_tries < CDB_STAGNATE);
+			assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 			t_retry(cdb_sc_blkmod);
 			return RETRY_SWAP;
 		}
@@ -463,12 +476,11 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 		assert(blk_seg_cnt == parent_blk_size);
 		if (!BLK_FINI(bs_ptr, bs1))
 		{
-			assert(t_tries < CDB_STAGNATE);
+			assert((0 == upg_mv_block) && (t_tries < CDB_STAGNATE));
 			t_retry(cdb_sc_blkmod);
 			return RETRY_SWAP;
 		}
-		t_write(&dir_hist_ptr->h[parent_blk_lvl], (unsigned char *)bs1, 0, 0, parent_blk_lvl, FALSE, TRUE,
-			GDS_WRITE_KILLTN);
+		t_write(&dir_hist_ptr->h[parent_blk_lvl], bs1, 0, 0, parent_blk_lvl, FALSE, TRUE, GDS_WRITE_KILLTN);
 		/* To indicate later snapshot file writing process during fast_integ must write the block to snapshot file */
 		BIT_SET_DIR_TREE(cw_set[cw_set_depth - 1].blk_prior_state);
 	}
@@ -476,7 +488,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	PUT_BLK_ID(update_array_ptr, free_bit);
 	save_cw_set_depth = cw_set_depth; /* Bit maps go on end of cw_set (more fake acquired) */
 	assert(!cw_map_depth);
-	t_write_map(&bmlhist, (uchar_ptr_t)update_array_ptr, curr_tn, 1);
+	t_write_map(&bmlhist, (block_id *)update_array_ptr, curr_tn, 1);
 	cw_map_depth = cw_set_depth;
 	cw_set_depth = save_cw_set_depth;
 	update_array_ptr += SIZEOF(block_id);
