@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2021 Fidelity National Information	*
+ * Copyright (c) 2012-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -76,6 +76,9 @@
 #include "db_write_eof_block.h"
 #include "mvalconv.h"
 
+/* Prototypes */
+#include "is_proc_alive.h"
+
 error_def(ERR_BUFFLUFAILED);
 error_def(ERR_DBFILERR);
 error_def(ERR_DBFSYNCERR);
@@ -84,11 +87,11 @@ error_def(ERR_JNLFLUSH);
 error_def(ERR_MUTRUNCFAIL);
 error_def(ERR_MUTRUNCNOSPACE);
 error_def(ERR_MUTRUNCERROR);
-error_def(ERR_MUTRUNCNOV4);
 error_def(ERR_MUTRUNCNOTBG);
 error_def(ERR_MUTRUNCSSINPROG);
 error_def(ERR_MUTRUNCSUCCESS);
 error_def(ERR_MUTRUNCBACKINPROG);
+error_def(ERR_REORGUPCNFLCT);
 error_def(ERR_TEXT);
 error_def(ERR_TRUNCBACKUPPROG);
 error_def(ERR_TRUNCNOTRUN);
@@ -156,11 +159,7 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_MUTRUNCNOTBG, 2, REG_LEN_STR(gv_cur_region));
 		return TRUE;
 	}
-	if (((GDSVCURR != csd->desired_db_format) && !(BLK_ID_32_VER > csd->desired_db_format)) || (csd->blks_to_upgrd != 0))
-	{
-		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_MUTRUNCNOV4, 2, REG_LEN_STR(gv_cur_region));
-		return TRUE;
-	}
+	assert(csd->fully_upgraded || (0 == csd->blks_to_upgrd) || (BLK_ID_32_VER < csd->desired_db_format));
 	trunc_blocks = (truncate_percent * csa->ti->total_blks / 100);
 	if (csa->ti->free_blocks < trunc_blocks)
 	{
@@ -207,6 +206,18 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 					truncate_percent);
 			return TRUE;
 		}
+		if ((0 != csa->nl->reorg_upgrade_pid) && (is_proc_alive(csa->nl->reorg_upgrade_pid, 0)))
+		{	/* REORG -UPGRADE cannot run concurrently with TRUNCATE and is higher priority. Stop */
+			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+					LEN_AND_LIT("REORG -TRUNCATE"),
+					LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+					csa->nl->onln_rlbk_pid);
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+					LEN_AND_LIT("REORG -TRUNCATE"),
+					LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+					csa->nl->onln_rlbk_pid);
+			return TRUE;
+		}
 		lmap_blk_num = lmap_num * BLKS_PER_LMAP;
 		if (csa->nl->highest_lbm_with_busy_blk >= lmap_blk_num)
 		{
@@ -227,11 +238,16 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 				bmphist.blk_num = lmap_blk_num;
 				bmphist.buffaddr = t_qread(bmphist.blk_num, &bmphist.cycle, &bmphist.cr);
 				lmap_blk_hdr = (blk_hdr_ptr_t)bmphist.buffaddr;
-				if (!(bmphist.buffaddr) || (BM_SIZE(BLKS_PER_LMAP) != lmap_blk_hdr->bsiz))
+				if (NULL == bmphist.buffaddr)
 				{ /* Could not read the block successfully. Retry. */
 					t_retry((enum cdb_sc)rdfail_detail);
 					continue;
 				}
+				if (BM_SIZE(BLKS_PER_LMAP) != lmap_blk_hdr->bsiz)
+				{ /* Lost the buffer. Retry. */
+					t_retry(cdb_sc_blkmod);
+					continue;
+				}
 				lmap_addr = bmphist.buffaddr + SIZEOF(blk_hdr);
 				/* starting from the hint (blk itself), find the first busy or recycled block */
 				blk = bml_find_busy_recycled(blk, lmap_addr, blks_in_lmap, &bml_status);
@@ -274,7 +290,7 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 						t_retry(cdb_sc_lostbmlcr);
 						continue;
 					}
-					t_write_map(&bmphist, (unsigned char *)update_array, curr_tn, 0);
+					t_write_map(&bmphist, (block_id *)update_array, curr_tn, 0);
 					/* Set the opcode for INCTN record written by t_end() */
 					inctn_opcode = inctn_blkmarkfree;
 					if ((trans_num)0 == t_end(&alt_hist, NULL, TN_NOT_SPECIFIED))
@@ -309,12 +325,17 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 			/* Read the nth local bitmap into memory */
 			blkhist->buffaddr = t_qread(lmap_blk_num, (sm_int_ptr_t)&blkhist->cycle, &blkhist->cr);
 			lmap_blk_hdr = (blk_hdr_ptr_t)blkhist->buffaddr;
-			if (!(blkhist->buffaddr) || (BM_SIZE(BLKS_PER_LMAP) != lmap_blk_hdr->bsiz))
+			if (NULL == blkhist->buffaddr)
 			{ /* Could not read the block successfully. Retry. */
 				t_retry((enum cdb_sc)rdfail_detail);
 				continue;
 			}
-			t_write_map(blkhist, (unsigned char *)blkid_ptr, curr_tn, 0);
+			if (BM_SIZE(BLKS_PER_LMAP) != lmap_blk_hdr->bsiz)
+			{ /* Lost the buffer. Retry */
+				t_retry(cdb_sc_blkmod);
+				continue;
+			}
+			t_write_map(blkhist, blkid_ptr, curr_tn, 0);
 			blkhist->blk_num = 0; /* create empty history for bitmap block */
 			if ((trans_num)0 == t_end(&alt_hist, NULL, TN_NOT_SPECIFIED))
 				continue;
@@ -328,10 +349,10 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 		grab_crit(gv_cur_region, WS_88);
 		if (FROZEN_CHILLED(cs_addrs))
 			DO_CHILLED_AUTORELEASE(csa, cs_data);
-		if (!FROZEN(cs_data) && !IS_REPL_INST_FROZEN)
+		if (!FROZEN(cs_data) && !IS_REPL_INST_FROZEN(RECOGNIZE_FREEZES))
 			break;
 		rel_crit(gv_cur_region);
-		while (FROZEN(cs_data) || IS_REPL_INST_FROZEN)
+		while (FROZEN(cs_data) || IS_REPL_INST_FROZEN(RECOGNIZE_FREEZES))
 		{
 			hiber_start(1000);
 			if (FROZEN_CHILLED(cs_addrs) && CHILLED_AUTORELEASE(cs_addrs))
@@ -351,6 +372,7 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 	csa->nl->highest_lbm_with_busy_blk = MAX(found_busy_blk, csa->nl->highest_lbm_with_busy_blk);
 	assert(IS_BITMAP_BLK(csa->nl->highest_lbm_with_busy_blk));
 	new_total = MAX(MIN(old_total, csa->nl->highest_lbm_with_busy_blk + BLKS_PER_LMAP), keep_blocks);
+	assert(csd->fully_upgraded || (0 == csd->blks_to_upgrd) || (BLK_ID_32_VER < csd->desired_db_format));
 	if (mu_ctrly_occurred || mu_ctrlc_occurred)
 	{
 		rel_crit(gv_cur_region);
@@ -361,13 +383,6 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_MUTRUNCNOSPACE, 3, REG_LEN_STR(gv_cur_region), truncate_percent);
 		rel_crit(gv_cur_region);
 		return TRUE;
-	} else if (((GDSVCURR != csd->desired_db_format) && !(BLK_ID_32_VER > csd->desired_db_format))
-			|| csd->blks_to_upgrd != 0
-			|| !csd->fully_upgraded)
-	{
-		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_MUTRUNCNOV4, 2, REG_LEN_STR(gv_cur_region));
-		rel_crit(gv_cur_region);
-		return TRUE;
 	} else if (SNAPSHOTS_IN_PROG(csa->nl))
 	{
 		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_MUTRUNCSSINPROG, 2, REG_LEN_STR(gv_cur_region));
@@ -378,6 +393,18 @@ boolean_t mu_truncate(int4 truncate_percent, mval *keep_mval)
 		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_MUTRUNCBACKINPROG, 2, REG_LEN_STR(gv_cur_region));
 		rel_crit(gv_cur_region);
 		return TRUE;
+	} else if ((0 != csa->nl->reorg_upgrade_pid) && (is_proc_alive(csa->nl->reorg_upgrade_pid, 0)))
+	{	/* REORG -UPGRADE cannot run concurrently with TRUNCATE and is higher priority. Stop */
+		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+				LEN_AND_LIT("REORG -TRUNCATE"),
+				LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+				csa->nl->onln_rlbk_pid);
+		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(7) MAKE_MSG_INFO(ERR_REORGUPCNFLCT), 5,
+				LEN_AND_LIT("REORG -TRUNCATE"),
+				LEN_AND_LIT("MUPIP REORG -UPGRADE in progress"),
+				csa->nl->onln_rlbk_pid);
+		rel_crit(gv_cur_region);
+		return TRUE;
 	}
 	DEFER_INTERRUPTS(INTRPT_IN_TRUNC, prev_intrpt_state);
 	if (JNL_ENABLED(csa))
