@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2022 Fidelity National Information	*
+ * Copyright (c) 2006-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -185,7 +185,9 @@ void mupip_endiancvt(void)
 	uint4			cli_status;
 	unsigned short		n_len, outdb_len, t_len;
 	ZOS_ONLY(int		realfiletag;)
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	if (CLI_PRESENT == (cli_status = cli_present("OUTDB")))
 	{
 		outdb_specified = TRUE;
@@ -278,7 +280,7 @@ void mupip_endiancvt(void)
 			}
 		}
 		swap_dbver = (enum db_ver)GTM_BYTESWAP_32(old_data->desired_db_format);
-		if (!swap_dbver || ((GDSVCURR != swap_dbver) && !(BLK_ID_32_VER > swap_dbver)))
+		if ((0 >= swap_dbver) || (GDSV7 < swap_dbver))
 		{
 			check_error = NOTCURRDBFORMAT;
 			GTM_PUTMSG_CSA(VARLSTCNT(6) ERR_NOENDIANCVT, 4, n_len, db_name, LEN_AND_STR(check_error));
@@ -394,8 +396,7 @@ void mupip_endiancvt(void)
 				GTM_PUTMSG_CSA(VARLSTCNT(6) ERR_NOENDIANCVT, 4, n_len, db_name, LEN_AND_STR(check_error));
 			}
 		}
-		if (!old_data->desired_db_format || ((GDSVCURR != old_data->desired_db_format)
-			&& !(BLK_ID_32_VER > old_data->desired_db_format)))
+		if ((0 >= old_data->desired_db_format) || (GDSV7 < old_data->desired_db_format))
 		{
 			check_error = NOTCURRDBFORMAT;
 			GTM_PUTMSG_CSA(VARLSTCNT(6) ERR_NOENDIANCVT, 4, n_len, db_name, LEN_AND_STR(check_error));
@@ -530,12 +531,7 @@ void mupip_endiancvt(void)
 			TAG_POLICY_GTM_PUTMSG(outdb, realfiletag, TAG_BINARY, errno);
 #		endif
 		new_data->file_corrupt = endian_native ? GTM_BYTESWAP_32(TRUE) : TRUE;
-		if (0 == memcmp(new_data->label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
-		{
-			db_header_dwnconv(new_data);
-			LSEEKWRITE(outdb_fd, 0, new_data, SIZEOF(sgmnt_data), save_errno);
-		}else
-			LSEEKWRITE(outdb_fd, 0, new_data, SIZEOF(sgmnt_data), save_errno);
+		LSEEKWRITE(outdb_fd, 0, new_data, SIZEOF(sgmnt_data), save_errno);
 		if (0 != save_errno)
 		{
 			free(new_data);
@@ -619,13 +615,9 @@ void mupip_endiancvt(void)
 	new_data->file_corrupt = endian_native ? GTM_BYTESWAP_32(FALSE) : FALSE;
 	if (outdb_specified)
 	{
-		if (0 == memcmp(new_data->label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
-			db_header_dwnconv(new_data);
 		LSEEKWRITE(outdb_fd, 0, new_data, SIZEOF(sgmnt_data), save_errno);
 	} else
 	{
-		if (0 == memcmp(new_data->label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
-			db_header_dwnconv(new_data);
 		DB_LSEEKWRITE((sgmnt_addrs *)NULL, ((unix_db_info *)NULL), (char *)NULL, db_fd, 0,
 				new_data, SIZEOF(sgmnt_data), save_errno);
 	}
@@ -916,9 +908,7 @@ void endian_header(sgmnt_data *new, sgmnt_data *old, boolean_t new_is_native)
 	/************* SECSHR_DB_CLNUP RELATED FIELDS (now moved to node_local) ***********/
 	/* "secshr_ops_index_filler" and "secshr_ops_array_filler" need not be converted (they are fillers) */
 	/********************************************************/
-	/* "next_upgrd_warn" isn't valid since the database is fully_upgraded and the latch values differ by platform
-	 * and since we don't know where the db will be used, we will ignore it.
-	 */
+	/* "next_upgrd_warn" is unused and need not be converted */
 	SWAP_SD4(is_encrypted);
 	SWAP_SD4(db_trigger_cycle);
 	/************* SUPPLEMENTARY REPLICATION INSTANCE RELATED FIELDS ****************/
@@ -996,6 +986,7 @@ void	v6_endian_header(v6_sgmnt_data *new, v6_sgmnt_data *old, boolean_t new_is_n
 	SWAP_SD4(last_com_bkup_last_blk);
 	SWAP_SD4(last_rec_bkup_last_blk);
 	SWAP_SD4(reorg_restart_block);
+	/* Filler to align text with the V7 version */
 	/************* FIELDS SET WHEN DB IS OPEN ********************************/
 	new->image_count = 0;		/* should be zero when db is not open so reset it unconditionally */
 	new->freeze = 0;		/* should be zero when db is not open so reset it unconditionally */
@@ -1134,6 +1125,20 @@ void	v6_endian_header(v6_sgmnt_data *new, v6_sgmnt_data *old, boolean_t new_is_n
 #	define TAB_GVSTATS_REC(COUNTER,TEXT1,TEXT2)	SWAP_SD8(gvstats_rec.COUNTER);
 #	include "tab_gvstats_rec.h"
 #	undef TAB_GVSTATS_REC
+	/* Filler
+	 * space
+	 * for
+	 * alignment
+	 * with
+	 * the
+	 * V7
+	 * version
+	 * of
+	 * this
+	 * function
+	 *
+	 *
+	 */
 	/************* INTERRUPTED RECOVERY RELATED FIELDS continued ****************/
 	for (idx = 0; idx < MAX_SUPPL_STRMS; idx++)
 		SWAP_SD8(intrpt_recov_resync_strm_seqno[idx]);
@@ -1144,9 +1149,7 @@ void	v6_endian_header(v6_sgmnt_data *new, v6_sgmnt_data *old, boolean_t new_is_n
 	/************* SECSHR_DB_CLNUP RELATED FIELDS (now moved to node_local) ***********/
 	/* "secshr_ops_index_filler" and "secshr_ops_array_filler" need not be converted (they are fillers) */
 	/********************************************************/
-	/* "next_upgrd_warn" isn't valid since the database is fully_upgraded and the latch values differ by platform
-	 * and since we don't know where the db will be used, we will ignore it.
-	 */
+	/* "next_upgrd_warn" is unused and need not be converted */
 	SWAP_SD4(is_encrypted);
 	SWAP_SD4(db_trigger_cycle);
 	/************* SUPPLEMENTARY REPLICATION INSTANCE RELATED FIELDS ****************/
