@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2015-2021 Fidelity National Information	*
+ * Copyright (c) 2015-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -48,6 +48,7 @@
 #include "do_semop.h"
 #include "ftok_sems.h"
 #include "have_crit.h"
+#include "mu_updwn_ver_inline.h"
 #include "mupip_reorg_encrypt.h"
 #include "t_abort.h"
 #include "interlock.h"
@@ -98,7 +99,6 @@ error_def(ERR_MUREORGFAIL);
 error_def(ERR_ENCRYPTCONFLT);
 error_def(ERR_MUREENCRYPTEND);
 error_def(ERR_MUREENCRYPTSTART);
-error_def(ERR_MUREENCRYPTV4NOALLOW);
 error_def(ERR_REORGCTRLY);
 
 STATICFNDEF boolean_t	wait_for_concurrent_reads(sgmnt_addrs *csa);
@@ -169,7 +169,7 @@ void mupip_reorg_encrypt(void)
 	uint4			lcl_update_trans, pid, bptr_size;
 	jnl_private_control	*jpc;
 #	ifdef DEBUG
-	uint4			reencryption_count;
+	uint4			reencryption_count = 0;
 	block_id		initial_blk_cnt;
 #	endif
 	unix_db_info		*udi;
@@ -298,12 +298,7 @@ void mupip_reorg_encrypt(void)
 		}
 		/* ++++++++++++++++++++++++++ IN CRIT ++++++++++++++++++++++++++ */
 		grab_crit(reg, WS_91);
-		if (!csd->fully_upgraded)
-		{
-			CONTINUE_TO_NEXT_REGION(csa, csd, cnl, reg, reg_status, status,
-				REORG_IN_PROG_NOT_SET, HOLDING_CRIT, IS_ERROR,
-				gtm_putmsg_csa, (CSA_ARG(csa) VARLSTCNT(4) ERR_MUREENCRYPTV4NOALLOW, 2, DB_LEN_STR(reg)));
-		}
+		assert(csd->fully_upgraded || (0 == csd->blks_to_upgrd) || (BLK_ID_32_VER < csd->desired_db_format));
 		if (cnl->mupip_extract_count)
 		{
 			CONTINUE_TO_NEXT_REGION(csa, csd, cnl, reg, reg_status, status,
@@ -549,10 +544,24 @@ void mupip_reorg_encrypt(void)
 					mu_reorg_encrypt_in_prog = MUPIP_REORG_IN_PROG_TRUE;
 					if (SS_NORMAL != status1)
 					{
-						gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg), status1);
-						util_out_print("Region !AD : Error occurred while reading block [0x!XL]",
-								TRUE, REG_LEN_STR(reg), curblk);
-						reg_status = ERR_MUNOFINISH;
+						if ((-1 == status1) && (curblk >= csd->trans_hist.total_blks))
+						{
+							total_blks = csd->trans_hist.total_blks;
+							last_bmp = ROUND_DOWN2(total_blks - 1, BLKS_PER_LMAP);
+							assert(NULL == reorg_encrypt_restart_csa);
+							assert(SS_NORMAL == reg_status);
+							curbmp = last_bmp; /* to "break" out of 1st level (outermost) for loop */
+							lcnt = mapsize;	/* to break out of 2nd level (outer) for-loop */
+							util_out_print("Region !AD : Concurrent TRUNCATE reduced blocks [0x!@XQ]",
+									TRUE, REG_LEN_STR(reg), &csd->trans_hist.total_blks);
+						} else
+						{
+							gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(reg),
+									status1);
+							reg_status = ERR_MUNOFINISH;
+						}
+						util_out_print("Region !AD : Error occurred while reading block [0x!@XQ]",
+								TRUE, REG_LEN_STR(reg), &curblk);
 						break;
 					}
 					blk_tn = ((blk_hdr_ptr_t)buff)->tn;
@@ -602,7 +611,7 @@ void mupip_reorg_encrypt(void)
 					{	/* Means a bitmap block. */
 						BLK_ADDR(blkid_ptr, SIZEOF(block_id), block_id);
 						*blkid_ptr = 0;
-						t_write_map(blkhist, (unsigned char *)blkid_ptr, curr_tn, 0);
+						t_write_map(blkhist, blkid_ptr, curr_tn, 0);
 						assert(&alt_hist.h[0] == blkhist);
 						alt_hist.h[0].blk_num = 0; /* create empty history for bitmap block */
 						assert(update_trans);
@@ -645,7 +654,7 @@ void mupip_reorg_encrypt(void)
 							BLK_SEG(bs_ptr, blkBase + SIZEOF(new_hdr),
 								new_hdr.bsiz - SIZEOF(new_hdr));
 							BLK_FINI(bs_ptr, bs1);
-							t_write(blkhist, (unsigned char *)bs1, 0, 0,
+							t_write(blkhist, bs1, 0, 0,
 								((blk_hdr_ptr_t)blkBase)->levl, FALSE,
 								FALSE, GDS_WRITE_PLAIN);
 							/* The directory tree status for now is only used to determine whether
