@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2023 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -56,12 +56,12 @@
 #include "wcs_flu.h"
 #include "gds_rundown.h"
 #include "change_reg.h"
-#include "desired_db_format_set.h"
 #include "gtmmsg.h"		/* for gtm_putmsg prototype */
 #include "gtmcrypt.h"
 #include "anticipatory_freeze.h"
 #include "get_fs_block_size.h"
 #include "interlock.h"
+#include "min_max.h"
 
 GBLREF	bool			in_backup;
 GBLREF	bool			region;
@@ -74,7 +74,6 @@ LITREF char			*gtm_dbversion_table[];
 
 #define GTMCRYPT_ERRLIT		"during GT.M startup"
 
-error_def(ERR_ASYNCIONOV4);
 error_def(ERR_ASYNCIONOMM);
 error_def(ERR_CRYPTDLNOOPEN);
 error_def(ERR_CRYPTDLNOOPEN2);
@@ -88,10 +87,8 @@ error_def(ERR_DBRDERR);
 error_def(ERR_DBRDONLY);
 error_def(ERR_GTMCURUNSUPP);
 error_def(ERR_INVACCMETHOD);
-error_def(ERR_MMNODYNDWNGRD);
 error_def(ERR_MUPIPSET2BIG);
 error_def(ERR_MUPIPSET2SML);
-error_def(ERR_MUREENCRYPTV4NOALLOW);
 error_def(ERR_NODFRALLOCSUPP);
 error_def(ERR_NOUSERDB);
 error_def(ERR_OFRZACTIVE);
@@ -119,22 +116,22 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 	char			acc_spec[MAX_ACC_METH_LEN + 1], *command = "MUPIP SET VERSION", *errptr, exit_stat, *fn,
 				ver_spec[MAX_DB_VER_LEN + 1];
 	enum db_acc_method	access, access_new;
-	enum db_ver		desired_dbver;
 	uint4			fbwsize;
 	int4			dblksize;
 	gd_region		*temp_cur_region;
 	gd_segment		*seg;
-	int			asyncio_status, defer_allocate_status, defer_status, disk_wait_status, encryptable_status,
-				encryption_complete_status, epoch_taper_status, extn_count_status, fd, fn_len, glbl_buff_status,
-				gtmcrypt_errno, hard_spin_status, inst_freeze_on_error_status, key_size_status, locksharesdbcrit,
-				lock_space_status, mutex_space_status, null_subs_status, qdbrundown_status, rec_size_status,
-				reg_exit_stat, rc, rsrvd_bytes_status, sleep_cnt_status, save_errno, stats_status, status,
-				status1, stdb_alloc_status, stdnullcoll_status, trigger_flush_limit_status, wrt_per_flu_status,
-				full_blkwrt_status, problksplit_status;
-	int4			defer_time, new_cache_size, new_disk_wait, new_extn_count, new_flush_trigger, new_hard_spin,
-				new_key_size, new_lock_space, new_mutex_space, new_null_subs, new_rec_size, new_sleep_cnt,
-				new_spin_sleep, new_statsdb_alloc, new_stdnullcoll, new_wrt_per_flu, reserved_bytes,
-				spin_sleep_status, read_only_status, new_full_blkwrt, new_problksplit;
+	int			asyncio_status, defer_allocate_status, defer_status, disk_wait_status, d_rsrvd_bytes_status,
+				encryptable_status,encryption_complete_status, epoch_taper_status, extn_count_status, fd,
+				fn_len, glbl_buff_status, gtmcrypt_errno, hard_spin_status, inst_freeze_on_error_status,
+				i_rsrvd_bytes_status, key_size_status, locksharesdbcrit,lock_space_status, mutex_space_status,
+				null_subs_status, qdbrundown_status, rec_size_status, reg_exit_stat, rc, rsrvd_bytes_status,
+				sleep_cnt_status, save_errno, stats_status, status, status1, stdb_alloc_status,
+				stdnullcoll_status, trigger_flush_limit_status, wrt_per_flu_status, full_blkwrt_status,
+				problksplit_status;
+	int4			defer_time, d_reserved_bytes, i_reserved_bytes, new_cache_size, new_disk_wait, new_extn_count,
+				new_flush_trigger, new_hard_spin, new_key_size, new_lock_space, new_mutex_space, new_null_subs = -1,
+				new_rec_size, new_sleep_cnt, new_spin_sleep, new_statsdb_alloc, new_stdnullcoll, new_wrt_per_flu,
+				reserved_bytes, spin_sleep_status, read_only_status, new_full_blkwrt, new_problksplit;
 	sgmnt_data_ptr_t	csd, pvt_csd;
 	tp_region		*rptr, single;
 	uint4			fsb_size, reservedDBFlags;
@@ -166,16 +163,17 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		{
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_INVACCMETHOD);
 			exit_stat |= EXIT_ERR;
+			access = dba_dummy;
 		}
 		need_standalone = TRUE;
 	} else
 		access = n_dba;		/* really want to keep current method,
 					    which has not yet been read */
-	if (asyncio_status = cli_present("ASYNCIO"))
+	if ((asyncio_status = cli_present("ASYNCIO")))
 		need_standalone = TRUE;
-	if (defer_allocate_status = cli_present("DEFER_ALLOCATE"))
+	if ((defer_allocate_status = cli_present("DEFER_ALLOCATE")))
 		flush_buffers = TRUE;
-	if (encryptable_status = cli_present("ENCRYPTABLE"))
+	if ((encryptable_status = cli_present("ENCRYPTABLE")))
 	{
 		need_standalone = TRUE;
 		if (CLI_PRESENT == encryptable_status)
@@ -197,18 +195,18 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		} else /* When disabling encryption ignore invalid encryption setup errors */
 			TREF(mu_set_file_noencryptable) = TRUE;
 	}
-	if (read_only_status = cli_present("READ_ONLY")) /* Note assignment */
+	if ((read_only_status = cli_present("READ_ONLY")) /* Note assignment */)
 		need_standalone = TRUE;
 	encryption_complete_status = cli_present("ENCRYPTIONCOMPLETE");
 	epoch_taper_status = cli_present("EPOCHTAPER");
-	if (problksplit_status = cli_present("PROBLKSPLIT"))
+	if ((problksplit_status = cli_present("PROBLKSPLIT")))
 		if (!cli_get_int("PROBLKSPLIT", &new_problksplit))
 		{
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SETQUALPROB, 2, LEN_AND_LIT("PROBLKSPLIT"));
 			exit_stat |= EXIT_ERR;
 		}
 	/* EXTENSION_COUNT does not require standalone access and hence need_standalone will not be set to TRUE for this. */
-	if (extn_count_status = cli_present("EXTENSION_COUNT"))
+	if ((extn_count_status = cli_present("EXTENSION_COUNT")))
 	{
 		if (cli_get_int("EXTENSION_COUNT", &new_extn_count))
 		{	/* minimum is 0 & mupip_cmd defines this qualifier to not accept negative values, so no min check */
@@ -225,7 +223,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		flush_buffers = TRUE;
 	}
-	if (full_blkwrt_status = cli_present("FULLBLKWRT"))
+	if ((full_blkwrt_status = cli_present("FULLBLKWRT")))
 	{
 		if (cli_get_int("FULLBLKWRT", &new_full_blkwrt))
 		{
@@ -243,7 +241,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			exit_stat |= EXIT_ERR;
 		}
 	}
-	if (glbl_buff_status = cli_present("GLOBAL_BUFFERS"))
+	if ((glbl_buff_status = cli_present("GLOBAL_BUFFERS")))
 	{
 		if (cli_get_int("GLOBAL_BUFFERS", &new_cache_size))
 		{
@@ -266,7 +264,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		need_standalone = TRUE;
 	}
-	if (hard_spin_status = cli_present("HARD_SPIN_COUNT"))
+	if ((hard_spin_status = cli_present("HARD_SPIN_COUNT")))
 	{	/* No min or max tests needed because mupip_cmd enforces min of 0 and no max requirement is documented*/
 		if (!cli_get_int("HARD_SPIN_COUNT", &new_hard_spin))
 		{
@@ -275,7 +273,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 	}
 	inst_freeze_on_error_status = cli_present("INST_FREEZE_ON_ERROR");
-	if (key_size_status = cli_present("KEY_SIZE"))
+	if ((key_size_status = cli_present("KEY_SIZE")))
 	{
 		if (cli_get_int("KEY_SIZE", &new_key_size))
 		{
@@ -298,9 +296,9 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		need_standalone = TRUE;
 	}
-	if (locksharesdbcrit = cli_present("LCK_SHARES_DB_CRIT"))
+	if ((locksharesdbcrit = cli_present("LCK_SHARES_DB_CRIT")))
 		need_standalone = TRUE;
-	if (lock_space_status = cli_present("LOCK_SPACE"))
+	if ((lock_space_status = cli_present("LOCK_SPACE")))
 	{
 		if (cli_get_int("LOCK_SPACE", &new_lock_space))
 		{
@@ -323,7 +321,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		need_standalone = TRUE;
 	}
-	if (mutex_space_status = cli_present("MUTEX_SLOTS"))
+	if ((mutex_space_status = cli_present("MUTEX_SLOTS")))
 	{
 		if (cli_get_int("MUTEX_SLOTS", &new_mutex_space))
 		{
@@ -345,15 +343,15 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		need_standalone = TRUE;
 	}
-	if (null_subs_status = cli_present("NULL_SUBSCRIPTS"))
+	if ((null_subs_status = cli_present("NULL_SUBSCRIPTS")))
 	{
 		if (-1 == (new_null_subs = cli_n_a_e("NULL_SUBSCRIPTS")))
 			exit_stat |= EXIT_ERR;
 		need_standalone = TRUE;
 	}
-	if (qdbrundown_status = cli_present("QDBRUNDOWN"))
+	if ((qdbrundown_status = cli_present("QDBRUNDOWN")))
 		need_standalone = TRUE;
-	if (rec_size_status = cli_present("RECORD_SIZE"))
+	if ((rec_size_status = cli_present("RECORD_SIZE")))
 	{
 		if (cli_get_int("RECORD_SIZE", &new_rec_size))
 		{	/* minimum is 0 & mupip_cmd defines this qualifier to not accept negative values, so no min check */
@@ -370,17 +368,32 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		need_standalone = TRUE;
 	}
-	if (rsrvd_bytes_status = cli_present("RESERVED_BYTES"))
+	if ((rsrvd_bytes_status = cli_present("RESERVED_BYTES")))
 	{
 		if (!cli_get_int("RESERVED_BYTES", &reserved_bytes))
 		{
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SETQUALPROB, 2, LEN_AND_LIT("RESERVED_BYTES"));
 			exit_stat |= EXIT_ERR;
 		}
-		need_standalone = TRUE;
+	}
+	if ((i_rsrvd_bytes_status = cli_present("INDEX_RESERVED_BYTES")))
+	{
+		if (!cli_get_int("INDEX_RESERVED_BYTES", &i_reserved_bytes))
+		{
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SETQUALPROB, 2, LEN_AND_LIT("INDEX_RESERVED_BYTES"));
+			exit_stat |= EXIT_ERR;
+		}
+	}
+	if ((d_rsrvd_bytes_status = cli_present("DATA_RESERVED_BYTES")))
+	{
+		if (!cli_get_int("DATA_RESERVED_BYTES", &d_reserved_bytes))
+		{
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SETQUALPROB, 2, LEN_AND_LIT("DATA_RESERVED_BYTES"));
+			exit_stat |= EXIT_ERR;
+		}
 	}
 	/* SLEEP_SPIN_COUNT does not require standalone access and hence need_standalone will not be set to TRUE for this. */
-	if (sleep_cnt_status = cli_present("SLEEP_SPIN_COUNT"))
+	if ((sleep_cnt_status = cli_present("SLEEP_SPIN_COUNT")))
 	{
 		if (cli_get_int("SLEEP_SPIN_COUNT", &new_sleep_cnt))
 		{	/* minimum is 0 & mupip_cmd defines this qualifier to not accept negative values, so no min check */
@@ -396,7 +409,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			exit_stat |= EXIT_ERR;
 		}
 	}
-	if (spin_sleep_status = cli_present("SPIN_SLEEP_MASK"))
+	if ((spin_sleep_status = cli_present("SPIN_SLEEP_MASK")))
 	{
 		if (cli_get_hex("SPIN_SLEEP_MASK", (uint4 *) &new_spin_sleep))
 		{
@@ -413,9 +426,9 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			exit_stat |= EXIT_ERR;
 		}
 	}
-	if (stats_status = cli_present("STATS"))
+	if ((stats_status = cli_present("STATS")))
 		need_standalone = TRUE;
-	if (stdb_alloc_status = cli_present("STATSDB_ALLOCATION"))
+	if ((stdb_alloc_status = cli_present("STATSDB_ALLOCATION")))
 	{
 		if (cli_get_int("STATSDB_ALLOCATION", &new_statsdb_alloc))
 		{
@@ -438,29 +451,15 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			exit_stat |= EXIT_ERR;
 		}
 	}
-	if (stdnullcoll_status = cli_present("STDNULLCOLL"))
+	if ((stdnullcoll_status = cli_present("STDNULLCOLL")))
 		need_standalone = TRUE;
 	if (cli_present("VERSION"))
 	{
-		cli_get_str("VERSION", ver_spec, &ver_spec_len);
-		ver_spec[ver_spec_len] = '\0';
-		cli_strupper(ver_spec);
-		if (0 == memcmp(ver_spec, "V7", ver_spec_len + 1))
-			desired_dbver = GDSV7;
-		else if (0 == memcmp(ver_spec, "V6", ver_spec_len + 1))
-			desired_dbver = GDSV6;
-		else if (0 == memcmp(ver_spec, "V4", ver_spec_len + 1))
-		{
-			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_GTMCURUNSUPP);
-			exit_stat |= EXIT_ERR;
-			desired_dbver = GDSV4;
-		}
-		else
-			assertpro(FALSE);		/* CLI should prevent us ever getting here */
+		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_GTMCURUNSUPP);
+		exit_stat |= EXIT_ERR;
 		flush_buffers = TRUE;
-	} else
-		desired_dbver = GDSVLAST;	/* really want to keep version, which has not yet been read */
-	if (disk_wait_status = cli_present("WAIT_DISK"))
+	}
+	if ((disk_wait_status = cli_present("WAIT_DISK")))
 	{
 		if (cli_get_int("WAIT_DISK", &new_disk_wait))
 		{
@@ -549,6 +548,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				DO_CLNUP_AND_SET_EXIT_STAT(exit_stat, EXIT_ERR);
 				continue;
 			}
+			assert(dba_dummy != access);
 			if ((n_dba != access) && (pvt_csd->acc_meth != access))	/* n_dba is a proxy for no change */
 			{
 				acc_meth_changing = TRUE;
@@ -613,13 +613,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 						reg_exit_stat |= EXIT_WRN;
 					}
 				}
-				if (pvt_csd->blks_to_upgrd)
-				{
-					util_out_print("MM access method cannot be set if there are blocks to upgrade",	TRUE);
-					util_out_print("Database file !AD not changed", TRUE, fn_len, fn);
-					reg_exit_stat |= EXIT_WRN;
-				}
-				if (pvt_csd->offset && ((GDSVCURR - 1) == pvt_csd->desired_db_format))
+				if (pvt_csd->offset && (GDSV6p == pvt_csd->desired_db_format))
 				{	/* if we support downgrade, enhance the above conditions */
 					util_out_print("MM access method cannot be set during DB upgrade",
 						TRUE);
@@ -650,11 +644,9 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			}
 			if (encryptable_status)
 			{
-				if (!pvt_csd->fully_upgraded)
-				{
-					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_MUREENCRYPTV4NOALLOW, 2, fn_len, fn);
-					reg_exit_stat |= EXIT_WRN;
-				} else if (dba_mm == access_new)
+				assert(pvt_csd->fully_upgraded || (0 == pvt_csd->blks_to_upgrd)
+						|| (BLK_ID_32_VER < pvt_csd->desired_db_format));
+				if (dba_mm == access_new)
 				{
 					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_CRYPTNOMM, 2, fn_len, fn);
 					reg_exit_stat |= EXIT_WRN;
@@ -680,7 +672,8 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			{
 				long_blk_id = BLK_ID_32_VER < pvt_csd->desired_db_format;
 				key_size_status = pvt_csd->blk_size - SIZEOF(blk_hdr) - SIZEOF(rec_hdr) -
-						SIZEOF_BLK_ID(long_blk_id) - bstar_rec_size(long_blk_id) - pvt_csd->reserved_bytes;
+					SIZEOF_BLK_ID(long_blk_id) - bstar_rec_size(long_blk_id) -
+					(MAX(pvt_csd->reserved_bytes, pvt_csd->i_reserved_bytes));
 				if (key_size_status < new_key_size)
 				{	/* too big for block */
 					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_MUPIPSET2BIG, 4, new_key_size,
@@ -702,7 +695,10 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			if (mutex_space_status)
 				NUM_CRIT_ENTRY(pvt_csd) = new_mutex_space;
 			if (null_subs_status)
+			{
+				assert(-1 != new_null_subs);
 				gv_cur_region->null_subs = pvt_csd->null_subs = (unsigned char)new_null_subs;
+			}
 			if (qdbrundown_status)
 				pvt_csd->mumps_can_bypass = (CLI_PRESENT == qdbrundown_status);
 			if (rec_size_status)
@@ -715,17 +711,6 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				}
 				pvt_csd->max_rec_size = new_rec_size;
 			}
-			if (rsrvd_bytes_status)
-			{
-				long_blk_id = BLK_ID_32_VER < pvt_csd->desired_db_format;
-				if (reserved_bytes > MAX_RESERVE_B(pvt_csd, long_blk_id))
-				{
-					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_MUPIPSET2BIG, 4, reserved_bytes,
-						LEN_AND_LIT("RESERVED_BYTES"), MAX_RESERVE_B(pvt_csd, long_blk_id));
-					reg_exit_stat |= EXIT_WRN;
-				}
-				pvt_csd->reserved_bytes = reserved_bytes;
-			}
 			if (stats_status)
 			{
 				reservedDBFlags = pvt_csd->reservedDBFlags & ~RDBF_NOSTATS;
@@ -749,9 +734,14 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				}
 			}
 			if (CLI_NEGATED == stdnullcoll_status)
-				gv_cur_region->std_null_coll = pvt_csd->std_null_coll = FALSE;
-			else if (CLI_PRESENT == stdnullcoll_status)
-				gv_cur_region->std_null_coll = pvt_csd->std_null_coll = TRUE;
+			{
+				gv_cur_region->std_null_coll = FALSE;
+				pvt_csd->std_null_coll = FALSE;
+			} else if (CLI_PRESENT == stdnullcoll_status)
+			{
+				gv_cur_region->std_null_coll = TRUE;
+				pvt_csd->std_null_coll = TRUE;
+			}
 			if (EXIT_NRM != reg_exit_stat)
 			{
 				DO_CLNUP_AND_SET_EXIT_STAT(exit_stat, reg_exit_stat);
@@ -804,27 +794,14 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				mu_gv_cur_reg_free();
 				continue;
 			}
+			fd = FD_INVALID;
 		}
+		assert(dba_dummy != access);
 		access_new = (n_dba == access ? csd->acc_meth : access);
-		if (GDSVLAST != desired_dbver)
-		{	/* TODO: reject GDSV4 requests and figure out what to do with MM; revise desired_db_format_set */
-			if ((dba_mm != access_new) || (GDSV4 != desired_dbver))
-				(void)desired_db_format_set(gv_cur_region, desired_dbver, command);
-			else	/* for other errors desired_db_format_set prints appropriate error messages */
-				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_MMNODYNDWNGRD, 2, REG_LEN_STR(gv_cur_region));
-			if (csd->desired_db_format != desired_dbver)
-				reg_exit_stat |= EXIT_WRN;
-			else
-				util_out_print("Database file !AD now has desired DB format !AD", TRUE,
-					fn_len, fn, LEN_AND_STR(gtm_dbversion_table[csd->desired_db_format]));
-		}
 		if (encryption_complete_status)
 		{
-			if (!csd->fully_upgraded)
-			{
-				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_MUREENCRYPTV4NOALLOW, 2, fn_len, fn);
-				reg_exit_stat |= EXIT_WRN;
-			} else if (dba_mm == access_new)
+			assert(csd->fully_upgraded || (0 == csd->blks_to_upgrd) || (BLK_ID_32_VER < csd->desired_db_format));
+			if (dba_mm == access_new)
 			{
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_CRYPTNOMM, 2, fn_len, fn);
 				reg_exit_stat |= EXIT_WRN;
@@ -839,12 +816,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 		}
 		if (asyncio_status && (CLI_PRESENT == asyncio_status))
 		{
-			if (!csd->fully_upgraded)
-			{
-				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ASYNCIONOV4, 6, DB_LEN_STR(gv_cur_region),
-					LEN_AND_LIT("V4 format"), LEN_AND_LIT("enable ASYNCIO"));
-				reg_exit_stat |= EXIT_WRN;
-			}
+			assert(csd->fully_upgraded || (0 == csd->blks_to_upgrd) || (BLK_ID_32_VER < csd->desired_db_format));
 			if (!acc_meth_changing && (dba_bg != access_new))
 			{
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_ASYNCIONOMM, 6, DB_LEN_STR(gv_cur_region),
@@ -853,6 +825,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			}
 			seg = gv_cur_region->dyn.addr;
 			/* AIO = ON, implies we need to use O_DIRECT. Check for db vs fs blksize alignment issues. */
+			assert(!got_standalone || (FD_INVALID != fd));
 			fsb_size = get_fs_block_size(got_standalone ? fd : FILE_INFO(gv_cur_region)->fd);
 			if (0 != (csd->blk_size % fsb_size))
 			{
@@ -861,7 +834,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				reg_exit_stat |= EXIT_WRN;
 			}
 		}
-		if (wrt_per_flu_status = (CLI_PRESENT == cli_present("WRITES_PER_FLUSH")))
+		if ((wrt_per_flu_status = (CLI_PRESENT == cli_present("WRITES_PER_FLUSH"))))
 		{
 			if (cli_get_int("WRITES_PER_FLUSH", &new_wrt_per_flu))
 			{
@@ -882,7 +855,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				reg_exit_stat |= EXIT_ERR;
 			}
 		}
-		if (trigger_flush_limit_status = (CLI_PRESENT == cli_present("TRIGGER_FLUSH_LIMIT")))
+		if ((trigger_flush_limit_status = (CLI_PRESENT == cli_present("TRIGGER_FLUSH_LIMIT"))))
 		{
 			if (cli_get_int("TRIGGER_FLUSH_LIMIT", &new_flush_trigger))
 			{
@@ -907,6 +880,42 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				reg_exit_stat |= EXIT_ERR;
 			}
 		}
+		if (rsrvd_bytes_status)
+		{
+			long_blk_id = BLK_ID_32_VER < csd->desired_db_format;
+			if (reserved_bytes > MAX_RESERVE_B(csd, long_blk_id))
+			{
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_MUPIPSET2BIG, 4, reserved_bytes,
+						LEN_AND_LIT("RESERVED_BYTES"), MAX_RESERVE_B(csd, long_blk_id));
+				reg_exit_stat |= EXIT_WRN;
+			} else
+			{
+				csd->reserved_bytes = reserved_bytes;
+				csd->i_reserved_bytes = reserved_bytes;
+			}
+		}
+		if (i_rsrvd_bytes_status)
+		{
+			long_blk_id = BLK_ID_32_VER < csd->desired_db_format;
+			if (i_reserved_bytes > MAX_RESERVE_B(csd, long_blk_id))
+			{
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_MUPIPSET2BIG, 4, i_reserved_bytes,
+						LEN_AND_LIT("INDEX_RESERVED_BYTES"), MAX_RESERVE_B(csd, long_blk_id));
+				reg_exit_stat |= EXIT_WRN;
+			} else
+				csd->i_reserved_bytes = i_reserved_bytes;
+		}
+		if (d_rsrvd_bytes_status)
+		{
+			long_blk_id = BLK_ID_32_VER < csd->desired_db_format;
+			if (d_reserved_bytes > MAX_RESERVE_B(csd, long_blk_id))
+			{
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_MUPIPSET2BIG, 4, d_reserved_bytes,
+						LEN_AND_LIT("DATA_RESERVED_BYTES"), MAX_RESERVE_B(csd,long_blk_id));
+				reg_exit_stat |= EXIT_WRN;
+			} else
+				csd->reserved_bytes = d_reserved_bytes;
+		}
 		if (EXIT_NRM == reg_exit_stat)
 		{
 			if (n_dba != access)
@@ -1019,8 +1028,18 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			if (problksplit_status)
 				util_out_print("Database file !AD now has proactive block split flag set to !UL", TRUE,
 					fn_len, fn, csd->problksplit);
+			if (rsrvd_bytes_status)
+				util_out_print("Database file !AD now has !UL reserved bytes",
+						TRUE, fn_len, fn, csd->reserved_bytes);
+			if (i_rsrvd_bytes_status)
+				util_out_print("Database file !AD now has !UL index reserved bytes",
+						TRUE, fn_len, fn, csd->i_reserved_bytes);
+			if (d_rsrvd_bytes_status)
+				util_out_print("Database file !AD now has !UL data reserved bytes",
+						TRUE, fn_len, fn, csd->reserved_bytes);
 			if (got_standalone)
 			{
+				assert(FD_INVALID != fd);
 				if (0 == memcmp(pvt_csd->label, V6_GDS_LABEL, GDS_LABEL_SZ - 1))
 					db_header_dwnconv(pvt_csd);
 				DB_LSEEKWRITE(NULL,((unix_db_info *)NULL),NULL,fd,0,pvt_csd,SIZEOF(sgmnt_data),status);
@@ -1067,9 +1086,6 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				if (rec_size_status)
 					util_out_print("Database file !AD now has maximum record size !UL",
 							TRUE, fn_len, fn, pvt_csd->max_rec_size);
-				if (rsrvd_bytes_status)
-					util_out_print("Database file !AD now has !UL reserved bytes",
-							TRUE, fn_len, fn, pvt_csd->reserved_bytes);
 				if (stats_status)
 					util_out_print("Database file !AD now has sharing of gvstats set to !AD", TRUE,
 							fn_len, fn, 5, (CLI_PRESENT == stats_status) ? " TRUE" : "FALSE");
@@ -1094,6 +1110,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			exit_stat |= reg_exit_stat;
 		if (got_standalone)
 		{
+			assert(FD_INVALID != fd);
 			CLOSEFILE_RESET(fd, rc);	/* resets "fd" to FD_INVALID */
 			db_ipcs_reset(gv_cur_region);
 		} else
