@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -20,7 +20,7 @@
 #include "gtm_stdlib.h"
 #include "gtm_unistd.h"
 #include "gtm_stdio.h"
-#include "gtm_select.h"
+#include "gtm_poll.h"
 #include "gtm_un.h"
 
 #include <errno.h>
@@ -41,6 +41,7 @@
 #include "filestruct.h"
 #include "io.h"
 #include "jnl.h"
+#include "jnlbufs.h"
 #include "gdsbgtr.h"
 #include "mutex.h"
 #include "relqueopi.h"
@@ -52,9 +53,11 @@
 #include "gtm_rel_quant.h"
 #include "add_inter.h"
 #include "mutex_deadlock_check.h"
+#include "memcoherency.h"
 #include "gt_timer.h"
 #include "gtmio.h"
 #include "gtm_c_stack_trace.h"
+#include "gtm_reservedDB.h"
 #include "sleep.h"
 #include "anticipatory_freeze.h"
 #ifdef DEBUG
@@ -141,12 +144,16 @@ MBSTART {															\
 	}															\
 } MBEND
 
+#define IS_ONLNRLBK_ACTIVE(CSA)	(0 != CSA->nl->onln_rlbk_pid)
 
 GBLREF int			num_additional_processors;
 GBLREF jnl_gbls_t		jgbl;
 GBLREF jnlpool_addrs_ptr_t	jnlpool;
 GBLREF uint4			process_id;
 GBLREF uint4			mutex_per_process_init_pid;
+GBLREF uint4			mu_upgrade_in_prog;
+GBLREF volatile boolean_t	timer_in_handler;
+GBLREF volatile boolean_t	exit_handler_active;
 #ifdef MUTEX_MSEM_WAKE
 #  ifdef POSIX_MSEM
 static sem_t			*mutex_wake_msem_ptr = NULL;
@@ -155,7 +162,6 @@ static msemaphore		*mutex_wake_msem_ptr = NULL;
 #  endif
 static mutex_que_entry_ptr_t	msem_slot;
 #else
-GBLREF fd_set			mutex_wait_on_descs;
 GBLREF int			mutex_sock_fd;
 #endif
 #ifdef DEBUG
@@ -181,6 +187,7 @@ error_def(ERR_MUTEXERR);
 error_def(ERR_MUTEXFRCDTERM);
 error_def(ERR_MUTEXLCKALERT);
 error_def(ERR_ORLBKINPROG);
+error_def(ERR_REORGUPCNFLCT);
 error_def(ERR_TEXT);
 error_def(ERR_WCBLOCKED);
 
@@ -253,6 +260,155 @@ error_def(ERR_WCBLOCKED);
  *		Fields may be interspersed with fillers for alignment purposes.
  */
 
+void rollback_mutex_cln_ctl(seq_num max_seqno)
+{
+	jnlpool_ctl_ptr_t	jpl;
+	unsigned int		start, top, i, j;
+	mutex_cln_ctl_struct	lcl_cln_ctl = { .top = 0, .pids = {0}, .seqnos = {0} };
+
+	assert(jnlpool && jnlpool->jnlpool_ctl);
+	assert((MAX_MUTEX_CLNS & (MAX_MUTEX_CLNS - 1)) == 0);
+	jpl = jnlpool->jnlpool_ctl;
+	/* If top is zero, then the array is empty. We guarantee this with our overflow handling. Values of top from 0 to
+	 * MAX_MUTEX_CLNS count the number of valid entries starting at index 0; values over MAX_MUTEX_CLNS indicate there are
+	 * MAX_MUTEX_CLNS entries and the start location is at (top - MAX_MUTEX_CLNS) % MAX_MUTEX_CLNS (since MAX_MUTEX_CLNS is
+	 * a power of 2, (top - MAX_MUTEX_CLNS) & (MAX_MUTEX_CLNS - 1).
+	 */
+	if ((top = jpl->mutex_cln_ctl.top)) /* WARNING - assignment. */
+	{
+		start = (top < MAX_MUTEX_CLNS) ? 0 : (top - MAX_MUTEX_CLNS);
+		for (j = start; j < top; j++)
+		{
+			i = j & (MAX_MUTEX_CLNS - 1);
+			if (jpl->mutex_cln_ctl.seqnos[i] >= max_seqno)
+				break;
+			lcl_cln_ctl.pids[lcl_cln_ctl.top] = jpl->mutex_cln_ctl.pids[i];
+			lcl_cln_ctl.seqnos[lcl_cln_ctl.top] = jpl->mutex_cln_ctl.seqnos[i];
+			lcl_cln_ctl.top++;
+		}
+	}
+	/* A full array can and normally does have top grow larger than this, since it's a virtual index which corresponds
+	 * to a mod location. But the only way for the array to be less than full is for the top to be strictly less than
+	 * MAX_MUTEX_CLNS. Therefore if the array is less than full, or if any rollback occurs, then the top will be less
+	 * than MAX_MUTEX_CLNS and if the array starts out full and there is no rollback needed the top will equal
+	 * MAX_MUTEX_CLNS.
+	 */
+	assert(lcl_cln_ctl.top <= MAX_MUTEX_CLNS);
+	jpl->mutex_cln_ctl = lcl_cln_ctl;
+	return;
+}
+
+/* Called during a region-crit mutex_clean_dead_owner. */
+static inline enum mutex_cln_status get_mutex_cln_info(sgmnt_addrs *csa, uint4 holder_pid, seq_num jbuf_seqno, mutex_cln_info *info)
+{
+	jnlpool_ctl_ptr_t	jpl;
+	unsigned int		start, top, i, j;
+	seq_num			rec_seqno, min_seqno = UINT64_MAX;
+	sgmnt_addrs		*repl_csa = NULL;
+	boolean_t		was_crit = FALSE, in_async_code;
+#	ifdef DEBUG
+	seq_num			last_seqno = 0;
+#	endif
+	uint4			rec_pid;
+	enum mutex_cln_status	status;
+
+	assert(csa->hdr);
+	assert((MAX_MUTEX_CLNS & (MAX_MUTEX_CLNS - 1)) == 0);
+	info->pid = 0;
+	info->seqno = 0;
+	status = mutex_cln_absent;
+	if (!jnlpool || (jnlpool != csa->jnlpool) || !(jpl = jnlpool->jnlpool_ctl)) /* WARNING - assignment */
+	{
+		assert(!jnlpool || (jnlpool != csa->jnlpool));
+		return mutex_cln_invalid;
+	}
+	repl_csa = &FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs;
+	was_crit = repl_csa->now_crit;
+	assert(!was_crit);
+	in_async_code = timer_in_handler || exit_handler_active;
+	if (!was_crit && !grab_lock(jnlpool->jnlpool_dummy_reg, !in_async_code, ASSERT_NO_ONLINE_ROLLBACK))
+		return mutex_cln_invalid;
+	/* If top is zero, then the array is empty. We guarantee this with our overflow handling. Values of top from 0 to
+	 * MAX_MUTEX_CLNS count the number of valid entries starting at index 0; values over MAX_MUTEX_CLNS indicate there are
+	 * MAX_MUTEX_CLNS entries and the start location is at (top - MAX_MUTEX_CLNS) % MAX_MUTEX_CLNS (since MAX_MUTEX_CLNS is
+	 * a power of 2, (top - MAX_MUTEX_CLNS) & (MAX_MUTEX_CLNS - 1).
+	 */
+	if ((top = jpl->mutex_cln_ctl.top)) /* WARNING - assignment. */
+	{
+		start = (top < MAX_MUTEX_CLNS) ? 0 : (top - MAX_MUTEX_CLNS);
+		for (j = start; j < top; j++)
+		{
+			i = j & (MAX_MUTEX_CLNS - 1);
+			rec_pid = jpl->mutex_cln_ctl.pids[i];
+			rec_seqno = jpl->mutex_cln_ctl.seqnos[i];
+#			ifdef DEBUG
+			assert(rec_seqno >= last_seqno);
+			last_seqno = rec_seqno;
+#			endif
+			if (rec_seqno)
+				min_seqno = (min_seqno > rec_seqno) ? rec_seqno : min_seqno;
+			if (holder_pid == rec_pid)
+			{
+				if (info->pid)
+				{
+					/* Multiple possible matches, so ignore. */
+					status = mutex_cln_invalid;
+					break;
+				}
+				if (rec_seqno && ((rec_seqno == jbuf_seqno) || (rec_seqno == (jbuf_seqno + 1))))
+				{
+					info->pid = rec_pid;
+					info->seqno = rec_seqno;
+					status = mutex_cln_present;
+				}
+			}
+		}
+	}
+	/* If an overflow has taken place and the minimum seqno seen is larger than the jbuf seqno, do
+	 * not report an 'absent' entry, which says that there is an informative absence (ie the
+	 * process was not in instance crit when it died), but rather this function must report that
+	 * no valid inference can be made from the data available.
+	 */
+	if ((top >= MAX_MUTEX_CLNS) && (min_seqno >= jbuf_seqno) && (mutex_cln_absent == status))
+		status = mutex_cln_invalid;
+	if (!was_crit)
+		rel_lock(jnlpool->jnlpool_dummy_reg);
+	return status;
+}
+
+/* Called during a mutex_clean_dead_owner on a jnlpool mutex. The corresponding get_mutex_cln_info is called
+ * during a region-crit mutex_clean_dead_owner.
+ */
+static inline void set_mutex_cln_info(sgmnt_addrs *csa, uint4 holder_pid)
+{
+	jnlpool_ctl_ptr_t	jpl;
+	unsigned int		i;
+
+	assert(!csa->hdr);
+	assert(jnlpool);
+	assert((MAX_MUTEX_CLNS & (MAX_MUTEX_CLNS - 1)) == 0);
+	/* Gracefully return even in cases we think are impossible since this is a sensitive location. */
+	if ((!jnlpool) || !(jpl = jnlpool->jnlpool_ctl))
+	{
+		assert(FALSE);
+		return;
+	}
+	/* Handle overflow: if on the last value, skip to first value that mods the same, then increment. Otherwise just increment.
+	 * This guarantees that the first MAX_MUTEX_CLNS indices are special values which indicate that the array has really that
+	 * many elements and also that we can change the size of the array if necessary.
+	 */
+	assert((UINT_MAX / 2) > MAX_MUTEX_CLNS);
+	if (UINT_MAX == jpl->mutex_cln_ctl.top)
+	{
+		jpl->mutex_cln_ctl.top &= (MAX_MUTEX_CLNS - 1);
+		jpl->mutex_cln_ctl.top += MAX_MUTEX_CLNS;
+	}
+	i = jpl->mutex_cln_ctl.top & (MAX_MUTEX_CLNS - 1);
+	jpl->mutex_cln_ctl.pids[i] = holder_pid;
+	jpl->mutex_cln_ctl.seqnos[i] = jpl->jnl_seqno;
+	jpl->mutex_cln_ctl.top++;
+}
+
 #ifndef CRIT_USE_PTHREAD_MUTEX
 static	void	clean_initialize(mutex_struct_ptr_t addr, int n, bool crash)
 {
@@ -365,6 +521,9 @@ static	enum cdb_sc mutex_long_sleep(mutex_struct_ptr_t addr, sgmnt_addrs *csa,
 	ssize_t			nbrecvd;
 	int			timeout_intr_slpcnt;
 	long			timeout_val;
+	int			poll_timeout;
+	nfds_t			poll_nfds;
+	struct pollfd		poll_fdlist[1];
 #	endif
 
 #	ifdef DEBUG
@@ -437,15 +596,17 @@ static	enum cdb_sc mutex_long_sleep(mutex_struct_ptr_t addr, sgmnt_addrs *csa,
 			 */
 			timeout_intr_slpcnt = MUTEX_INTR_SLPCNT;
 			MUTEX_DPRINT4("%d: Sleeping for %d s %d us\n", process_id, timeout.tv_sec, timeout.tv_usec);
-			assertpro(FD_SETSIZE > mutex_sock_fd);
-			FD_SET(mutex_sock_fd, &mutex_wait_on_descs);
 			MUTEX_TRACE_CNTR(mutex_trc_slp);
 			/*
 			 * the check for EINTR below is valid and should not be converted to an EINTR
 			 * wrapper macro, since it might be a timeout.
 			 */
-			while (-1 == (sel_stat =
-				select(mutex_sock_fd + 1, &mutex_wait_on_descs, (fd_set *)NULL, (fd_set *)NULL, &timeout)))
+			poll_fdlist[0].fd = mutex_sock_fd;
+			poll_fdlist[0].events = POLLIN;
+			poll_nfds = 1;
+			poll_timeout = (long)((timeout.tv_sec * MILLISECS_IN_SEC) +
+					DIVIDE_ROUND_UP(timeout.tv_usec, MICROSECS_IN_MSEC));
+			while (-1 == (sel_stat = poll(&poll_fdlist[0], poll_nfds, poll_timeout)))
 			{
 				if (EINTR == errno)
 				{	/* somebody interrupted me, reduce the timeout by half and continue */
@@ -464,9 +625,6 @@ static	enum cdb_sc mutex_long_sleep(mutex_struct_ptr_t addr, sgmnt_addrs *csa,
 				timeout.tv_usec = (gtm_tv_usec_t)(timeout_val % E_6);
 				MUTEX_DPRINT4("%d: Interrupted select, new timeout %d s %d us\n", process_id, timeout.tv_sec,
 					timeout.tv_usec);
-				/* the next line deals with the case that an interrupted select has changed mutex_wait_on_descs */
-				assertpro(FD_SETSIZE > mutex_sock_fd);
-				FD_SET(mutex_sock_fd, &mutex_wait_on_descs);
 				MUTEX_TRACE_CNTR(mutex_trc_slp);
 			}
 			if (1 == sel_stat) /* Somebody woke me up */
@@ -698,7 +856,7 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 	ABS_TIME 		atstart;
 	latch_t			local_crit_cycle = 0;
 	int4			local_stuck_cycle = 0;
-	uint4			timeout_count = 0;
+	uint4			timeout_count = 0, in_crit_pid = 0;
 #	ifdef CRIT_USE_PTHREAD_MUTEX
 	int			status;
 	ABS_TIME 		atend;
@@ -711,7 +869,6 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 	int 			n_queslots, redo_cntr;
 	mutex_struct_ptr_t 	addr;
 	mutex_que_entry_ptr_t	free_slot;
-	uint4			in_crit_pid;
 	jnlpool_addrs_ptr_t	save_jnlpool;
 	time_t			curr_time;
 	uint4			curr_time_uint4, next_alert_uint4;
@@ -754,6 +911,11 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 				INCR_GVSTATS_COUNTER(csa, cnl, n_crits_in_epch, 1);
 			local_crit_cycle = csa->critical->crit_cycle;
 			local_stuck_cycle = csa->critical->stuck_cycle;
+			if (mu_upgrade_in_prog && IS_ONLNRLBK_ACTIVE(csa))
+				RTS_ERROR_CSA_ABT(CSA_ARG(NULL) VARLSTCNT(7) ERR_REORGUPCNFLCT, 5,
+						LEN_AND_LIT("REORG -UPGRADE"),
+						LEN_AND_LIT("MUPIP ROLLBACK -ONLINE in progress"),
+						csa->nl->onln_rlbk_pid);
 			status = clock_gettime(CLOCK_REALTIME, &timeout);
 			if (0 != status)
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5,
@@ -764,7 +926,15 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 		switch (status)
 		{
 			case EOWNERDEAD:
-				mutex_clean_dead_owner(reg, cnl->in_crit);
+				in_crit_pid = cnl->in_crit;
+				if (!in_crit_pid)
+				{
+					SHM_READ_MEMORY_BARRIER;
+					in_crit_pid = cnl->recov_pid;
+				}
+				if (in_crit_pid)
+					mutex_clean_dead_owner(reg, in_crit_pid);
+				cnl->recov_pid = 0;
 				/* Record salvage event in db file header if applicable.
 				 * Take care not to do it for jnlpool which has no concept of a db cache.
 				 * In that case csa->hdr is NULL so check accordingly.
@@ -810,7 +980,7 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 				assert((MUTEX_CONST_TIMEOUT_VAL * 2) == MUTEXLCKALERT_INTERVAL);
 				if ((0 == (++timeout_count % 2)) && (csa->critical->crit_cycle == local_crit_cycle))
 				{
-					if (IS_REPL_INST_FROZEN)
+					if (IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)))
 						break;
 					if (0 != cnl->onln_rlbk_pid)
 					{
@@ -914,7 +1084,7 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 		assert(save_jnlpool == jnlpool);
 		assert(!csa->jnlpool || (csa->jnlpool == jnlpool));
 		assert((curr_time_uint4 <= next_alert_uint4) || (!csa->jnlpool || (csa->jnlpool == jnlpool)));
-		if ((curr_time_uint4 > next_alert_uint4) && !IS_REPL_INST_FROZEN)
+		if ((curr_time_uint4 > next_alert_uint4) && !IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)))
 		{	/* We've waited long enough and the Instance is not frozen - might be time to send MUTEXLCKALERT */
 			if (COMPSWAP_LOCK(&csa->critical->stuckexec.time_latch, next_alert_uint4, 0,
 				(curr_time_uint4 + MUTEXLCKALERT_INTERVAL), 0))
@@ -931,7 +1101,8 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 					if (local_crit_cycle == csa->critical->crit_cycle)
 					{	/* and things aren't moving */
 						assert(local_crit_cycle);
-						if (IS_REPL_INST_FROZEN)		/* recheck to minimize spurious reports */
+						/* recheck to minimize spurious reports */
+						if (IS_REPL_INST_FROZEN(TREF(defer_instance_freeze)))
 							continue;
 						if (0 == cnl->onln_rlbk_pid)
 						{	/* not rollback - send_msg after trace less likely to lose process */
@@ -984,6 +1155,11 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 		}
 		for (redo_cntr = MUTEX_MAX_WAIT_FOR_PROGRESS_CNTR; redo_cntr;)
 		{	/* loop on getting a slot on the queue - every time through, if crit is available, grab it and go */
+			if (mu_upgrade_in_prog && IS_ONLNRLBK_ACTIVE(csa))
+				RTS_ERROR_CSA_ABT(CSA_ARG(NULL) VARLSTCNT(7) ERR_REORGUPCNFLCT, 5,
+						LEN_AND_LIT("REORG -UPGRADE"),
+						LEN_AND_LIT("MUPIP ROLLBACK -ONLINE in progress"),
+						csa->nl->onln_rlbk_pid);
 			ONE_MUTEX_TRY(csa, addr, crash_count, process_id, mutex_lock_type,	/* lock type is MUTEX_LOCK_WRITE */
 				spins, (gtm_int64_t)-1, yields, (gtm_int64_t)-1, queue_sleeps, epoch_count, atstart, state);
 			free_slot = (mutex_que_entry_ptr_t)REMQHI((que_head_ptr_t)&addr->freehead);
@@ -1163,23 +1339,28 @@ void mutex_clean_dead_owner(gd_region* reg, uint4 holder_pid)
 	node_local		*cnl;
 	sgmnt_data_ptr_t	csd;
 	jnlpool_ctl_ptr_t	jpl;
-	int 			index1, index2,orig_index2;
+	uint 			index1, index2, orig_index2;
 	jnl_buffer_ptr_t	jbp;
 	uint4			start_freeaddr, orig_freeaddr;
 	seq_num			jnl_seqno, strm_seqno;
-	int			strmIndex;
-	seq_num			strmSeqno60;
+	int			strmIndex = 0;
+	seq_num			strmSeqno60 = 0UL;
 	jpl_phase2_in_prog_t	*lastJplCmt;
 	jbuf_phase2_in_prog_t	*lastJbufCmt;
+	enum mutex_cln_status	cln_srch_status = mutex_cln_invalid;
+	mutex_cln_info		cln_info = { .pid = 0, .seqno = 0 };
 
+	assert(holder_pid);
 	csa = &FILE_INFO(reg)->s_addrs;
 	send_msg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_MUTEXFRCDTERM, 3, holder_pid, DB_LEN_STR(reg));
 	cnl = csa->nl;
+	cnl->recov_pid = holder_pid;
+	SHM_WRITE_MEMORY_BARRIER;
 	cnl->in_crit = 0;
 	/* Mutex crash repaired, want to do write cache recovery, in case previous holder of crit had set
 	 * some cr->in_cw_set to a non-zero value. Not doing cache recovery could cause incorrect
-	 * GTMASSERTs in PIN_CACHE_RECORD macro in t_end/tp_tend.				BYPASSOK(GTMASSERT)
-	 * Take care not to do it for jnlpool (csa->hdr is NULL in that case) which has no concept of a db cache.
+	 * asserts in PIN_CACHE_RECORD macro in t_end/tp_tend. Take care not to do it for jnlpool
+	 * (csa->hdr is NULL in that case) which has no concept of a db cache.
 	 */
 	csd = csa->hdr;
 	assert((NULL != csd) || (NULL != jnlpool));
@@ -1189,6 +1370,12 @@ void mutex_clean_dead_owner(gd_region* reg, uint4 holder_pid)
 		/* This is a jnlpool. Check if a process in t_end/tp_tend was killed BEFORE
 		 * it incremented jpl->jnl_seqno. If so, undo any changes done in UPDATE_JPL_RSRV_WRITE_ADDR.
 		 */
+		/* In order to inform the process which will recover the region-crit associated with the failed commit about
+		 * whether to roll-back or roll-forward:
+		 */
+		set_mutex_cln_info(csa, holder_pid);
+		if (jnlpool->jnlpool_ctl->last_skip_jplwrites)
+			return;
 		jpl = jnlpool->jnlpool_ctl;
 		assert(NULL != jpl);
 		index1 = jpl->phase2_commit_index1;
@@ -1198,48 +1385,63 @@ void mutex_clean_dead_owner(gd_region* reg, uint4 holder_pid)
 		DECR_PHASE2_COMMIT_INDEX(index2, JPL_PHASE2_COMMIT_ARRAY_SIZE);
 		lastJplCmt = &jpl->phase2_commit_array[index2];
 		/* This process could have been killed during a commit in t_end/tp_tend.
-		 *	a) In the middle of Step CMT03 (see secshr_db_clnup.c for CMTxx steps)
+		 *	a) In the middle of Step CMT05 (see secshr_db_clnup.c for CMTxx steps)
 		 *		UPDATE_JPL_RSRV_WRITE_ADDR macro OR
-		 *	b) After Step CMT03 but before Step CMT07 finished.
+		 *	b) After step CMT05 but before step CMT07 finished.
+		 *	c) After step CMT07 but before step CMT08.
+		 *	d) After rel_lock (CMT08) but before any region reaches CMT09. This situation is not covered
+		 *	here; since in this case the process succesfully releases the journalpool lock, it won't cause
+		 *	a mutex_clean_dead_owner of that lock.
 		 * In either case, we need to reset jpl to what it was BEFORE Step CMT03 began i.e. roll back.
 		 * If the process gets killed AFTER CMT07 finishes, the transaction is rolled forward even
 		 *	if it means writing JRT_NULL and/or JRT_INCTN records in jnlpool and/or jnlbuff.
-		 * Note that there is still a small window of instructions after CMT07 is done but before
-		 *	CMT08 is done (for the first region in case of a multi-region TP transaction) if
-		 *	a process gets killed, we will roll forward the jnlpool but roll back the jnlbuff
-		 *	and so there would be a seqno with no corresponding journal records in the journal
-		 *	files. This is not easily handled so is left as a todo for the future.
 		 */
-		if ((index1 == orig_index2) || (lastJplCmt->process_id != holder_pid))
+		if ((index1 == orig_index2) || (lastJplCmt->process_id != holder_pid) || lastJplCmt->write_complete)
 		{
-			/* CMT02 < killed <= CMT03.
-			 * Kill could have happened before CMT03 finished so reset things.
-			 * This reset is a no-op if the kill happened even before CMT03 started.
+			/* CMT04 < killed <= CMT05.
+			 * Kill could have happened before CMT05 finished so reset things.
+			 * This reset is a no-op if the kill happened even before CMT05 started.
 			 * This is Case (a).
 			 */
+			/* Can't assert later than CMT04 since the process could have acquired
+			 * the lock and died before setting the jnlpool's cur_cmt_step flag, and
+			 * can't assert earlier than CMT05 because in mm mode we complete phase2 early
+			 * (and this section turns into a no-op in that case). That is: because in mm we
+			 * set lastJplCmt->write_complete much earlier, the window for entering here is
+			 * much wider, and that's correct: we don't have anything to roll back in the
+			 * journalpool, and this section is a no-op whenever the lastJplCmt is valid and complete.
+			 * In theory this assert can trigger if cnl->cur_cmt_step == DECL_CMT06b. In that case,
+			 * it will be worth examining if it is possible to provide any additional protection, as it
+			 * would indicate an inconsistent state between the journalpool and buffers, but protecting against
+			 * this is non-trivial.
+			 */
+			assert(CMT05 >= cnl->cur_cmt_step || CMT06b <= cnl->cur_cmt_step);
 			jpl->rsrv_write_addr = lastJplCmt->start_write_addr + lastJplCmt->tot_jrec_len;
-			assert(((lastJplCmt->jnl_seqno + 1) == jpl->jnl_seqno) || !lastJplCmt->jnl_seqno);
+			assert((((lastJplCmt->jnl_seqno + 1) == jpl->jnl_seqno) || !lastJplCmt->jnl_seqno)
+					|| ((lastJplCmt->jnl_seqno == jpl->jnl_seqno) && (CMT06b <= cnl->cur_cmt_step)));
 			jpl->lastwrite_len = lastJplCmt->tot_jrec_len;
 		}
 		else
 		{
+			SHM_READ_MEMORY_BARRIER;
+			assert(DECL_CMT05 <= cnl->cur_cmt_step);
 			assert((lastJplCmt->jnl_seqno == jpl->jnl_seqno) || ((lastJplCmt->jnl_seqno + 1) == jpl->jnl_seqno));
 			if (lastJplCmt->jnl_seqno == jpl->jnl_seqno)
 			{
-				/* CMT03 < killed < CMT07 */
+				/* CMT05 < killed < CMT07 */
+				assert(CMT08 > cnl->cur_cmt_step);
 				jpl->rsrv_write_addr = lastJplCmt->start_write_addr;
 				jpl->lastwrite_len = lastJplCmt->prev_jrec_len;
-				; /* similar layout as UPDATE_JPL_RSRV_WRITE_ADDR */
+				/* similar layout as UPDATE_JPL_RSRV_WRITE_ADDR */
 				jpl->phase2_commit_index2 = index2; /* remove last commit entry */
 			}
-			/* else : CMT07 < killed and so no rollback needed */
 		}
 	}
 	else
 	{
 		/* This is a database shm. Check if a process in t_end/tp_tend was killed BEFORE
-		 * Step CMT08 (see secshr_db_clnup.c) when it would have set cnl->update_underway_tn.
-		 * If so, undo any changes done in Step CMT06 (UPDATE_JBP_RSRV_FREEADDR).
+		 * Step CMT09 (see secshr_db_clnup.c) when it would have set cnl->update_underway_tn.
+		 * If so, undo any changes done in Step CMT06 (UPDATE_JRS_RSRV_FREEADDR).
 		 * Effectively rolling back the aborted commit in this region.
 		 */
 		assert((csd->trans_hist.early_tn == csd->trans_hist.curr_tn)
@@ -1248,9 +1450,10 @@ void mutex_clean_dead_owner(gd_region* reg, uint4 holder_pid)
 		assert(csd->trans_hist.early_tn >= cnl->update_underway_tn);
 		if (JNL_ENABLED(csd) && (csd->trans_hist.early_tn != csd->trans_hist.curr_tn))
 		{
-			/* i.e. Process was killed after CMT04 but before CMT12. It is represented as
-			 *	CMT04 < killed < CMT12
-			 */
+			/* i.e. Process was killed after CMT02 but before CMT13.
+			 * It is represented as: */
+			assert(((DECL_CMT02 < cnl->cur_cmt_step) && (CMT13 > cnl->cur_cmt_step))
+					|| (cnl->last_wcs_recover_tn == csd->trans_hist.curr_tn));
 			assert(NULL != csa->jnl);
 			assert(NULL != csa->jnl->jnl_buff);
 			jbp = csa->jnl->jnl_buff;
@@ -1262,103 +1465,143 @@ void mutex_clean_dead_owner(gd_region* reg, uint4 holder_pid)
 			lastJbufCmt = &jbp->phase2_commit_array[index2];
 			if (cnl->update_underway_tn != csd->trans_hist.curr_tn)
 			{
-				/* CMT04 < killed < CMT08.
+				/* CMT02 <= killed < CMT09.
 				 * ----------------------------------------------------------------
-				 * Roll-back the entire transaction's effect on this database file
+				 * If < CMT07, roll-back the entire transaction's effect on this database file
 				 * ----------------------------------------------------------------
 				 */
+				assert(((DECL_CMT02 <= cnl->cur_cmt_step) && (DECL_CMT09 >= cnl->cur_cmt_step))
+						|| (csd->trans_hist.curr_tn == cnl->last_wcs_recover_tn));
 				start_freeaddr = lastJbufCmt->start_freeaddr;
+				assert((csd->trans_hist.curr_tn != cnl->last_wcs_recover_tn)
+						|| ((index1 == orig_index2) || (lastJbufCmt->process_id != holder_pid)
+							|| (lastJbufCmt->curr_tn != csd->trans_hist.curr_tn)
+							|| lastJbufCmt->write_complete));
 				if ((index1 == orig_index2) || (lastJbufCmt->process_id != holder_pid)
-						|| (lastJbufCmt->curr_tn != csd->trans_hist.curr_tn))
+						|| (lastJbufCmt->curr_tn != csd->trans_hist.curr_tn)
+						|| lastJbufCmt->write_complete)
 				{
-					/* CMT04 < KILLED <= CMT06.
+					/* CMT01 <= KILLED <= CMT06.
 					 * Kill could have happened before CMT06 finished so reset things.
 					 * This reset is a no-op if the kill happened even before CMT06 started.
 					 */
 					SET_JBP_RSRV_FREEADDR(jbp, start_freeaddr + lastJbufCmt->tot_jrec_len);
+					csd->trans_hist.early_tn = csd->trans_hist.curr_tn; /* Undo CMT04 */
 				}
 				else
 				{
-					/* CMTO6 < killed < CMT08 */
-					assert(lastJbufCmt->curr_tn == csd->trans_hist.curr_tn);
-					/* CMT06 finished. So undo it as a whole */
-					assert(lastJbufCmt->process_id == holder_pid);
-					/* If CMT06a was in progress when the process was KILLED, then it is
-					 * possible jbp->freeaddr was updated as CMT16 (which is what CMT06a
-					 * executes) progressed. So undo that too. Likewise for dskaddr,
-					 * fsync_dskaddr etc. And finally reset rsrv_freeaddr.
-					 */
-					assert(jbp->fsync_dskaddr <= jbp->dskaddr);
-					orig_freeaddr = jbp->freeaddr;
-					if (orig_freeaddr > start_freeaddr)
+					assert(mutex_cln_invalid == cln_srch_status);
+					if (lastJbufCmt->replication)
+						cln_srch_status = get_mutex_cln_info(csa, holder_pid, lastJbufCmt->jnl_seqno,
+								&cln_info);
+					if ((mutex_cln_invalid == cln_srch_status)
+							|| ((mutex_cln_present == cln_srch_status)
+								&& (lastJbufCmt->jnl_seqno == cln_info.seqno)))
 					{
-						jbp->freeaddr = start_freeaddr;
-						jbp->free = start_freeaddr % jbp->size;
-					}
-					else
-						assert(jbp->free == (orig_freeaddr % jbp->size));
-
-					if (jbp->dskaddr > start_freeaddr)
-					{
-						assert(!GLOBAL_LATCH_HELD_BY_US(&jbp->io_in_prog_latch));
-						grab_latch(&jbp->io_in_prog_latch, GRAB_LATCH_INDEFINITE_WAIT, WS_35, csa);
-						/* Fix jbp->dskaddr & jbp->dsk while holding io latch */
-						assert(orig_freeaddr > start_freeaddr);
-						jbp->dskaddr = start_freeaddr;
-						jbp->dsk = start_freeaddr % jbp->size;
-						/* Setting jbp->dskaddr to start_freeaddr is not enough.
-						 * We also need to re-read the partial filesystem-block-size
-						 * aligned block of data that precedes the new jbp->dskaddr
-						 * since that part is most likely no longer in the jnl buffer
-						 * (have been overwritten by the current aborted tn's jnl records).
-						 * We can try and optimize this by avoiding setting
-						 * jbp->re_read_dskaddr in case no overwrite happened. But it is
-						 * not straightforward to detect that and the risk is journal
-						 * file corruption. Given "mutex_salvage" is a rare occurrence,
-						 * it is safer to re-read unconditionally.
+						/* CMTO6 < killed < CMT07, or < CMT09 and rolling back anyway because
+						 * we can't find additional information in the mutex_cln_ctl struct.
 						 */
-						jbp->re_read_dskaddr = start_freeaddr;
-						rel_latch(&jbp->io_in_prog_latch);
-						if (jbp->fsync_dskaddr > start_freeaddr)
+						assert(lastJbufCmt->curr_tn == csd->trans_hist.curr_tn);
+						/* CMT06 finished. So undo it as a whole */
+						assert(lastJbufCmt->process_id == holder_pid);
+						/* If CMT06a was in progress when the process was KILLED, then it is
+						 * possible jbp->freeaddr was updated as CMT16 (which is what CMT06a
+						 * executes) progressed. So undo that too. Likewise for dskaddr,
+						 * fsync_dskaddr etc. And finally reset rsrv_freeaddr.
+						 */
+						assert(jbp->fsync_dskaddr <= jbp->dskaddr);
+						orig_freeaddr = jbp->freeaddr;
+						if (orig_freeaddr > start_freeaddr)
 						{
-							/* Fix jbp->fsync_dskaddr while holding fsync io latch */
-							assert(!GLOBAL_LATCH_HELD_BY_US(&jbp->fsync_in_prog_latch));
-							grab_latch(&jbp->fsync_in_prog_latch, GRAB_LATCH_INDEFINITE_WAIT,
-								WS_36, csa);
-							jbp->fsync_dskaddr = start_freeaddr;
-							rel_latch(&jbp->fsync_in_prog_latch);
+							jbp->freeaddr = start_freeaddr;
+							jbp->free = start_freeaddr % jbp->size;
 						}
+						else
+							assert(jbp->free == (orig_freeaddr % jbp->size));
+						if (jbp->dskaddr > start_freeaddr)
+						{
+							assert(!GLOBAL_LATCH_HELD_BY_US(&jbp->io_in_prog_latch));
+							grab_latch(&jbp->io_in_prog_latch, GRAB_LATCH_INDEFINITE_WAIT, WS_35, csa);
+							/* Fix jbp->dskaddr & jbp->dsk while holding io latch */
+							assert(orig_freeaddr > start_freeaddr);
+							jbp->dskaddr = start_freeaddr;
+							jbp->dsk = start_freeaddr % jbp->size;
+							/* Setting jbp->dskaddr to start_freeaddr is not enough.
+							 * We also need to re-read the partial filesystem-block-size
+							 * aligned block of data that precedes the new jbp->dskaddr
+							 * since that part is most likely no longer in the jnl buffer
+							 * (have been overwritten by the current aborted tn's jnl records).
+							 * We can try and optimize this by avoiding setting
+							 * jbp->re_read_dskaddr in case no overwrite happened. But it is
+							 * not straightforward to detect that and the risk is journal
+							 * file corruption. Given "mutex_salvage" is a rare occurrence,
+							 * it is safer to re-read unconditionally.
+							 */
+							jbp->re_read_dskaddr = start_freeaddr;
+							rel_latch(&jbp->io_in_prog_latch);
+							if (jbp->fsync_dskaddr > start_freeaddr)
+							{
+								/* Fix jbp->fsync_dskaddr while holding fsync io latch */
+								assert(!GLOBAL_LATCH_HELD_BY_US(&jbp->fsync_in_prog_latch));
+								grab_latch(&jbp->fsync_in_prog_latch, GRAB_LATCH_INDEFINITE_WAIT,
+										WS_36, csa);
+								jbp->fsync_dskaddr = start_freeaddr;
+								rel_latch(&jbp->fsync_in_prog_latch);
+							}
+						}
+						else
+							assert(jbp->dsk == (jbp->dskaddr % jbp->size));
+						/* "jnl_write_phase2" is never called with JRT_EPOCH (see assert there
+						 * at function entry of possible rectype values and EPOCH is not in that
+						 * list). Therefore we are guaranteed a "jnl_write_epoch_rec" call never
+						 * happened since the first call to "jnl_write_reserve" happened in this
+						 * transaction. Therefore no UNDO of the effects of "jnl_write_epoch_rec"
+						 * needed here (e.g. jbp->post_epoch_freeaddr).
+						 */
+						assert(jbp->post_epoch_freeaddr <= start_freeaddr);
+						SET_JBP_RSRV_FREEADDR(jbp, start_freeaddr); /* see corresponding
+											     * SHM_READ_MEMORY_BARRIER in
+											     * "jnl_phase2_cleanup".
+											     */
+						jbp->phase2_commit_index2 = index2; /* remove last commit entry */
+						csd->trans_hist.early_tn = csd->trans_hist.curr_tn; /* Undo CMT04 */
+					} else
+					{
+						/* CMT07 < killed < CMT09.
+						 * -------------------------------------------------------------------
+						 * Roll-forward the entire transaction's effect on this database file
+						 * -------------------------------------------------------------------
+						 * In case process got killed before CMT09 occurred, redo it.
+						 * If the process got killed after CMT09, the below redo is a no-op.
+						 */
+						/* CMT10 redo : start */
+						jnl_seqno = lastJbufCmt->jnl_seqno + 1;
+						strm_seqno = lastJbufCmt->strm_seqno;
+						/* If "strm_seqno" is 0, we are guaranteed this is not a supplementary
+						 * instance (i.e. "supplementary" variable in t_end/tp_tend is FALSE).
+						 */
+						if (strm_seqno)
+						{
+							strmIndex = GET_STRM_INDEX(strm_seqno);
+							strmSeqno60 = GET_STRM_SEQ60(strm_seqno) + 1;
+						}
+						SET_REG_SEQNO(csa, jnl_seqno, strm_seqno, strmIndex, strmSeqno60, SKIP_ASSERT_TRUE);
+						/* CMT10 redo : end */
+						csd->trans_hist.curr_tn = csd->trans_hist.early_tn; /* Redo CMT12 */
 					}
-					else
-						assert(jbp->dsk == (jbp->dskaddr % jbp->size));
-
-					/* "jnl_write_phase2" is never called with JRT_EPOCH (see assert there
-					 * at function entry of possible rectype values and EPOCH is not in that
-					 * list). Therefore we are guaranteed a "jnl_write_epoch_rec" call never
-					 * happened since the first call to "jnl_write_reserve" happened in this
-					 * transaction. Therefore no UNDO of the effects of "jnl_write_epoch_rec"
-					 * needed here (e.g. jbp->post_epoch_freeaddr).
-					 */
-					assert(jbp->post_epoch_freeaddr <= start_freeaddr);
-					SET_JBP_RSRV_FREEADDR(jbp, start_freeaddr);; /* see corresponding
-					 * SHM_READ_MEMORY_BARRIER in
-					 * "jnl_phase2_cleanup".
-					 */
-					jbp->phase2_commit_index2 = index2; /* remove last commit entry */
 				}
-				csd->trans_hist.early_tn = csd->trans_hist.curr_tn; /* Undo CMT04 */
 				/* CMT07 is jnlpool related, so no undo done here (in db mutex_salvage) for that */
 			}
 			else
 			{
-				/* CMT08 < killed < CMT12.
+				/* CMT09 < killed < CMT13.
 				 * -------------------------------------------------------------------
 				 * Roll-forward the entire transaction's effect on this database file
 				 * -------------------------------------------------------------------
 				 * In case process got killed before CMT09 occurred, redo it.
 				 * If the process got killed after CMT09, the below redo is a no-op.
 				 */
-				/* CMT09 redo : start */
+				/* CMT10 redo : start */
 				jnl_seqno = lastJbufCmt->jnl_seqno + 1;
 				strm_seqno = lastJbufCmt->strm_seqno;
 				/* If "strm_seqno" is 0, we are guaranteed this is not a supplementary
@@ -1370,7 +1613,7 @@ void mutex_clean_dead_owner(gd_region* reg, uint4 holder_pid)
 					strmSeqno60 = GET_STRM_SEQ60(strm_seqno) + 1;
 				}
 				SET_REG_SEQNO(csa, jnl_seqno, strm_seqno, strmIndex, strmSeqno60, SKIP_ASSERT_TRUE);
-				/* CMT09 redo : end */
+				/* CMT10 redo : end */
 				csd->trans_hist.curr_tn = csd->trans_hist.early_tn; /* Redo CMT12 */
 			}
 		}
