@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2021 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -76,6 +76,12 @@ GBLREF	boolean_t		holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];
 GBLREF	boolean_t		is_rcvr_server;
 GBLREF	char			repl_instfilename[];
 GBLREF	gd_addr			*repl_inst_from_gld;
+GBLREF	uint4			process_id;
+#ifdef DEBUG
+GBLREF	bool			only_usr_jnlpool_flush;
+#endif
+GBLREF int			in_rlbk;
+GBLREF boolean_t		repl_inst_rlbk_fd;
 
 ZOS_ONLY(error_def(ERR_BADTAG);)
 error_def(ERR_LOGTOOLONG);
@@ -88,6 +94,7 @@ error_def(ERR_REPLINSTFMT);
 error_def(ERR_REPLINSTNOHIST);
 error_def(ERR_REPLINSTOPEN);
 error_def(ERR_REPLINSTREAD);
+error_def(ERR_REPLINSTRECR);
 error_def(ERR_REPLINSTSEQORD);
 error_def(ERR_REPLINSTUNDEF);
 error_def(ERR_REPLINSTWRITE);
@@ -149,11 +156,9 @@ struct gd_addr_struct *repl_inst_get_name(char *fn, unsigned int *fn_len, unsign
 				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_REPLINSTACC, 2, log_nam.len, log_nam.addr,
 					ERR_TEXT, 2, RTS_ERROR_LITERAL("from global directory"));
 		}
-	} else if ('\0' == repl_instfilename[0])
-	{	/* save information from first instance for syslog */
+	} else if (!inst_from_gld && ('\0' == repl_instfilename[0]))
+	{	/* save expanded environment variable gtm_repl_instance for gtmpcat */
 		memcpy(repl_instfilename, fn, *fn_len + 1);	/* include null from get_full_path */
-		if (inst_from_gld)
-			repl_inst_from_gld = gd_local;		/* got from global directory */
 	}
 	if (FALSE == ret)
 		return NULL;
@@ -175,7 +180,7 @@ struct gd_addr_struct *repl_inst_get_name(char *fn, unsigned int *fn_len, unsign
  */
 void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 {
-	int			status, fd;
+	int			status, fd, oflag;
 	size_t			actual_readlen;
 	unix_db_info		*udi;
 	gd_region		*reg;
@@ -207,9 +212,14 @@ void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 		assert(udi->grabbed_ftok_sem || ((jnlpool && jnlpool->jnlpool_ctl) && udi->s_addrs.now_crit)
 				|| jgbl.mur_rollback || ((NULL != jnlpool) && in_repl_inst_edit));
 	}
-	OPENFILE_CLOEXEC(fn, O_RDONLY, fd);
-	if (FD_INVALID == fd)
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+	oflag = in_rlbk ? O_RDWR : O_RDONLY; /* for onrlbk hold fd for repl_inst_write() so use O_RDWR */
+	if (!in_rlbk || (in_rlbk && (FD_INVALID == repl_inst_rlbk_fd)))
+	{
+		OPENFILE_CLOEXEC(fn, oflag, fd);
+		if (FD_INVALID == fd)
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+	} else
+		fd = repl_inst_rlbk_fd;
 	assert(0 < buflen);
 	if (0 != offset)
 	{
@@ -274,10 +284,15 @@ void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_REPLINSTREAD, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn),
 				status);
 	}
-	CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
-	assert(0 == status);
-	if (0 != status)
-		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+	if (in_rlbk)
+		repl_inst_rlbk_fd = fd;
+	else
+	{
+		CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
+		assert(0 == status);
+		if (0 != status)
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+	}
 }
 
 /* Description:
@@ -292,10 +307,22 @@ void	repl_inst_read(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
  */
 void	repl_inst_write(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 {
-	int		status, fd, oflag;
-	unix_db_info	*udi;
-	gd_region	*reg;
-	ZOS_ONLY(int	realfiletag;)
+	int			status, fd, oflag;
+	unix_db_info		*udi;
+	gd_region		*reg;
+	ZOS_ONLY(int		realfiletag;)
+#	ifdef DEBUG
+	int			i;
+	int4			latch_pid;
+	size_t			gtmsrc_lcl_start, gtmsrc_lcl_top;
+	gtmsource_local_ptr_t	gtmsourcelocal_ptr;
+#	endif
+	boolean_t		replfilegone, recreatereplfile;
+#define CPBUFSIZE		4096
+	char			cpbuf[CPBUFSIZE];
+	ssize_t			bytesread, byteswritten;
+	off_t			lseekreturn;
+	int			save_errno;
 
 	assert(!jgbl.mur_rollback || !jgbl.mur_options_forward); /* ROLLBACK -FORWARD should not call this function */
 	/* Assert that except for MUPIP REPLIC -INSTANCE_CREATE or -EDITINSTANCE, all callers hold the FTOK semaphore on the
@@ -310,7 +337,7 @@ void	repl_inst_write(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 	 * accordingly.
 	 */
 	assert(((NULL == jnlpool) || (NULL == jnlpool->jnlpool_dummy_reg)) || (NULL == recvpool.recvpool_dummy_reg)
-		|| (NULL != jnlpool) && (jnlpool->jnlpool_dummy_reg == recvpool.recvpool_dummy_reg));
+		|| ((NULL != jnlpool) && (jnlpool->jnlpool_dummy_reg == recvpool.recvpool_dummy_reg)));
 	DEBUG_ONLY(
 		reg = jnlpool ? jnlpool->jnlpool_dummy_reg : NULL;
 		if (NULL == reg)
@@ -319,40 +346,138 @@ void	repl_inst_write(char *fn, off_t offset, sm_uc_ptr_t buff, size_t buflen)
 	assert(((NULL == reg) && (in_repl_inst_create || in_repl_inst_edit))
 		|| ((NULL != reg) && !in_repl_inst_create && !in_repl_inst_edit)
 		|| ((NULL != reg) && (NULL != jnlpool) && in_repl_inst_edit));
-	DEBUG_ONLY(
-		if (NULL != reg)
-		{
-			udi = FILE_INFO(reg);
-			assert(udi->grabbed_ftok_sem || (jnlpool && (NULL != jnlpool->jnlpool_ctl) && udi->s_addrs.now_crit)
-				|| jgbl.mur_rollback || ((NULL != jnlpool) && in_repl_inst_edit));
-		}
-	)
-	oflag = O_RDWR;
-	if (in_repl_inst_create)
-		oflag |= (O_CREAT | O_EXCL);
-	OPENFILE3_CLOEXEC(fn, oflag, 0666, fd);
-	if (FD_INVALID == fd)
+#	ifdef DEBUG
+	if (NULL != reg)
 	{
-		if (!in_repl_inst_create)
-			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
-		else
-			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCREATE, 2, LEN_AND_STR(fn), errno);
+		udi = FILE_INFO(reg);
+		assert(udi->grabbed_ftok_sem
+				|| (jnlpool && (NULL != jnlpool->jnlpool_ctl) && udi->s_addrs.now_crit)
+				|| jgbl.mur_rollback
+				|| ((NULL != jnlpool) && in_repl_inst_edit)
+				|| ((jnlpool && jnlpool->gtmsource_local)
+					&& (process_id == jnlpool->gtmsource_local->gtmsource_srv_latch.u.parts.latch_pid)));
+		for (i = 0; i < NUM_GTMSRC_LCL; i++)
+		{
+			gtmsrc_lcl_start = ((REPL_INST_HDR_SIZE + (SIZEOF(gtmsrc_lcl) * i)));
+			gtmsrc_lcl_top = gtmsrc_lcl_start + SIZEOF(gtmsrc_lcl);
+			if ((offset < gtmsrc_lcl_top) && ((offset + buflen) > gtmsrc_lcl_start))
+			{
+				gtmsourcelocal_ptr = NULL;
+				latch_pid = 0;
+				if (jnlpool && jnlpool->gtmsource_local_array)
+				{
+					gtmsourcelocal_ptr = &jnlpool->gtmsource_local_array[i];
+					latch_pid = gtmsourcelocal_ptr->gtmsource_srv_latch.u.parts.latch_pid;
+				}
+				if (jgbl.mur_rollback)
+				{	/* Assert that we properly exclude everyone we need to. */
+					assert(only_usr_jnlpool_flush || (gtmsourcelocal_ptr && (process_id == latch_pid)));
+				} else if (!jnlpool)
+				{
+					/* If there's no journalpool, assert that we have the ftok on the file */
+					assert(udi->grabbed_ftok_sem);
+				} else if (in_repl_inst_edit)
+				{
+					if (!gtmsourcelocal_ptr || (process_id != latch_pid))
+					{
+						assert(udi->grabbed_ftok_sem);
+						assert((NULL == jnlpool->repl_inst_filehdr)
+								|| (INVALID_SEMID == jnlpool->repl_inst_filehdr->jnlpool_semid)
+								|| (INVALID_SHMID == jnlpool->repl_inst_filehdr->jnlpool_shmid));
+					}
+				} else if (!only_usr_jnlpool_flush)
+				{	/* Otherwise, if there are jnlpool users, assert that we've got the appropriate lock */
+					assert(gtmsourcelocal_ptr && (process_id == latch_pid));
+				}
+			}
+		}
 	}
+#	endif
+	replfilegone = (access(fn, F_OK) != 0);
+	/* RLBKEXITING == in_rlbk means we are about to exit rollback so re-create repl inst file if needed */
+	recreatereplfile = ((RLBKEXITING == in_rlbk) && replfilegone && (FD_INVALID != repl_inst_rlbk_fd));
+	oflag = O_RDWR;
+	if ((in_repl_inst_create) || recreatereplfile )
+		oflag |= (O_CREAT | O_EXCL);
+	if (!in_rlbk || (in_rlbk && (FD_INVALID == repl_inst_rlbk_fd)) || recreatereplfile)
+	{
+		OPENFILE3_CLOEXEC(fn, oflag, 0666, fd);
+		if (FD_INVALID == fd)
+		{
+			if (!in_repl_inst_create)
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+			else
+				RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCREATE, 2, LEN_AND_STR(fn), errno);
+		}
+	}
+	else
+		fd = repl_inst_rlbk_fd;
 #ifdef __MVS__
 	if (-1 == (in_repl_inst_create ? gtm_zos_set_tag(fd, TAG_BINARY, TAG_NOTTEXT, TAG_FORCE, &realfiletag) :
 					 gtm_zos_tag_to_policy(fd, TAG_BINARY, &realfiletag)))
 		TAG_POLICY_GTM_PUTMSG(fn, errno, realfiletag, TAG_BINARY);
 #endif
 	assert(0 < buflen);
+	if (recreatereplfile && (FD_INVALID != repl_inst_rlbk_fd))
+	{
+		/* For the online_rollback/orlbkinstrecr test, which only runs in DEBUG, ensure we have up-to-date
+		 * ipc info otherwise other processes might have issues exiting.
+		 */
+#       	ifdef DEBUG
+		REPL_INST_LSEEKWRITE(fd, offset, buff, buflen, status);
+		REPL_INST_LSEEKWRITE(repl_inst_rlbk_fd, offset, buff, buflen, status);
+#		endif
+		/* Go to the beginning using the file descriptor we have held open */
+		lseekreturn = lseek(repl_inst_rlbk_fd, 0, SEEK_SET);
+		save_errno = errno;
+lseekfail:
+		if (-1 != lseekreturn)
+		{
+			for (;;)
+			{	/* read from the file descriptor we have held open ... */
+				DOREADRL(repl_inst_rlbk_fd, cpbuf, CPBUFSIZE, bytesread);
+				save_errno = errno;
+bytesreadfail:
+				if (bytesread > 0)
+				{
+					/* write to the recreated repl instance file */
+					DOWRITERL(fd, cpbuf, bytesread, byteswritten);
+					save_errno = errno;
+byteswrittenfail:
+					if (bytesread != byteswritten)
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTRECR, 2, LEN_AND_STR(fn),
+							save_errno);
+				} else if (0 == bytesread)
+						break; /* No more bytes to copy so we are done */
+					else
+						RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTRECR, 2, LEN_AND_STR(fn),
+							save_errno);
+			}
+		} else
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTRECR, 2, LEN_AND_STR(fn), save_errno);
+		CLOSEFILE_RESET(repl_inst_rlbk_fd, status);    /* resets "repl_inst_rlbk_fd" to FD_INVALID */
+	}
 	REPL_INST_LSEEKWRITE(fd, offset, buff, buflen, status);
 	assert(0 == status);
 	if (0 != status)
 		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(7) ERR_REPLINSTWRITE, 4, buflen, (qw_off_t *)&offset, LEN_AND_STR(fn),
 			status);
-	CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
-	assert(0 == status);
-	if (0 != status)
-		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+	if (in_rlbk)
+		repl_inst_rlbk_fd = fd;
+	else
+	{
+		CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
+		assert(0 == status);
+		if (0 != status)
+		{
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+			assert(FALSE);
+			/* Dummy gotos satisfy the compiler that labels used for breakpoints testing are legit */
+			goto lseekfail;
+			goto bytesreadfail;
+			goto byteswrittenfail;
+		}
+	}
 }
 
 /* Description:
@@ -387,17 +512,27 @@ void	repl_inst_sync(char *fn)
 		assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl) && udi->s_addrs.now_crit);
 	)
 	oflag = O_RDWR;
-	OPENFILE3_CLOEXEC(fn, oflag, 0666, fd);
-	if (FD_INVALID == fd)
-		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+	if (!in_rlbk || (in_rlbk && (FD_INVALID == repl_inst_rlbk_fd)))
+	{
+		OPENFILE3_CLOEXEC(fn, oflag, 0666, fd);
+		if (FD_INVALID == fd)
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTOPEN, 2, LEN_AND_STR(fn), errno);
+	}
+	else
+		fd = repl_inst_rlbk_fd;
 	GTM_REPL_INST_FSYNC(fd, status);
 	assert(0 == status);
 	if (0 != status)
 		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(8) ERR_SYSCALL, 5, RTS_ERROR_LITERAL("fsync()"), CALLFROM, errno);
-	CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
-	assert(0 == status);
-	if (0 != status)
-		RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+	if (in_rlbk)
+		repl_inst_rlbk_fd = fd;
+	else
+	{
+		CLOSEFILE_RESET(fd, status);	/* resets "fd" to FD_INVALID */
+		assert(0 == status);
+		if (0 != status)
+			RTS_ERROR_CSA_ABT(NULL, VARLSTCNT(5) ERR_REPLINSTCLOSE, 2, LEN_AND_STR(fn), status);
+	}
 }
 
 /* Description:
@@ -602,7 +737,8 @@ int4	repl_inst_histinfo_find_seqno(seq_num seqno, int4 strm_idx, repl_histinfo *
 	assert(0 != seqno);
 	inst_hdr = jnlpool->repl_inst_filehdr;
 	assert(NULL != inst_hdr);
-	assert((INVALID_SUPPL_STRM == strm_idx) || inst_hdr->is_supplementary && (0 <= strm_idx) && (MAX_SUPPL_STRMS > strm_idx));
+	assert((INVALID_SUPPL_STRM == strm_idx)
+			|| (inst_hdr->is_supplementary && (0 <= strm_idx) && (MAX_SUPPL_STRMS > strm_idx)));
 	assert(inst_hdr->num_histinfo <= inst_hdr->num_alloc_histinfo);
 	if (INVALID_SUPPL_STRM == strm_idx)
 		histnum = inst_hdr->num_histinfo - 1;
@@ -701,7 +837,7 @@ void	repl_inst_histinfo_add(repl_histinfo *histinfo)
 	 */
 	assert((0 < histinfo_num) || (0 == strm_idx));
 	is_supplementary = jnlpool->repl_inst_filehdr->is_supplementary;
-	assert(!is_supplementary && (0 == strm_idx) || (is_supplementary && (0 <= strm_idx) && (MAX_SUPPL_STRMS > strm_idx)));
+	assert((!is_supplementary && (0 == strm_idx)) || (is_supplementary && (0 <= strm_idx) && (MAX_SUPPL_STRMS > strm_idx)));
 	/* If -updateresync is specified and instance is not supplementary, then there better be NO history records */
 	assert((HISTINFO_TYPE_UPDRESYNC != histinfo->history_type) || is_supplementary || (0 == histinfo_num));
 	assert(NULL != jnlpool->jnlpool_ctl);
@@ -776,7 +912,7 @@ void	repl_inst_histinfo_add(repl_histinfo *histinfo)
 		if (histinfo_strm_seqno == prev_strm_seqno)
 		{
 			start_seqno_equal = (histinfo->start_seqno == last_strm_histinfo->start_seqno);
-			if (histinfo_strm_seqno && strm_idx || start_seqno_equal)
+			if ((histinfo_strm_seqno && strm_idx) || start_seqno_equal)
 			{
 				assert(prev_histinfo_num > last_strm_histinfo->prev_histinfo_num);
 				prev_histinfo_num = last_strm_histinfo->prev_histinfo_num;
@@ -833,7 +969,7 @@ void	repl_inst_histinfo_add(repl_histinfo *histinfo)
 	for (idx = 0; idx < MAX_SUPPL_STRMS; idx++)
 	{
 		assert((jnlpool->repl_inst_filehdr->last_histinfo_num[idx] < histinfo_num)
-			|| (idx == strm_idx) && (jnlpool->repl_inst_filehdr->last_histinfo_num[idx] == histinfo_num));
+			|| ((idx == strm_idx) && (jnlpool->repl_inst_filehdr->last_histinfo_num[idx] == histinfo_num)));
 		histinfo->last_histinfo_num[idx] = jnlpool->repl_inst_filehdr->last_histinfo_num[idx];
 	}
 	if (strm_histinfo_num == histinfo_num)
@@ -1150,11 +1286,16 @@ void	repl_inst_flush_gtmsrc_lcl()
 	int4			index;
 	off_t			offset;
 	gtmsrc_lcl_ptr_t	gtmsrclcl_ptr;
+#	ifdef DEBUG
+	seq_num			tmp_rseqno, tmp_cseqno;
+	tmp_rseqno = jnlpool->gtmsource_local->read_jnl_seqno;
+	tmp_cseqno = jnlpool->gtmsource_local->connect_jnl_seqno;
+#	endif
 
 	assert(NULL != jnlpool);
 	udi = FILE_INFO(jnlpool->jnlpool_dummy_reg);
 	assert(!jgbl.mur_rollback); /* Rollback should never reach here */
-	assert(udi->s_addrs.now_crit);
+	assert(process_id == jnlpool->gtmsource_local->gtmsource_srv_latch.u.parts.latch_pid);
 	assert(NULL != jnlpool->gtmsource_local);
 	index = jnlpool->gtmsource_local->gtmsrc_lcl_array_index;
 	assert(0 <= index);
@@ -1167,7 +1308,17 @@ void	repl_inst_flush_gtmsrc_lcl()
 	 */
 	COPY_GTMSOURCELOCAL_TO_GTMSRCLCL(jnlpool->gtmsource_local, gtmsrclcl_ptr);
 	offset = REPL_INST_HDR_SIZE + (SIZEOF(gtmsrc_lcl) * (off_t)index);
+#	ifdef DEBUG
+	if (WBTEST_ENABLED(WBTEST_FLUSH_FH))
+	{	/* Sleep for a long enough time to provide substantial opportunity for any competitors for these fields to try
+		 * to access/modify them and to test the concurrency control system that protects them.
+		 */
+		LONG_SLEEP(5);
+	}
+#	endif
 	repl_inst_write(udi->fn, offset, (sm_uc_ptr_t)gtmsrclcl_ptr, SIZEOF(gtmsrc_lcl));
+	assert((tmp_rseqno == gtmsrclcl_ptr->resync_seqno) && (tmp_rseqno == jnlpool->gtmsource_local->read_jnl_seqno));
+	assert((tmp_cseqno == gtmsrclcl_ptr->connect_jnl_seqno) && (tmp_cseqno == jnlpool->gtmsource_local->connect_jnl_seqno));
 	jnlpool->gtmsource_local->last_flush_resync_seqno = jnlpool->gtmsource_local->read_jnl_seqno;
 }
 
