@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information	*
+ * Copyright (c) 2001-2024 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -47,7 +47,6 @@
 #include "wcs_recover.h"
 #include "gtm_string.h"
 #include "have_crit.h"
-#include "gds_blk_downgrade.h"
 #include "deferred_signal_handler.h"
 #include "memcoherency.h"
 #include "wbox_test_init.h"
@@ -137,7 +136,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 	boolean_t		need_jnl_sync, queue_empty, got_lock, bmp_status, do_asyncio, wtfini_called_once;
 	cache_que_head_ptr_t	ahead, whead;
 	cache_state_rec_ptr_t	csr, csrfirst;
-	int4			err_status = 0, n, n1, n2, max_ent, max_writes, save_errno;
+	int4			err_status = 0, n = INTERLOCK_FAIL, n1, n2, max_ent, max_writes, save_errno;
 	size_t			size ;
 	jnl_buffer_ptr_t	jb;
 	jnl_private_control	*jpc;
@@ -154,16 +153,16 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 	boolean_t		is_mm, was_crit;
 	uint4			curr_wbox_seq_num;
 	int			try_sleep, rc;
-	gd_region		*sav_cur_region;
-	sgmnt_addrs		*sav_cs_addrs;
-	sgmnt_data		*sav_cs_data;
-	jnlpool_addrs_ptr_t	sav_jnlpool;
+	gd_region		*sav_cur_region = NULL;
+	sgmnt_addrs		*sav_cs_addrs = NULL;
+	sgmnt_data		*sav_cs_data = NULL;
+	jnlpool_addrs_ptr_t	sav_jnlpool = NULL;
 	jnlpool_addrs_ptr_t	local_jnlpool;	/* needed by INST_FREEZE_ON_ERROR_POLICY_CSA */
 	intrpt_state_t		prev_intrpt_state;
 	char			*in, *out;
 	int			in_len;
 	int4			gtmcrypt_errno = 0;
-	gd_segment		*seg;
+	gd_segment		*seg = FALSE;
 	boolean_t		use_new_key, skip_in_trans, skip_sync, sync_keys;
 	que_ent_ptr_t		next, prev;
 	void_ptr_t		retcsrptr;
@@ -248,6 +247,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 	{
 		queue_empty = TRUE;
 		n1 = 1; /* set to a non-zero value so dbsync timer canceling (if needed) can happen */
+		n2 = 0;
 		goto writes_completed; /* to avoid unnecessary IF checks in the more common case (BG) */
 	}
 	ahead = &csa->acc_meth.bg.cache_state->cacheq_active;
@@ -284,7 +284,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				if (grab_latch(&ahead->latch, WT_LATCH_TIMEOUT_SEC, WS_26, csa))
 				{
 					cr = cr2flush;
-					csr = (cache_state_rec_ptr_t)((sm_uc_ptr_t)cr + SIZEOF(cr->blkque));
+					csr = CR2CSR(cr);
 					if (csr->dirty && !csr->epid && csr->state_que.fl)
 					{	/* Now that we know csr is in the active queue, remove it. */
 						retcsrptr = remqh((que_ent_ptr_t)((sm_uc_ptr_t)&csr->state_que
@@ -319,7 +319,10 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 						csr = NULL; /* no longer on the active queue */
 					rel_latch(&ahead->latch);
 				} else
+				{
+					cr = NULL;
 					csr = NULL; /* did not get the lock */
+				}
 				--fast_lock_count;
 				assert(0 <= fast_lock_count);
 			}
@@ -333,7 +336,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				BG_TRACE_PRO_ANY(csa, wcb_wtstart_lckfail1);
 				break;
 			}
-			cr = (cache_rec_ptr_t)((sm_uc_ptr_t)csr - SIZEOF(cr->blkque));
+			cr = CSR2CR(csr);
 		}
 		if (NULL == csr)
 			break;				/* the queue is empty */
@@ -344,28 +347,28 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			assert(!keep_buff_lock);
 			/* the if check and lock clear is for PRO just in case */
 			if (keep_buff_lock)
-				CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+				CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 			REINSERT_CR_AT_TAIL(csr, ahead, n, csa, csd, wcb_wtstart_lckfail2);
 			if (INTERLOCK_FAIL == n)
 				err_status = ERR_DBCCERR;
 			break;
 		}
 		assert(!CR_NOT_ALIGNED(cr, cr_lo) && !CR_NOT_IN_RANGE(cr, cr_lo, cr_hi));
-		if (CR_BLKEMPTY == csr->blk)
+		if (CR_BLKEMPTY == cr->blk)
 		{	/* must be left by t_commit_cleanup - removing it from the queue and the following
 			 * completes the cleanup
 			 */
 			WCS_OPS_TRACE(csa, process_id, wcs_ops_wtstart4, cr->blk, GDS_ANY_ABS2REL(csa,cr), cr->dirty, 0, 0);
-			assert(0 != csr->dirty);
-			assert(csr->data_invalid);
-			csr->data_invalid = FALSE;
-			csr->dirty = 0;
+			assert(0 != cr->dirty);
+			assert(cr->data_invalid);
+			cr->data_invalid = FALSE;
+			cr->dirty = 0;
 			assert(!keep_buff_lock);
 			/* the if check and lock clear is for PRO just in case */
 			if (keep_buff_lock)
-				CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+				CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 			ADD_ENT_TO_FREE_QUE_CNT(cnl);
-			assert(LATCH_CLEAR == WRITE_LATCH_VAL(csr));
+			assert(LATCH_CLEAR == WRITE_LATCH_VAL(cr));
 			queue_empty = !SUB_ENT_FROM_ACTIVE_QUE_CNT(cnl);
 			continue;
 		}
@@ -373,10 +376,10 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 		if (JNL_ENABLED(csd))
 		{	/* this looks to be a long lock and hence should use a mutex */
 			jb = jpc->jnl_buff;
-			need_jnl_sync = (csr->jnl_addr > jb->fsync_dskaddr);
+			need_jnl_sync = (cr->jnl_addr > jb->fsync_dskaddr);
 			assert(!need_jnl_sync || ((NOJNL) != jpc->channel) || (cnl->wcsflu_pid != process_id));
 			got_lock = FALSE;
-			if ((csr->jnl_addr > jb->dskaddr)
+			if ((cr->jnl_addr > jb->dskaddr)
 			    || (need_jnl_sync && (NOJNL == jpc->channel
 			    || (FALSE == (got_lock = GET_SWAPLOCK(&jb->fsync_in_prog_latch))))))
 			{
@@ -385,7 +388,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				if (need_jnl_sync)
 					BG_TRACE_PRO_ANY(csa, n_jnl_fsync_tries);
 				if (keep_buff_lock)
-					CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+					CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 				REINSERT_CR_AT_TAIL(csr, ahead, n, csa, csd, wcb_wtstart_lckfail3);
 				if (INTERLOCK_FAIL == n)
 				{
@@ -414,7 +417,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 							 ERR_TEXT, 2, RTS_ERROR_TEXT("Error with fsync"), errno);
 						RELEASE_SWAPLOCK(&jb->fsync_in_prog_latch);
 						if (keep_buff_lock)
-							CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+							CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 						REINSERT_CR_AT_TAIL(csr, ahead, n, csa, csd, wcb_wtstart_lckfail3);
 						if (INTERLOCK_FAIL == n)
 						{
@@ -436,7 +439,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 		/* If twin exists then do not issue write of NEWER twin until OLDER twin has been removed from WIP queue.
 		 * The act of removal from the WIP queue clears "csr->twin" so checking just that is enough.
 		 */
-		if (csr->twin)
+		if (cr->twin)
 		{
 			assert(csd->asyncio);	/* Assert that ASYNCIO is turned ON as that is a necessity for twinning */
 			/* Check if crit can be obtained right away. If so, call "wcs_wtfini" after getting crit.
@@ -448,10 +451,10 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			if (!wtfini_called_once &&
 				(was_crit || grab_crit_immediate(region, OK_FOR_WCS_RECOVER_FALSE, NOT_APPLICABLE)))
 			{
-				if (csr->twin)
+				if (cr->twin)
 				{
 					DEBUG_ONLY(dbg_wtfini_lcnt = dbg_wtfini_wcs_wtstart);	/* used by "wcs_wtfini" */
-					older_twin = (csr->bt_index ? (cache_rec_ptr_t)GDS_ANY_REL2ABS(csa, csr->twin) : cr);
+					older_twin = (cr->bt_index ? (cache_rec_ptr_t)GDS_ANY_REL2ABS(csa, cr->twin) : cr);
 					assert(!older_twin->bt_index);
 					wcs_wtfini(region, CHECK_IS_PROC_ALIVE_FALSE, older_twin);
 					wtfini_called_once = TRUE;
@@ -464,12 +467,12 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			 * the write got killed and "wcs_wtfini" moved the csr back into the active queue. csr->bt_index
 			 * being non-zero indicates it is a NEWER twin in which case we need to wait for the twin link to be broken.
 			 */
-			if (csr->twin && csr->bt_index)
+			if (cr->twin && cr->bt_index)
 			{
 				WCS_OPS_TRACE(csa, process_id, wcs_ops_wtstart6, cr->blk, GDS_ANY_ABS2REL(csa,cr), cr->dirty,	\
 					cr->bt_index, 0);
 				if (keep_buff_lock)
-					CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+					CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 				REINSERT_CR_AT_TAIL(csr, ahead, n, csa, csd, wcb_wtstart_lckfail3);
 				if (INTERLOCK_FAIL == n)
 				{
@@ -481,85 +484,50 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				continue;
 			}
 		}
-		csr->aio_issued = FALSE;	/* set this to TRUE before csr->epid is set to a non-zero value.
-						 * To avoid out-of-order execution place this BEFORE the LOCK_BUFF_FOR_WRITE.
-						 * It does not hurt in the case we skip the "if (OWN_BUFF(n))" check.
-						 */
+		if (csd->asyncio)
+			cr->aio_issued = FALSE;	/* set this to TRUE before csr->epid is set to a non-zero value.
+						* To avoid out-of-order execution place this BEFORE the LOCK_BUFF_FOR_WRITE.
+						* It does not hurt in the case we skip the "if (OWN_BUFF(n))" check.
+						*/
 		if (!keep_buff_lock)
-			LOCK_BUFF_FOR_WRITE(csr, n, &cnl->db_latch);
+			LOCK_BUFF_FOR_WRITE(cr, n, &cnl->db_latch);
 		else
 			assert(OWN_BUFF(n)); /* since we keep it we better own it */
-		assert(WRITE_LATCH_VAL(csr) >= LATCH_CLEAR);
-		assert(WRITE_LATCH_VAL(csr) <= LATCH_CONFLICT);
+		assert(WRITE_LATCH_VAL(cr) >= LATCH_CLEAR);
+		assert(WRITE_LATCH_VAL(cr) <= LATCH_CONFLICT);
 		if (OWN_BUFF(n))
 		{	/* sole owner */
-			assert(csr->dirty);
-			assert(WRITE_LATCH_VAL(csr) > LATCH_CLEAR);
+			assert(cr->dirty);
+			assert(WRITE_LATCH_VAL(cr) > LATCH_CLEAR);
 			assert(0 == n);
 			/* We're going to write this block out now */
 			save_errno = 0;
-			assert(FALSE == csr->data_invalid);	/* check that buffer has valid data */
-			csr->epid = process_id;
+			assert(FALSE == cr->data_invalid);	/* check that buffer has valid data */
+			cr->epid = process_id;
 			CR_BUFFER_CHECK1(region, csa, csd, cr, cr_lo, cr_hi);
-			bp = (blk_hdr_ptr_t)(GDS_ANY_REL2ABS(csa, csr->buffaddr));
-			VALIDATE_BM_BLK(csr->blk, bp, csa, region, bmp_status);	/* bmp_status holds bmp buffer's validity */
-			/* GDSV4 (0) version uses this field as a block length so should always be > 0. Assert that.
-			 * There is one exception in that we might have a crash test where a process was killed just before
-			 * populating a block in bg_update_phase2 when cr->data_invalid is still 0 but cr->in_tend is non-zero
-			 * (so the block-header is still null) in which case "wcs_recover" would have kept cr->dirty non-zero
-			 * even though the block-header is empty. So assert that. Note that gds_blk_downgrade has a safety
-			 * check for bver == 0 and returns immediately in that case so it is okay to call it with a 0 bver in pro.
-			 */
+			bp = (blk_hdr_ptr_t)(GDS_ANY_REL2ABS(csa, cr->buffaddr));
+			VALIDATE_BM_BLK(cr->blk, bp, csa, region, bmp_status);	/* bmp_status holds bmp buffer's validity */
 			assert((((blk_hdr_ptr_t)bp)->bver)
 			       || WBTEST_ENABLED(WBTEST_CRASH_SHUTDOWN_EXPECTED)
 			       || WBTEST_ENABLED(WBTEST_MURUNDOWN_KILLCMT06));
-			if (IS_GDS_BLK_DOWNGRADE_NEEDED(csr->ondsk_blkver))
-			{	/* Need to downgrade/reformat this block back to a previous format. */
-				assert(!csd->asyncio);	/* asyncio & V4 format are not supported together */
-				assert((0 == reformat_buffer_in_use) || process_exiting);
-				DEBUG_ONLY(reformat_buffer_in_use++;)
-				DEBUG_DYNGRD_ONLY(PRINTF("WCS_WTSTART: Block %d being dynamically downgraded on write\n", \
-							 csr->blk));
-				if (csd->blk_size > reformat_buffer_len)
-				{	/* Buffer not big enough (or does not exist)
-					 * .. get a new one releasing old if it exists
-					 */
-					assert(0 == gtmMallocDepth);	/* should not be in a nested free/malloc */
-					if (reformat_buffer)
-						free(reformat_buffer);	/* Different blksized databases in use
-									 * .. keep only largest one
-									 */
-					reformat_buffer = malloc(csd->blk_size);
-					reformat_buffer_len = csd->blk_size;
-				}
-				gds_blk_downgrade((v15_blk_hdr_ptr_t)reformat_buffer, (blk_hdr_ptr_t)bp);
-				bp = (blk_hdr_ptr_t)reformat_buffer;
-				size = (((v15_blk_hdr_ptr_t)bp)->bsiz + 1) & ~1;
-			}
-			else DEBUG_ONLY(if ((GDSV7 >= csr->ondsk_blkver) && (GDSV6 <= csr->ondsk_blkver)))
-				size = (bp->bsiz + 1) & ~1;
-#			ifdef DEBUG
-			else
-			{
-				assert((GDSV7 >= csr->ondsk_blkver) && (GDSV6 <= csr->ondsk_blkver));
-			}
-#			endif
+			/* Previously, blocks could be downgraded as needed */
+			size = (bp->bsiz + 1) & ~1;
 			if (csd->write_fullblk)
 			{	/* See similiar logic in wcs_wtstart.c */
 #			ifdef DEBUG
-				if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == csr->blk))
+				if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == cr->blk))
 				{
 					DBGFPF((stdout,"Rounding the write size: %d with %d\n", size, \
-					((FULL_DATABASE_WRITE == csd->write_fullblk && csr->needs_first_write) ? \
+					((FULL_DATABASE_WRITE == csd->write_fullblk && cr->needs_first_write) ? \
 					csd->blk_size : csa->fullblockwrite_len)));
 				}
 #			endif
 				size = (int)ROUND_UP(size,
-						(FULL_DATABASE_WRITE == csd->write_fullblk && csr->needs_first_write)
+						(FULL_DATABASE_WRITE == csd->write_fullblk && cr->needs_first_write)
 						? csd->blk_size : csa->fullblockwrite_len);
 			}
 #			ifdef DEBUG
-			else if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == csr->blk))
+			else if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == cr->blk))
 				DBGFPF((stdout, "Not rounding the write size\n"));
 #			endif
 			assert(size <= csd->blk_size);
@@ -616,6 +584,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 					 * "reorg_encrypt_restart_csa", no crit on any region etc.) all of which are not
 					 * guaranteed in some cases.
 					 */
+					assert(seg);
 					INIT_DB_OR_JNL_ENCRYPTION(csa, csd, seg->fname_len, seg->fname, gtmcrypt_errno);
 					save_errno = gtmcrypt_errno;
 					if (0 == save_errno)
@@ -645,21 +614,21 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 							GTMCRYPT_ENCRYPT(csa, csd->non_null_iv, csa->encr_key_handle, in,
 									in_len, out, bp, SIZEOF(blk_hdr), gtmcrypt_errno);
 						}
-						DBG_RECORD_BLOCK_WRITE(csd, csa, cnl, process_id, csr->blk,
+						DBG_RECORD_BLOCK_WRITE(csd, csa, cnl, process_id, cr->blk,
 							((blk_hdr *)bp)->tn,
 							4, use_new_key, bp, save_bp, bp->bsiz, in_len);
 						save_errno = gtmcrypt_errno;
 					} else
 					{
 						memcpy(save_bp, bp, bp->bsiz);
-						DBG_RECORD_BLOCK_WRITE(csd, csa, cnl, process_id, csr->blk,
+						DBG_RECORD_BLOCK_WRITE(csd, csa, cnl, process_id, cr->blk,
 							((blk_hdr *)bp)->tn,
 							5, use_new_key, bp, save_bp, bp->bsiz, in_len);
 					}
 				}
 			} else
 			{
-				DBG_RECORD_BLOCK_WRITE(csd, csa, cnl, process_id, csr->blk,
+				DBG_RECORD_BLOCK_WRITE(csd, csa, cnl, process_id, cr->blk,
 					((blk_hdr *)bp)->tn,
 					6, use_new_key, bp, save_bp, bp->bsiz, 0);
 			}
@@ -684,19 +653,19 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				 */
 #ifdef DEBUG
 				/* Going to do a write below, check the size being written for full blk writes */
-				if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == csr->blk))
+				if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == cr->blk))
 				{
-					DBGFPF((stdout, "Region : %s Blk num : %ld ",region->rname, csr->blk));
-					DBGFPF((stdout, "needs_first_write : %d ", csr->needs_first_write));
+					DBGFPF((stdout, "Region : %s Blk num : %ld ",region->rname, cr->blk));
+					DBGFPF((stdout, "needs_first_write : %d ", cr->needs_first_write));
 					DBGFPF((stdout, "size written: %ld ", size));
 					DBGFPF((stdout, "fullblkwrite_len : %ld\n", csa->fullblockwrite_len));
 				}
 #endif
-				offset = BLK_ZERO_OFF(csd->start_vbn) + (off_t)csr->blk * csd->blk_size;
+				offset = BLK_ZERO_OFF(csd->start_vbn) + (off_t)cr->blk * csd->blk_size;
 				if (!do_asyncio)
 				{
 					DB_LSEEKWRITE(csa, udi, udi->fn, udi->fd, offset, save_bp, size, save_errno);
-					csr->needs_first_write = FALSE;
+					cr->needs_first_write = FALSE;
 				} else
 				{
 					cr->wip_is_encr_buf = (save_bp != bp);
@@ -710,11 +679,11 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 							BG_TRACE_PRO_ANY(csa, wcs_wtstart_eagain_incrit);
 							DB_LSEEKWRITE(csa, udi, udi->fn, udi->fd, offset,		\
 											save_bp, size, save_errno);
-							csr->needs_first_write = FALSE;
+							cr->needs_first_write = FALSE;
 						}
 						/* else: We do not hold crit so flushing this is not critical. */
 					} else if (0 == save_errno)
-						csr->aio_issued = TRUE;
+						cr->aio_issued = TRUE;
 				}
 			}
 			if ((blk_hdr_ptr_t)reformat_buffer == bp)
@@ -729,8 +698,8 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				WCS_OPS_TRACE(csa, process_id, wcs_ops_wtstart7, cr->blk, GDS_ANY_ABS2REL(csa,cr), cr->dirty,	\
 					skip_in_trans, save_errno);
 				assert((ERR_ENOSPCQIODEFER != save_errno) || !was_crit || skip_in_trans);
-				csr->epid = 0; /* before releasing update lock, clear epid */
-				CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+				cr->epid = 0; /* before releasing update lock, clear epid */
+				CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 				REINSERT_CR_AT_TAIL(csr, ahead, n, csa, csd, wcb_wtstart_lckfail4);
 				if (INTERLOCK_FAIL == n)
 				{
@@ -754,7 +723,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				if (INTERLOCK_FAIL == n)
 				{
 					assert(FALSE);
-					csr->epid = 0;
+					cr->epid = 0;
 					SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 					BG_TRACE_PRO_ANY(csa, wcb_wtstart_lckfail4);
 					err_status = ERR_DBCCERR;
@@ -776,17 +745,17 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			queue_empty = !SUB_ENT_FROM_ACTIVE_QUE_CNT(cnl);
 			if (!do_asyncio)
 			{
-				csr->flushed_dirty_tn = csr->dirty;
-				csr->epid = 0;
+				cr->flushed_dirty_tn = cr->dirty;
+				cr->epid = 0;
 				ADD_ENT_TO_FREE_QUE_CNT(cnl);
-				csr->dirty = 0;
+				cr->dirty = 0;
 				/* Even though asyncio is ON we may have done a synchronous I/O to get it done, e.g.,
 				 * we were holding crit and got an asyncio error. If that is the case, check for
 				 * a twin.
 				 */
-				if (csd->asyncio && csr->twin)
-					BREAK_TWIN(csr, csa);
-				CLEAR_BUFF_UPDATE_LOCK(csr, &cnl->db_latch);
+				if (csd->asyncio && cr->twin)
+					BREAK_TWIN(cr, csa);
+				CLEAR_BUFF_UPDATE_LOCK(cr, &cnl->db_latch);
 				/* Note we are still under protection of wbuf_dqd lock at this point. Reason we keep
 				 * it so long is so that all the counters are updated along with the queue being correct.
 				 * The result of not doing this previously is that wcs_recover was NOT called when we
@@ -829,6 +798,7 @@ writes_completed:
 	{	/* Now that we have done all cleanup (reinserted the cache-record that failed the write and cleared cnl->in_wtstart
 		 * and cnl->intent_wtstart, go ahead and issue the error.
 		 */
+		assert(seg);
 		GTMCRYPT_REPORT_ERROR(gtmcrypt_errno, rts_error, seg->fname_len, seg->fname);
 	}
 	if (pushed_region)
