@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2022 Fidelity National Information 	*
+ * Copyright (c) 2001-2024 Fidelity National Information 	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -607,7 +607,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* KRNLKILL */
 	   0,	/* MEMORYRECURSIVE */
 	   0,	/* FREEZEID */
-	   0,	/* UNUSEDMSG778 */
+	   0,	/* FORCEDHALT2 */
 	   0,	/* DSEINVALBLKID */
 	   0,	/* PINENTRYERR */
 	   0,	/* BCKUPBUFLUSH */
@@ -669,7 +669,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* REORGINC */
 	   0,	/* ASC2EBCDICCONV */
 	   0,	/* GTMSECSHRSTART */
-	   0,	/* DBVERPERFWARN1 */
+	   0,	/* UNUSEDMSG840 */
 	   0,	/* FILEIDGBLSEC */
 	   0,	/* GBLSECNOTGDS */
 	   0,	/* BADGBLSECVER */
@@ -786,7 +786,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* MUPIPSET2SML */
 	   0,	/* DBREADBM */
 	   0,	/* DBCOMPTOOLRG */
-	   0,	/* DBVERPERFWARN2 */
+	   0,	/* UNUSEDMSG957 */
 	   0,	/* DBRBNTOOLRG */
 	   0,	/* DBRBNLBMN */
 	   0,	/* DBRBNNEG */
@@ -839,7 +839,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* NOTALLDBOPN */
 	   0,	/* MUSELFBKUP */
 	   0,	/* DBDANGER */
-	   0,	/* UNUSEDMSG1010 */
+	   0,	/* ORLBKROLLED */
 	   0,	/* TCGETATTR */
 	   0,	/* TCSETATTR */
 	   0,	/* IOWRITERR */
@@ -880,7 +880,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* FILTERBADCONV */
 	   0,	/* PRIMARYISROOT */
 	   0,	/* GVQUERYGETFAIL */
-	   0,	/* UNUSEDMSG1051 */
+	   0,	/* BSIZTOOLARGE */
 	  19,	/* MERGEDESC */
 	   0,	/* MERGEINCOMPL */
 	   0,	/* DBNAMEMISMATCH */
@@ -944,7 +944,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* JNLALIGNSZCHG */
 	   0,	/* SEFCTNEEDSFULLB */
 	   0,	/* GVFAILCORE */
-	   0,	/* UNUSEDMSG1115 */
+	   0,	/* RLNKCTLOPENDEL */
 	   0,	/* DBFRZRESETSUC */
 	   0,	/* JNLFILEXTERR */
 	   0,	/* JOBEXAMDONE */
@@ -1041,10 +1041,10 @@ const static readonly int error_ansi[] = {
 	   0,	/* DBBTUWRNG */
 	   0,	/* DBBTUFIXED */
 	   0,	/* DBMAXREC2BIG */
-	   0,	/* UNUSEDMSG1212 */
-	   0,	/* UNUSEDMSG1213 */
-	   0,	/* UNUSEDMSG1214 */
-	   0,	/* UNUSEDMSG1215 */
+	   0,	/* SOCKHANGUP */
+	   0,	/* REPLINSTRECR */
+	   0,	/* SEMUNDOOVERFLOW */
+	   0,	/* ENCODING */
 	   0,	/* DBMINRESBYTES */
 	   0,	/* UNUSEDMSG1217 */
 	   0,	/* UNUSEDMSG1218 */
@@ -1054,10 +1054,10 @@ const static readonly int error_ansi[] = {
 	   0,	/* UNUSEDMSG1222 */
 	   0,	/* UNUSEDMSG1223 */
 	   0,	/* DYNUPGRDFAIL */
-	   0,	/* MMNODYNDWNGRD */
-	   0,	/* MMNODYNUPGRD */
-	   0,	/* MUDWNGRDNRDY */
-	   0,	/* MUDWNGRDTN */
+	   0,	/* UNUSEDMSG1225 */
+	   0,	/* UNUSEDMSG1226 */
+	   0,	/* UNUSEDMSG1227 */
+	   0,	/* REORGUPCNFLCT */
 	   0,	/* MUDWNGRDNOTPOS */
 	   0,	/* MUUPGRDNRDY */
 	   0,	/* TNWARN */
@@ -1073,7 +1073,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* BKUPTMPFILWRITE */
 	   0,	/* SHMHUGETLB */
 	   0,	/* SHMLOCK */
-	   0,	/* UNUSEDMSG1244 */
+	   0,	/* SPCFCBUFDELAY */
 	   0,	/* REPLINSTMISMTCH */
 	   0,	/* REPLINSTREAD */
 	   0,	/* REPLINSTDBMATCH */
@@ -1146,7 +1146,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* CRYPTINIT */
 	   0,	/* CRYPTOPFAILED */
 	   0,	/* CRYPTDLNOOPEN */
-	   0,	/* CRYPTNOV4 */
+	   0,	/* UNUSEDMSG1317 */
 	   0,	/* CRYPTNOMM */
 	   0,	/* READONLYNOBG */
 	   0,	/* CRYPTKEYFETCHFAILED */
@@ -1161,8 +1161,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* LOCKSPACEFULL */
 	   0,	/* IOERROR */
 	   0,	/* MAXSSREACHED */
-	   0,	/* SNAPSHOTNOV4 */
-	   0,	/* SSV4NOALLOW */
+	   0,	/* UNUSEDMSG1332 */
+	   0,	/* DBUPGRDREQ */
 	   0,	/* SSTMPDIRSTAT */
 	   0,	/* SSTMPCREATE */
 	   0,	/* JNLFILEDUP */
@@ -1267,7 +1267,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* ORLBKNOSTP */
 	   0,	/* ORLBKFRZPROG */
 	   0,	/* ORLBKFRZOVER */
-	   0,	/* ORLBKNOV4BLK */
+	   0,	/* ORLBKDBUPGRDREQ */
 	   0,	/* DBROLLEDBACK */
 	   0,	/* DSEWCREINIT */
 	   0,	/* MURNDWNOVRD */
@@ -1282,7 +1282,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* MUTRUNCFAIL */
 	   0,	/* MUTRUNCNOSPACE */
 	   0,	/* MUTRUNCNOTBG */
-	   0,	/* MUTRUNCNOV4 */
+	   0,	/* PIDMISMATCH */
 	   0,	/* MUTRUNCPERCENT */
 	   0,	/* MUTRUNCSSINPROG */
 	   0,	/* MUTRUNCSUCCESS */
@@ -1352,7 +1352,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* NULLENTRYREF */
 	   0,	/* ZPEEKNORPLINFO */
 	   0,	/* MMREGNOACCESS */
-	   0,	/* UNUSEDMSG1525 */
+	   0,	/* FDSIZELMT */
 	   0,	/* MALLOCCRIT */
 	   0,	/* HOSTCONFLICT */
 	   0,	/* GETADDRINFO */
@@ -1447,13 +1447,13 @@ const static readonly int error_ansi[] = {
 	   0,	/* EXTRINTEGRITY */
 	   0,	/* CRYPTKEYRELEASEFAILED */
 	   0,	/* MUREENCRYPTSTART */
-	   0,	/* MUREENCRYPTV4NOALLOW */
+	   0,	/* BUFFSIZETOOSMALL */
 	   0,	/* ENCRYPTCONFLT */
 	   0,	/* JNLPOOLRECOVERY */
 	   0,	/* LOCKTIMINGINTP */
 	   0,	/* PBNUNSUPTYPE */
 	   0,	/* DBFHEADLRU */
-	   0,	/* ASYNCIONOV4 */
+	   0,	/* UNUSEDMSG1626 */
 	   0,	/* AIOCANCELTIMEOUT */
 	   0,	/* DBGLDMISMATCH */
 	   0,	/* DBBLKSIZEALIGN */
@@ -1561,4 +1561,6 @@ const static readonly int error_ansi[] = {
 	   0,	/* AUDCONNFAIL */
 	   0,	/* AUDLOGFAIL */
 	  80,	/* SOCKCLOSE */
+	   0,	/* LINETOOLONG */
+	   0,	/* ARGTRUNC */
 	};
